@@ -15,10 +15,10 @@
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 */
-
 #include "katesmartmanager.h"
 
 #include "katedocument.h"
+#include "kateedit.h"
 #include "katesmartcursor.h"
 #include "katesmartrange.h"
 
@@ -27,17 +27,136 @@
 
 #include <kdebug.h>
 
+//Uncomment this to debug the translation of ranges. If that is enabled,
+//all ranges are first translated completely separately out of the translation process,
+//and at the end the result is compared. If the result mismatches, an assertion is triggered.
+// #define DEBUG_TRANSLATION
+
 static const int s_defaultGroupSize = 40;
 static const int s_minimumGroupSize = 20;
 static const int s_maximumGroupSize = 60;
 
 using namespace KTextEditor;
 
+static void translate(KateEditInfo* edit, Cursor& ret, SmartCursor::InsertBehavior insertBehavior)
+{
+  // NOTE: copied from KateSmartCursor::translate()
+  // If this cursor is before the edit, no action is required
+  if (ret < edit->start())
+    return;
+
+  // If this cursor is on a line affected by the edit
+  if (edit->oldRange().overlapsLine(ret.line())) {
+    // If this cursor is at the start of the edit
+    if (ret == edit->start()) {
+      // And it doesn't need to move, no action is required
+      if (insertBehavior == SmartCursor::StayOnInsert)
+        return;
+    }
+
+    // Calculate the new position
+    Cursor newPos;
+    if (edit->oldRange().contains(ret)) {
+      if (insertBehavior == SmartCursor::MoveOnInsert)
+        ret = edit->newRange().end();
+      else
+        ret = edit->start();
+
+    } else {
+      ret += edit->translate();
+    }
+
+    return;
+  }
+
+  // just need to adjust line number
+  ret.setLine(ret.line() + edit->translate().line());
+}
+
+#ifdef DEBUG_TRANSLATION
+struct KateSmartManager::KateTranslationDebugger {
+  KateTranslationDebugger(KateSmartManager* manager, KateEditInfo* edit) : m_manager(manager), m_edit(edit) {
+    manager->m_currentKateTranslationDebugger = this;
+    foreach(KateSmartRange* range, manager->m_topRanges)
+      addRange(range);
+  }
+
+  ~KateTranslationDebugger() {
+    m_manager->m_currentKateTranslationDebugger = 0;
+  }
+
+  void addRange(SmartRange* _range) {
+
+    KateSmartRange* range = dynamic_cast<KateSmartRange*>(_range);
+    Q_ASSERT(range);
+
+    RangeTranslation translation;
+    translation.from = *range;
+    KTextEditor::Cursor toStart = range->start();
+    KTextEditor::Cursor toEnd = range->end();
+
+    translate(m_edit, toStart, (range->insertBehavior() & SmartRange::ExpandLeft) ? SmartCursor::StayOnInsert : SmartCursor::MoveOnInsert);
+    translate(m_edit, toEnd, (range->insertBehavior() & SmartRange::ExpandRight) ? SmartCursor::MoveOnInsert : SmartCursor::StayOnInsert);
+    translation.to = KTextEditor::Range(toStart, toEnd);
+
+    m_rangeTranslations[range] = translation;
+    m_cursorTranslations[&range->smartStart()] = CursorTranslation(range->start(), toStart);
+    m_cursorTranslations[&range->smartEnd()] = CursorTranslation(range->end(), toEnd);
+
+    foreach(SmartRange* child, range->childRanges())
+      addRange(child);
+  }
+
+  void verifyAll() {
+    for(QMap<const SmartRange*, RangeTranslation>::iterator it = m_rangeTranslations.begin(); it != m_rangeTranslations.end(); ++it) {
+      if(*it.key() != it.value().to) {
+    kDebug() << "mismatch. Translation should be:" << it.value().to << "translation is:" << *it.key() << "from:" << it.value().from;
+      kDebug() << "edit:" << m_edit->oldRange() << m_edit->newRange();
+    Q_ASSERT(0);
+      }
+    }
+  }
+
+  void verifyChange(SmartCursor* _cursor) {
+    if(!m_cursorTranslations.contains(_cursor))
+      return;
+    return;
+    KateSmartCursor* cursor = dynamic_cast<KateSmartCursor*>(_cursor);
+    Q_ASSERT(cursor);
+    KTextEditor::Cursor realPos( cursor->m_line + cursor->m_smartGroup->m_newStartLine, cursor->m_column);
+    kDebug() << "changing cursor from" << m_cursorTranslations[cursor].from << "to" << realPos;
+
+    if(m_cursorTranslations[cursor].to != realPos) {
+      kDebug() << "mismatch. Translation of cursor should be:" << m_cursorTranslations[cursor].to << "is:" << realPos << "from:" << m_cursorTranslations[cursor].from;
+      kDebug() << "edit:" << m_edit->oldRange() << m_edit->newRange();
+      Q_ASSERT(0);
+    }
+  }
+
+  struct RangeTranslation {
+    KTextEditor::Range from, to;
+  };
+
+  struct CursorTranslation {
+    CursorTranslation() {
+    }
+    CursorTranslation(const KTextEditor::Cursor& _from, const KTextEditor::Cursor& _to) : from(_from), to(_to) {
+    }
+    KTextEditor::Cursor from, to;
+  };
+
+  QMap<const SmartRange*, RangeTranslation> m_rangeTranslations;
+  QMap<const SmartCursor*, CursorTranslation> m_cursorTranslations;
+  KateSmartManager* m_manager;
+  KateEditInfo* m_edit;
+};
+#endif
 KateSmartManager::KateSmartManager(KateDocument* parent)
   : QObject(parent)
   , m_firstGroup(new KateSmartGroup(0, 0, 0L, 0L))
   , m_invalidGroup(new KateSmartGroup(-1, -1, 0L, 0L))
   , m_clearing(false)
+  , m_currentKateTranslationDebugger(0)
 {
   connect(doc()->history(), SIGNAL(editDone(KateEditInfo*)), SLOT(slotTextChanged(KateEditInfo*)));
   //connect(doc(), SIGNAL(textChanged(Document*)), SLOT(verifyCorrect()));
@@ -64,8 +183,10 @@ KateDocument * KateSmartManager::doc( ) const
 
 KateSmartCursor * KateSmartManager::newSmartCursor( const Cursor & position, SmartCursor::InsertBehavior insertBehavior, bool internal )
 {
+  QMutexLocker l(internal ? doc()->smartMutex() : 0);
+
   KateSmartCursor* c;
-  if (usingRevision() != -1)
+  if (usingRevision() != -1 && !internal)
     c = new KateSmartCursor(translateFromRevision(position), doc(), insertBehavior);
   else
     c = new KateSmartCursor(position, doc(), insertBehavior);
@@ -77,9 +198,11 @@ KateSmartCursor * KateSmartManager::newSmartCursor( const Cursor & position, Sma
 
 KateSmartRange * KateSmartManager::newSmartRange( const Range & range, SmartRange * parent, SmartRange::InsertBehaviors insertBehavior, bool internal )
 {
+  QMutexLocker l(internal ? doc()->smartMutex() : 0);
+
   KateSmartRange* newRange;
 
-  if (usingRevision() != -1)
+  if (usingRevision() != -1 && !internal)
     newRange = new KateSmartRange(translateFromRevision(range), doc(), parent, insertBehavior);
   else
     newRange = new KateSmartRange(range, doc(), parent, insertBehavior);
@@ -93,10 +216,15 @@ KateSmartRange * KateSmartManager::newSmartRange( const Range & range, SmartRang
 
 KateSmartRange * KateSmartManager::newSmartRange( KateSmartCursor * start, KateSmartCursor * end, SmartRange * parent, SmartRange::InsertBehaviors insertBehavior, bool internal )
 {
-  if (usingRevision() != -1) {
-    *start = translateFromRevision(*start, (insertBehavior & SmartRange::ExpandLeft) ? SmartCursor::StayOnInsert : SmartCursor::MoveOnInsert);
-    *end = translateFromRevision(*end, (insertBehavior & SmartRange::ExpandRight) ? SmartCursor::MoveOnInsert : SmartCursor::StayOnInsert);
-  }
+  QMutexLocker l(internal ? doc()->smartMutex() : 0);
+
+//Why translate "smart" cursors? They should translate automatically!
+//   if (usingRevision() != -1 && !internal) {
+//     KTextEditor::Cursor tempStart = translateFromRevision(*start, (insertBehavior & SmartRange::ExpandLeft) ? SmartCursor::StayOnInsert : SmartCursor::MoveOnInsert);
+//     KTextEditor::Cursor tempEnd = translateFromRevision(*end, (insertBehavior & SmartRange::ExpandRight) ? SmartCursor::MoveOnInsert : SmartCursor::StayOnInsert);
+//     *start = tempStart;
+//     *end = tempEnd;
+//   }
 
   KateSmartRange* newRange = new KateSmartRange(start, end, parent, insertBehavior);
   if (internal)
@@ -180,7 +308,9 @@ KateSmartGroup * KateSmartManager::groupForLine( int line ) const
 void KateSmartManager::slotTextChanged(KateEditInfo* edit)
 {
   QMutexLocker lock(doc()->smartMutex());
-
+#ifdef DEBUG_TRANSLATION
+  KateTranslationDebugger KateTranslationDebugger(this, edit);
+#endif
   KateSmartGroup* firstSmartGroup = groupForLine(edit->oldRange().start().line());
   KateSmartGroup* currentGroup = firstSmartGroup;
 
@@ -243,6 +373,7 @@ void KateSmartManager::slotTextChanged(KateEditInfo* edit)
   for (KateSmartGroup* smartGroup = firstSmartGroup; smartGroup; smartGroup = smartGroup->next()) {
     if (groupChanged) {
       groupChanged = smartGroup->startLine() <= edit->oldRange().end().line();
+      // Don't continue iterating if no line translation occurred.
       if (!groupChanged && !edit->translate().line())
         break;
     }
@@ -253,6 +384,18 @@ void KateSmartManager::slotTextChanged(KateEditInfo* edit)
       smartGroup->translatedShifted(*edit);
   }
 
+  // Allow rebuilding the child-structure of affected ranges
+  for (KateSmartGroup* smartGroup = firstSmartGroup; smartGroup; smartGroup = smartGroup->next()) {
+    if (smartGroup->startLine() > edit->oldRange().end().line())
+      break;
+
+    smartGroup->translatedChanged2(*edit);
+  }
+
+#ifdef DEBUG_TRANSLATION
+  KateTranslationDebugger.verifyAll();
+#endif
+
   // Range feedback
   foreach (KateSmartRange* range, m_topRanges) {
     KateSmartRange* mostSpecific = feedbackRange(*edit, range);
@@ -262,17 +405,24 @@ void KateSmartManager::slotTextChanged(KateEditInfo* edit)
     range->feedbackMostSpecific(mostSpecific);
   }
 
+#ifdef DEBUG_TRANSLATION
+  KateTranslationDebugger.verifyAll();
+#endif
   //debugOutput();
-  //verifyCorrect();
+//   verifyCorrect();
 }
 
 KateSmartRange* KateSmartManager::feedbackRange( const KateEditInfo& edit, KateSmartRange * range )
 {
   KateSmartRange* mostSpecific = 0L;
 
-  // This range preceeds the edit... no more to do
-  if (range->end() < edit.start())
+  // This range precedes the edit... no more to do
+  if ((range->end() < edit.start()  && range->kEnd().lastPosition() < edit.start()) ||
+    (range->end() == edit.start() && range->kEnd().lastPosition() == edit.start() && !range->isEmpty())
+  ) {
+    //kDebug() << "Not feeding back to " << *range << "as edit start" << edit.start();
     return mostSpecific;
+  }
 
   foreach (SmartRange* child, range->childRanges())
     if (!mostSpecific)
@@ -280,12 +430,18 @@ KateSmartRange* KateSmartManager::feedbackRange( const KateEditInfo& edit, KateS
     else
       feedbackRange(edit, static_cast<KateSmartRange*>(child));
 
-  if (range->start() > edit.oldRange().end()) {
+  //kDebug() << "edit" << edit.oldRange() << edit.newRange() << "last at" << range->kStart().lastPosition() << range->kEnd().lastPosition() << "now" << *range;
+
+  if (range->start() > edit.newRange().end() ||
+      (range->start() == edit.newRange().end() && range->kStart().lastPosition() == edit.oldRange().end()))
+  {
     // This range is after the edit... has only been shifted
+    //kDebug() << "Feeding back shifted to " << *range;
     range->shifted();
 
   } else {
     // This range is within the edit.
+    //kDebug() << "Feeding back translated to " << *range;
     if (!mostSpecific)
       if (range->start() < edit.oldRange().start() && range->end() > edit.oldRange().end())
         mostSpecific = range;
@@ -312,6 +468,7 @@ void KateSmartGroup::translateShifted(const KateEditInfo& edit)
 {
   m_newStartLine = m_startLine + edit.translate().line();
   m_newEndLine = m_endLine + edit.translate().line();
+  //Since shifting only does not change the overlap or order, we don't need to rebuild any child structures here
 }
 
 void KateSmartGroup::translatedChanged(const KateEditInfo& edit)
@@ -323,6 +480,25 @@ void KateSmartGroup::translatedChanged(const KateEditInfo& edit)
     cursor->translated(edit);
 }
 
+void KateSmartGroup::translatedChanged2(const KateEditInfo& edit)
+{
+  //Tell the affected parent smart-ranges to rebuild their child-structure, so they stay consistent
+  QSet<KTextEditor::SmartRange*> rebuilt;
+
+  foreach (KateSmartCursor* cursor, m_normalCursors + m_feedbackCursors) {
+    KTextEditor::SmartRange* range = cursor->smartRange();
+    if(range) {
+        KTextEditor::SmartRange* parent = range->parentRange();
+        if(parent && !rebuilt.contains(parent)) {
+            rebuilt.insert(parent);
+            KateSmartRange* kateSmart = dynamic_cast<KateSmartRange*>(parent);
+            Q_ASSERT(kateSmart);
+            kateSmart->rebuildChildStructure();
+        }
+    }
+  }
+}
+
 void KateSmartGroup::translatedShifted(const KateEditInfo& edit)
 {
   if (m_startLine != m_newStartLine) {
@@ -519,18 +695,17 @@ void KateSmartManager::clear( bool includingInternal )
 
 void KateSmartManager::useRevision(int revision)
 {
-  if (revision == -1)
-    // Clear current revision use
-    m_usingRevision.remove(QThread::currentThread());
-  else
-    m_usingRevision[QThread::currentThread()] = revision;
+  if (!m_usingRevision.hasLocalData())
+    m_usingRevision.setLocalData(new int);
+
+  *m_usingRevision.localData() = revision;
 }
 
 int KateSmartManager::usingRevision() const
 {
-  if (m_usingRevision.contains(QThread::currentThread())) {
-    return m_usingRevision[QThread::currentThread()];
-  }
+  if (m_usingRevision.hasLocalData())
+    return *m_usingRevision.localData();
+
   return -1;
 }
 
@@ -544,41 +719,6 @@ int KateSmartManager::currentRevision() const
   return doc()->history()->revision();
 }
 
-static void translate(KateEditInfo* edit, Cursor& ret, SmartCursor::InsertBehavior insertBehavior)
-{
-  // NOTE: copied from KateSmartCursor::translate()
-  // If this cursor is before the edit, no action is required
-  if (ret < edit->start())
-    return;
-
-  // If this cursor is on a line affected by the edit
-  if (edit->oldRange().overlapsLine(ret.line())) {
-    // If this cursor is at the start of the edit
-    if (ret == edit->start()) {
-      // And it doesn't need to move, no action is required
-      if (insertBehavior == SmartCursor::StayOnInsert)
-        return;
-    }
-
-    // Calculate the new position
-    Cursor newPos;
-    if (edit->oldRange().contains(ret)) {
-      if (insertBehavior == SmartCursor::MoveOnInsert)
-        ret = edit->newRange().end();
-      else
-        ret = edit->start();
-
-    } else {
-      ret += edit->translate();
-    }
-
-    return;
-  }
-
-  // just need to adjust line number
-  ret.setLine(ret.line() + edit->translate().line());
-}
-
 Cursor KateSmartManager::translateFromRevision(const Cursor& cursor, SmartCursor::InsertBehavior insertBehavior) const
 {
   Cursor ret = cursor;
@@ -591,14 +731,14 @@ Cursor KateSmartManager::translateFromRevision(const Cursor& cursor, SmartCursor
 
 Range KateSmartManager::translateFromRevision(const Range& range, KTextEditor::SmartRange::InsertBehaviors insertBehavior) const
 {
-  Range ret = range;
+  Cursor start = range.start(), end = range.end();
 
   foreach (KateEditInfo* edit, doc()->history()->editsBetweenRevisions(usingRevision())) {
-    translate(edit, ret.start(), insertBehavior & KTextEditor::SmartRange::ExpandLeft ? SmartCursor::StayOnInsert : SmartCursor::MoveOnInsert);
-    translate(edit, ret.end(), insertBehavior & KTextEditor::SmartRange::ExpandRight ? SmartCursor::MoveOnInsert : SmartCursor::StayOnInsert);
+    translate(edit, start, insertBehavior & KTextEditor::SmartRange::ExpandLeft ? SmartCursor::StayOnInsert : SmartCursor::MoveOnInsert);
+    translate(edit, end, insertBehavior & KTextEditor::SmartRange::ExpandRight ? SmartCursor::MoveOnInsert : SmartCursor::StayOnInsert);
   }
 
-  return ret;
+  return Range(start, end);
 }
 
 #include "katesmartmanager.moc"
