@@ -123,7 +123,7 @@ QString KateAutoIndent::tabString (int length, int align) const
   return s;
 }
 
-bool KateAutoIndent::doIndent(KateView *view, int line, int indentDepth, int align)
+bool KateAutoIndent::doIndent(int line, int indentDepth, int align)
 {
   kDebug (13060) << "doIndent: line: " << line << " indentDepth: " << indentDepth << " align: " << align;
 
@@ -137,15 +137,34 @@ bool KateAutoIndent::doIndent(KateView *view, int line, int indentDepth, int ali
   if (indentDepth < 0)
     indentDepth = 0;
 
-  QString indentString = tabString (indentDepth, align);
-
   int first_char = textline->firstChar();
-
   if (first_char < 0)
     first_char = textline->length();
 
+  // Preserve existing "tabs then spaces" alignment if and only if:
+  //  - no alignment was passed to doIndent and
+  //  - we aren't using spaces for indentation and
+  //  - we aren't rounding indentation up to the next multiple of the indentation width and
+  //  - we aren't using a combination to tabs and spaces for alignment, or in other words
+  //    the indent width is a multiple of the tab width.
+  bool preserveAlignment = !useSpaces && keepExtra && indentWidth % tabWidth == 0;
+  if (align == 0 && preserveAlignment)
+  {
+    // Count the number of consecutive spaces at the end of the existing indentation
+    QString oldIndentation = textline->string(0, first_char);
+    int i = oldIndentation.size() - 1;
+    while (i >= 0 && oldIndentation.at(i) == ' ')
+      --i;
+    // Use the passed indentDepth as the alignment, and set the indentDepth to
+    // that value minus the number of spaces found (but don't let it get negative).
+    align = indentDepth;
+    indentDepth = qMax(0, align - (oldIndentation.size() - 1 - i));
+  }
+
+  QString indentString = tabString(indentDepth, align);
+
   // remove leading whitespace, then insert the leading indentation
-  doc->editStart (view);
+  doc->editStart ();
   doc->editRemoveText (line, 0, first_char);
   doc->editInsertText (line, 0, indentString);
   doc->editEnd ();
@@ -153,7 +172,7 @@ bool KateAutoIndent::doIndent(KateView *view, int line, int indentDepth, int ali
   return true;
 }
 
-bool KateAutoIndent::doIndentRelative(KateView *view, int line, int change)
+bool KateAutoIndent::doIndentRelative(int line, int change)
 {
   kDebug (13060) << "doIndentRelative: line: " << line << " change: " << change;
 
@@ -176,10 +195,10 @@ bool KateAutoIndent::doIndentRelative(KateView *view, int line, int change)
   }
 
   // do indent
-  return doIndent(view, line, indentDepth);
+  return doIndent(line, indentDepth);
 }
 
-void KateAutoIndent::keepIndent ( KateView *view, int line )
+void KateAutoIndent::keepIndent ( int line )
 {
   // no line in front, no work...
   if (line <= 0)
@@ -191,7 +210,7 @@ void KateAutoIndent::keepIndent ( KateView *view, int line )
   if (!textline)
     return;
 
-  doIndent (view, line, textline->indentDepth (tabWidth));
+  doIndent (line, textline->indentDepth (tabWidth));
 }
 
 void KateAutoIndent::scriptIndent (KateView *view, const KTextEditor::Cursor &position, QChar typedChar)
@@ -207,7 +226,7 @@ void KateAutoIndent::scriptIndent (KateView *view, const KTextEditor::Cursor &po
   if (newIndentInChars == -1)
   {
     // keep indent of previous line
-    keepIndent (view, position.line());
+    keepIndent (position.line());
 
     return;
   }
@@ -217,7 +236,7 @@ void KateAutoIndent::scriptIndent (KateView *view, const KTextEditor::Cursor &po
     kDebug (13060) << "Align: " << align;
 
   // we got a positive or zero indent to use...
-  doIndent (view, position.line(), newIndentInChars, align);
+  doIndent (position.line(), newIndentInChars, align);
 }
 
 bool KateAutoIndent::isStyleProvided(KateIndentScript *script)
@@ -300,7 +319,7 @@ void KateAutoIndent::updateConfig ()
 }
 
 
-bool KateAutoIndent::changeIndent (KateView *view, const KTextEditor::Range &range, int change)
+bool KateAutoIndent::changeIndent (const KTextEditor::Range &range, int change)
 {
   QList<int> skippedLines;
 
@@ -321,14 +340,14 @@ bool KateAutoIndent::changeIndent (KateView *view, const KTextEditor::Range &ran
       continue;
     }
 
-    doIndentRelative(view, line, change * indentWidth);
+    doIndentRelative(line, change * indentWidth);
   }
 
   if (skippedLines.count() > range.numberOfLines())
   {
     // all lines were empty, so indent them nevertheless
     foreach (int line, skippedLines)
-      doIndentRelative(view, line, change * indentWidth);
+      doIndentRelative(line, change * indentWidth);
   }
 
   return true;
@@ -363,7 +382,7 @@ void KateAutoIndent::userTypedChar (KateView *view, const KTextEditor::Cursor &p
       return;
 
     // keep indent of previous line
-    keepIndent (view, position.line());
+    keepIndent (position.line());
 
     return;
   }
@@ -386,7 +405,7 @@ KateViewIndentationAction::KateViewIndentationAction(KateDocument *_doc, const Q
        : KActionMenu (text, parent), doc(_doc)
 {
   connect(menu(),SIGNAL(aboutToShow()),this,SLOT(slotAboutToShow()));
-
+  actionGroup = new QActionGroup(menu());
 }
 
 void KateViewIndentationAction::slotAboutToShow()
@@ -394,8 +413,12 @@ void KateViewIndentationAction::slotAboutToShow()
   QStringList modes = KateAutoIndent::listModes ();
 
   menu()->clear ();
+  foreach (QAction *action, actionGroup->actions()) {
+    actionGroup->removeAction(action);
+  }
   for (int z=0; z<modes.size(); ++z) {
     QAction *action = menu()->addAction( '&' + KateAutoIndent::modeDescription(z).replace('&', "&&") );
+    actionGroup->addAction(action);
     action->setCheckable( true );
     action->setData( z );
 
