@@ -26,38 +26,52 @@
 #include "katemodemanager.h"
 #include "kateschema.h"
 #include "kateconfig.h"
-#include "katejscript.h"
+#include "katescriptmanager.h"
 #include "katecmd.h"
 #include "katebuffer.h"
 #include "katepartpluginmanager.h"
+#include "kateviglobal.h"
+#include "katewordcompletion.h"
 
 #include <klocale.h>
 #include <kservicetypetrader.h>
 #include <kdirwatch.h>
 #include <kdebug.h>
+#include <kdeversion.h>
 #include <kpagedialog.h>
 #include <kpagewidgetmodel.h>
 #include <kiconloader.h>
 
+#include <QtCore/QPointer>
+
 #include <QtGui/QBoxLayout>
 
 KateGlobal *KateGlobal::s_self = 0;
 
 int KateGlobal::s_ref = 0;
 
+QString KateGlobal::katePartVersion()
+{
+  return QString("%1.%2").arg(KDE::versionMajor() - 1).arg(KDE::versionMinor());
+}
+
 KateGlobal::KateGlobal ()
  : KTextEditor::Editor (0)
- , m_aboutData ("katepart", 0, ki18n("Kate Part"), KATEPART_VERSION,
+ , m_aboutData ("katepart", 0, ki18n("Kate Part"), katePartVersion().toLatin1(),
              ki18n( "Embeddable editor component" ), KAboutData::License_LGPL_V2,
-             ki18n( "(c) 2000-2007 The Kate Authors" ), KLocalizedString(), "http://www.kate-editor.org")
+             ki18n( "(c) 2000-2008 The Kate Authors" ), KLocalizedString(), "http://www.kate-editor.org")
  , m_componentData (&m_aboutData)
 {
   // set s_self
   s_self = this;
 
+  // load the kate part translation catalog
+  KGlobal::locale()->insertCatalog("katepart4");
+
   //
   // fill about data
   //
+  m_aboutData.setProgramIconName("preferences-plugin");
   m_aboutData.addAuthor (ki18n("Christoph Cullmann"), ki18n("Maintainer"), "cullmann@kde.org", "http://www.babylon2k.de");
   m_aboutData.addAuthor (ki18n("Anders Lund"), ki18n("Core Developer"), "anders@alweb.dk", "http://www.alweb.dk");
   m_aboutData.addAuthor (ki18n("Joseph Wenninger"), ki18n("Core Developer"), "jowenn@kde.org","http://stud3.tuwien.ac.at/~e9925371");
@@ -79,6 +93,7 @@ KateGlobal::KateGlobal ()
   m_aboutData.addAuthor (ki18n("Mirko Stocker"), ki18n("Various bugfixes"), "me@misto.ch", "http://misto.ch/");
   m_aboutData.addAuthor (ki18n("Matthew Woehlke"), ki18n("Selection, KColorScheme integration"), "mw_triad@users.sourceforge.net");
   m_aboutData.addAuthor (ki18n("Sebastian Pipping"), ki18n("Search bar back- and front-end"), "webmaster@hartwork.org", "http://www.hartwork.org/");
+  m_aboutData.addAuthor (ki18n("Erlend Hamberg"), ki18n("Vi Input Mode"), "ehamberg@gmail.com", "http://hamberg.no/erlend");
 
   m_aboutData.addCredit (ki18n("Matteo Merli"), ki18n("Highlighting for RPM Spec-Files, Perl, Diff and more"), "merlim@libero.it");
   m_aboutData.addCredit (ki18n("Rocky Scaletta"), ki18n("Highlighting for VHDL"), "rocky@purdue.edu");
@@ -123,14 +138,19 @@ KateGlobal::KateGlobal ()
   //
   m_schemaManager = new KateSchemaManager ();
 
+  //
+  // vi input mode global
+  //
+  m_viInputModeGlobal = new KateViGlobal ();
+
   // config objects
   m_documentConfig = new KateDocumentConfig ();
   m_viewConfig = new KateViewConfig ();
   m_rendererConfig = new KateRendererConfig ();
 
-  // create script man (search scripts) + register commands
-  m_jscriptManager = new KateJScriptManager ();
-  KateCmd::self()->registerCommand (m_jscriptManager);
+  // create script manager (search scripts) + register commands
+  m_scriptManager = new KateScriptManager ();
+  KateCmd::self()->registerCommand (m_scriptManager);
 
   //
   // plugin manager
@@ -141,12 +161,16 @@ KateGlobal::KateGlobal ()
   // init the cmds
   //
   m_cmds.push_back (new KateCommands::CoreCommands());
+  m_cmds.push_back (new KateCommands::ViCommands());
   m_cmds.push_back (new KateCommands::SedReplace ());
   m_cmds.push_back (new KateCommands::Character ());
   m_cmds.push_back (new KateCommands::Date ());
 
   for ( QList<KTextEditor::Command *>::iterator it = m_cmds.begin(); it != m_cmds.end(); ++it )
     m_cmdManager->registerCommand (*it);
+
+  // global word completion model
+  m_wordCompletionModel = new KateWordCompletionModel (this);
 }
 
 KateGlobal::~KateGlobal()
@@ -166,10 +190,15 @@ KateGlobal::~KateGlobal()
   qDeleteAll (m_cmds);
 
   // cu managers
-  delete m_jscriptManager;
+  delete m_scriptManager;
   delete m_hlManager;
   delete m_cmdManager;
 
+  delete m_viInputModeGlobal;
+  
+  // cu model
+  delete m_wordCompletionModel;
+
   s_self = 0;
 }
 
@@ -225,7 +254,7 @@ bool KateGlobal::configDialogSupported () const
 
 void KateGlobal::configDialog(QWidget *parent)
 {
-  KPageDialog *kd = new KPageDialog(parent);
+  QPointer<KPageDialog> kd = new KPageDialog(parent);
   kd->setCaption( i18n("Configure") );
   kd->setButtons( KDialog::Ok | KDialog::Cancel | KDialog::Help );
   kd->setFaceType( KPageDialog::List );
@@ -244,14 +273,13 @@ void KateGlobal::configDialog(QWidget *parent)
 
     QVBoxLayout *topLayout = new QVBoxLayout( page );
     topLayout->setMargin( 0 );
-    topLayout->setSpacing( KDialog::spacingHint() );
 
     KTextEditor::ConfigPage *cp = configPage(i, page);
     topLayout->addWidget( cp);
     editorPages.append (cp);
   }
 
-  if (kd->exec())
+  if (kd->exec() && kd)
   {
     KateDocumentConfig::global()->configStart ();
     KateViewConfig::global()->configStart ();
@@ -274,7 +302,7 @@ void KateGlobal::configDialog(QWidget *parent)
 
 int KateGlobal::configPages () const
 {
-  return 6;
+  return 5;
 }
 
 KTextEditor::ConfigPage *KateGlobal::configPage (int number, QWidget *parent)
@@ -294,9 +322,6 @@ KTextEditor::ConfigPage *KateGlobal::configPage (int number, QWidget *parent)
       return new KateSaveConfigTab (parent);
 
     case 4:
-      return new KateEditKeyConfiguration (parent);
-
-    case 5:
       return new KatePartPluginConfigPage (parent);
 
     default:
@@ -323,9 +348,6 @@ QString KateGlobal::configPageName (int number) const
       return i18n("Open/Save");
 
     case 4:
-      return i18n ("Shortcuts");
-
-    case 5:
       return i18n ("Extensions");
 
     default:
@@ -352,9 +374,6 @@ QString KateGlobal::configPageFullName (int number) const
       return i18n("File Opening & Saving");
 
     case 4:
-      return i18n ("Shortcuts Configuration");
-
-    case 5:
       return i18n ("Extensions Manager");
 
     default:
@@ -381,10 +400,7 @@ KIcon KateGlobal::configPageIcon (int number) const
       return KIcon("document-save");
 
     case 4:
-      return KIcon("configure-shortcuts");
-
-    case 5:
-      return KIcon("network-connect"); // bad icon, please use "preferences-plugin" once available
+      return KIcon("preferences-plugin");
 
     default:
       return KIcon("document-properties");
@@ -446,4 +462,13 @@ QStringList KateGlobal::commandList() const
 //END command interface
 
 
+//BEGIN container interface
+QObject * KateGlobal::container()
+{return m_container.data();}
+
+void KateGlobal::setContainer( QObject * container )
+{m_container=container;}
+//END container interface
+
+
 // kate: space-indent on; indent-width 2; replace-tabs on;
