@@ -38,7 +38,7 @@
   class KConfigSkeletonItemPrivate;
   /**
    * \class KConfigSkeletonItem kcoreconfigskeleton.h <KConfigSkeletonItem>
-   * 
+   *
    * @short Class for storing a preferences setting
    * @author Cornelius Schumacher
    * @see KCoreConfigSkeleton
@@ -109,6 +109,18 @@
     */
     QString label() const;
 
+    /**
+      Set ToolTip description of item.
+      @since 4.2
+    */
+    void setToolTip( const QString &t );
+
+    /**
+      Return ToolTip description of item. See setToolTip().
+      @since 4.2
+    */
+    QString toolTip() const;
+
     /**
       Set WhatsThis description of item.
     */
@@ -142,15 +154,15 @@
     virtual void setProperty(const QVariant &p) = 0;
 
     /**
-     * Check whether the item is equal to v.
+     * Check whether the item is equal to p.
      *
      * Use this function to compare items that use custom types such as KUrl,
      * because QVariant::operator== will not work for those.
      *
-     * @param v QVariant to compare to
-     * @return true if the item is equal to v, false otherwise
+     * @param p QVariant to compare to
+     * @return true if the item is equal to p, false otherwise
      */
-    virtual bool isEqual(const QVariant &v) const = 0;
+    virtual bool isEqual(const QVariant &p) const = 0;
 
     /**
      * Return item as property
@@ -294,7 +306,7 @@ template < typename T > class KConfigSkeletonGenericItem:public KConfigSkeletonI
 
   /**
    * \class KCoreConfigSkeleton kcoreconfigskeleton.h <KCoreConfigSkeleton>
-   * 
+   *
    * @short Class for handling preferences settings for an application.
    * @author Cornelius Schumacher
    * @see KConfigSkeletonItem
@@ -394,10 +406,10 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
-    /** @copydoc KConfigSkeletonItem::property() */
+    /** @copydoc KConfigSkeletonItem::property() const */
     QVariant property() const;
 
   private:
@@ -450,10 +462,10 @@ public:
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
         void setProperty(const QVariant & p);
 
-        /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+        /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
 	bool isEqual(const QVariant &p) const;
 
-        /** @copydoc KConfigSkeletonItem::property() */
+        /** @copydoc KConfigSkeletonItem::property() const */
         QVariant property() const;
     };
 
@@ -470,9 +482,10 @@ public:
     void readConfig(KConfig * config);
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
+    /** @copydoc KConfigSkeletonItem::property() const */
     QVariant property() const;
   };
 
@@ -493,10 +506,10 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
-    /** @copydoc KConfigSkeletonItem::property() */
+    /** @copydoc KConfigSkeletonItem::property() const */
     QVariant property() const;
   };
 
@@ -517,7 +530,7 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
     /** @copydoc KConfigSkeletonItem::property() */
@@ -562,7 +575,7 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
     /** @copydoc KConfigSkeletonItem::property() */
@@ -594,6 +607,7 @@ public:
   class KDECORE_EXPORT ItemEnum:public ItemInt
   {
   public:
+    //KDE5: remove the old Choice struct, rename Choice2 to Choice
     struct Choice
     {
       QString name;
@@ -601,13 +615,28 @@ public:
       QString whatsThis;
     };
 
+    struct Choice2
+    {
+      QString name;
+      QString label;
+      QString toolTip;
+      QString whatsThis;
+    };
+
     /** @copydoc KConfigSkeletonGenericItem::KConfigSkeletonGenericItem
         @param choices The list of enums that can be stored in this item
       */
     ItemEnum(const QString & _group, const QString & _key, qint32 &reference,
              const QList<Choice> &choices, qint32 defaultValue = 0);
 
+    /** @copydoc KConfigSkeletonGenericItem::KConfigSkeletonGenericItem
+        @param choices The list of enums that can be stored in this item
+      */
+    ItemEnum(const QString & _group, const QString & _key, qint32 &reference,
+             const QList<Choice2> &choices, qint32 defaultValue = 0);
+
     QList<Choice> choices() const;
+    QList<Choice2> choices2() const;
 
     /** @copydoc KConfigSkeletonItem::readConfig(KConfig*) */
     void readConfig(KConfig * config);
@@ -616,7 +645,7 @@ public:
     void writeConfig(KConfig * config);
 
   private:
-      QList<Choice> mChoices;
+    QList<Choice2> mChoices;
   };
 
 
@@ -636,7 +665,7 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
     /** @copydoc KConfigSkeletonItem::property() */
@@ -677,7 +706,7 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
     /** @copydoc KConfigSkeletonItem::property() */
@@ -719,7 +748,7 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
     /** @copydoc KConfigSkeletonItem::property() */
@@ -761,7 +790,7 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
     /** @copydoc KConfigSkeletonItem::property() */
@@ -785,7 +814,7 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
     /** @copydoc KConfigSkeletonItem::property() */
@@ -809,7 +838,7 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
     /** @copydoc KConfigSkeletonItem::property() */
@@ -834,7 +863,7 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
     /** @copydoc KConfigSkeletonItem::property() */
@@ -859,7 +888,7 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
     /** @copydoc KConfigSkeletonItem::property() */
@@ -904,7 +933,7 @@ public:
         /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
         void setProperty(const QVariant & p);
 
-        /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+        /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
 	bool isEqual(const QVariant &p) const;
 
         /** @copydoc KConfigSkeletonItem::property() */
@@ -928,7 +957,7 @@ public:
     /** @copydoc KConfigSkeletonItem::setProperty(const QVariant&) */
     void setProperty(const QVariant & p);
 
-    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) */
+    /** @copydoc KConfigSkeletonItem::isEqual(const QVariant &) const */
     bool isEqual(const QVariant &p) const;
 
     /** @copydoc KConfigSkeletonItem::property() */
@@ -941,21 +970,23 @@ public:
    * Constructor.
    *
    * @param configname name of config file. If no name is given, the default
-   * config file as returned by KGlobal::config() is used.
+   *                   config file as returned by KGlobal::config() is used
+   * @param parent the parent object (see QObject documentation)
    */
   explicit KCoreConfigSkeleton(const QString & configname = QString(), QObject* parent = 0);
 
   /**
    * Constructor.
    *
-   * @param config configuration object to use.
+   * @param config configuration object to use
+   * @param parent the parent object (see QObject documentation)
    */
   explicit KCoreConfigSkeleton(KSharedConfig::Ptr config, QObject* parent = 0);
 
   /**
    * Destructor
    */
-    virtual ~ KCoreConfigSkeleton();
+  virtual ~KCoreConfigSkeleton();
 
   /**
    * Set all registered items to their default values.
@@ -1002,10 +1033,13 @@ public:
   QString currentGroup() const;
 
   /**
-   * Register a custom @ref KConfigSkeletonItem with a given name. If the name
-   * parameter is null, take the name from KConfigSkeletonItem::key().
+   * Register a custom @ref KConfigSkeletonItem with a given name.
+   *
+   * If the name parameter is null, take the name from KConfigSkeletonItem::key().
    * Note that all names must be unique but that multiple entries can have
    * the same key if they reside in different groups.
+   *
+   * KCoreConfigSkeleton takes ownership of the KConfigSkeletonItem.
    */
   void addItem(KConfigSkeletonItem *, const QString & name = QString() );
 
@@ -1133,6 +1167,10 @@ public:
                           qint64 defaultValue = 0,
                           const QString & key = QString());
 
+  /**
+   * @deprecated
+   * Use addItemLongLong().
+   */
   KDE_DEPRECATED ItemLongLong *addItemInt64( const QString& name, qint64 &reference,
                           qint64 defaultValue = 0,
                           const QString & key = QString());
@@ -1152,6 +1190,10 @@ public:
                             quint64 defaultValue = 0,
                             const QString & key = QString());
 
+  /**
+   * @deprecated
+   * Use addItemULongLong().
+   */
   KDE_DEPRECATED ItemULongLong *addItemUInt64(const QString & name, quint64 &reference,
                             quint64 defaultValue = 0,
                             const QString & key = QString());
