@@ -1,6 +1,6 @@
 /*
    This file is part of the KDE libraries
-   Copyright (c) 2005-2007 David Jarvie <djarvie@kde.org>
+   Copyright (c) 2005-2009 David Jarvie <djarvie@kde.org>
    Copyright (c) 2005 S.R.Haque <srhaque@iee.org>.
 
    This library is free software; you can redistribute it and/or
@@ -22,7 +22,9 @@
 // This file requires HAVE_STRUCT_TM_TM_ZONE to be defined if struct tm member tm_zone is available.
 // This file requires HAVE_TM_GMTOFF to be defined if struct tm member tm_gmtoff is available.
 
-#include "config.h"
+#include "ksystemtimezone.moc"
+
+#include <config.h>
 
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
@@ -30,10 +32,6 @@
 #ifdef HAVE_TIME_H
 #include <time.h>
 #endif
-
-#include "ksystemtimezone.moc"
-
-
 #include <climits>
 #include <cstdlib>
 
@@ -56,7 +54,9 @@
 #include <kdebug.h>
 #include <kconfiggroup.h>
 #include "ktzfiletimezone.h"
-
+#ifdef Q_OS_WIN
+#include "ktimezone_win.h"
+#endif
 
 #define KTIMEZONED_DBUS_IFACE "org.kde.KTimeZoned"
 
@@ -98,7 +98,7 @@ int gmtoff(time_t t)
     if (lwday != uwday)
     {
       // Adjust for different day
-      if (lwday == uwday + 1  ||  lwday == 0 && uwday == 6)
+      if (lwday == uwday + 1  ||  (lwday == 0 && uwday == 6))
         lt += 24*3600;
       else
         lt -= 24*3600;
@@ -118,7 +118,12 @@ public:
     static void setLocalZone();
     static void cleanup();
     static void readConfig(bool init);
-#ifndef Q_OS_WIN
+#ifdef Q_OS_WIN
+    static void updateTimezoneInformation()
+    {
+      instance()->updateTimezoneInformation(true);
+    }
+#else
     static void updateZonetab()  { instance()->readZoneTab(true); }
 #endif
 
@@ -130,7 +135,9 @@ public:
 
 private:
     KSystemTimeZonesPrivate() {}
-#ifndef Q_OS_WIN
+#ifdef Q_OS_WIN
+    void updateTimezoneInformation(bool update);
+#else
     void readZoneTab(bool update);
     static float convertCoordinate(const QString &coordinate);
 #endif
@@ -160,6 +167,11 @@ KTzfileTimeZoneSource *KSystemTimeZonesPrivate::tzfileSource()
 }
 
 
+#ifndef NDEBUG
+K_GLOBAL_STATIC(KTimeZone, simulatedLocalZone)
+#endif
+
+
 KSystemTimeZones::KSystemTimeZones()
   : d(0)
 {
@@ -176,10 +188,36 @@ KSystemTimeZones::~KSystemTimeZones()
 
 KTimeZone KSystemTimeZones::local()
 {
+#ifndef NDEBUG
+    if (simulatedLocalZone->isValid())
+	return *simulatedLocalZone;
+#endif
     KSystemTimeZonesPrivate::instance();
     return KSystemTimeZonesPrivate::m_localZone;
 }
 
+KTimeZone KSystemTimeZones::realLocalZone()
+{
+    KSystemTimeZonesPrivate::instance();
+    return KSystemTimeZonesPrivate::m_localZone;
+}
+
+void KSystemTimeZones::setLocalZone(const KTimeZone& tz)
+{
+#ifndef NDEBUG
+    *simulatedLocalZone = tz;
+#endif
+}
+
+bool KSystemTimeZones::isSimulated()
+{
+#ifndef NDEBUG
+    return simulatedLocalZone->isValid();
+#else
+    return false;
+#endif
+}
+
 QString KSystemTimeZones::zoneinfoDir()
 {
     KSystemTimeZonesPrivate::instance();
@@ -258,6 +296,7 @@ kDebug(161)<<"instance(): ... initialised";
 #ifdef Q_OS_WIN
         // On Windows, HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Time Zones
         // is the place to look. The TZI binary value is the TIME_ZONE_INFORMATION structure.
+        m_instance->updateTimezoneInformation(false);
 #else
         // For Unix, read zone.tab.
         if (!m_zonetab.isEmpty())
@@ -315,7 +354,43 @@ void KSystemTimeZonesPrivate::cleanup()
     delete m_tzfileSource;
 }
 
-#ifndef Q_OS_WIN
+#ifdef Q_OS_WIN
+
+void KSystemTimeZonesPrivate::updateTimezoneInformation(bool update)
+{
+    if (!m_source)
+        m_source = new KSystemTimeZoneSourceWindows;
+    QStringList newZones;
+    Q_FOREACH( const QString & tz, KSystemTimeZoneWindows::listTimeZones() ) {
+       // const std::wstring wstr = tz.toStdWString();
+       // const KTimeZone info = make_time_zone( wstr.c_str() );
+      KSystemTimeZoneWindows stz(m_source, tz);
+      if (update)
+        {
+            // Update the existing collection with the new zone definition
+            newZones += stz.name();
+            KTimeZone oldTz = zone(stz.name());
+            if (oldTz.isValid())
+                oldTz.updateBase(stz);   // the zone previously existed, so update its definition
+            else
+                add(stz);   // the zone didn't previously exist, so add it
+        }
+        else
+            add(stz);
+    }
+    if (update)
+    {
+        // Remove any zones from the collection which no longer exist
+        const ZoneMap oldZones = zones();
+        for (ZoneMap::const_iterator it = oldZones.begin();  it != oldZones.end();  ++it)
+        {
+            if (newZones.indexOf(it.key()) < 0)
+                remove(it.value());
+        }
+    }
+}
+
+#else
 /*
  * Find the location of the zoneinfo files and store in mZoneinfoDir.
  * Parse zone.tab and for each time zone, create a KSystemTimeZone instance.
@@ -384,7 +459,7 @@ void KSystemTimeZonesPrivate::readZoneTab(bool update)
     {
         // Remove any zones from the collection which no longer exist
         const ZoneMap oldZones = zones();
-        for (ZoneMap::const_iterator it = oldZones.begin();  it != oldZones.end();  ++it)
+        for (ZoneMap::ConstIterator it = oldZones.constBegin();  it != oldZones.constEnd();  ++it)
         {
             if (newZones.indexOf(it.key()) < 0)
                 remove(it.value());
@@ -448,7 +523,7 @@ int KSystemTimeZoneBackend::offsetAtZoneTime(const KTimeZone *caller, const QDat
     if (!caller->isValid()  ||  !zoneDateTime.isValid()  ||  zoneDateTime.timeSpec() != Qt::LocalTime)
         return 0;
     // Make this time zone the current local time zone
-    const char *originalZone = ::getenv("TZ");   // save the original local time zone
+    const QByteArray originalZone = qgetenv("TZ");   // save the original local time zone
     QByteArray tz = caller->name().toUtf8();
     tz.prepend(":");
     bool change = (tz != originalZone);
@@ -503,7 +578,7 @@ int KSystemTimeZoneBackend::offsetAtZoneTime(const KTimeZone *caller, const QDat
     if (change)
     {
         // Restore the original local time zone
-        if (!originalZone)
+        if (originalZone.isEmpty())
             ::unsetenv("TZ");
         else
             ::setenv("TZ", originalZone, 1);
@@ -523,7 +598,7 @@ int KSystemTimeZoneBackend::offset(const KTimeZone *caller, time_t t) const
         return 0;
 
     // Make this time zone the current local time zone
-    const char *originalZone = ::getenv("TZ");   // save the original local time zone
+    const QByteArray originalZone = qgetenv("TZ");   // save the original local time zone
     QByteArray tz = caller->name().toUtf8();
     tz.prepend(":");
     bool change = (tz != originalZone);
@@ -538,7 +613,7 @@ int KSystemTimeZoneBackend::offset(const KTimeZone *caller, time_t t) const
     if (change)
     {
         // Restore the original local time zone
-        if (!originalZone)
+        if (originalZone.isEmpty())
             ::unsetenv("TZ");
         else
             ::setenv("TZ", originalZone, 1);
@@ -600,13 +675,13 @@ class KSystemTimeZoneSourcePrivate
 public:
     static void setTZ(const QByteArray &zoneName);
     static void restoreTZ();
-    static char      *savedTZ;       // temporary value of TZ environment variable saved by setTZ()
+    static QByteArray savedTZ;       // temporary value of TZ environment variable saved by setTZ()
     static QByteArray originalTZ;    // saved value of TZ environment variable during multiple parse() calls
     static bool       TZIsSaved;     // TZ has been saved in savedTZ
     static bool       multiParse;    // true if performing multiple parse() calls
 };
 
-char      *KSystemTimeZoneSourcePrivate::savedTZ;
+QByteArray KSystemTimeZoneSourcePrivate::savedTZ;
 QByteArray KSystemTimeZoneSourcePrivate::originalTZ;
 bool       KSystemTimeZoneSourcePrivate::TZIsSaved = false;
 bool       KSystemTimeZoneSourcePrivate::multiParse = false;
@@ -643,7 +718,7 @@ KTimeZoneData* KSystemTimeZoneSource::parse(const KTimeZone &zone) const
 
 void KSystemTimeZoneSource::startParseBlock()
 {
-    KSystemTimeZoneSourcePrivate::originalTZ = ::getenv("TZ");   // save the original local time zone
+    KSystemTimeZoneSourcePrivate::originalTZ = qgetenv("TZ");   // save the original local time zone
     KSystemTimeZoneSourcePrivate::multiParse = true;
 }
 
@@ -670,7 +745,7 @@ void KSystemTimeZoneSourcePrivate::setTZ(const QByteArray &zoneName)
     bool setTZ = multiParse;
     if (!setTZ)
     {
-        savedTZ = ::getenv("TZ");   // save the original local time zone
+        savedTZ = qgetenv("TZ");   // save the original local time zone
         TZIsSaved = true;
         setTZ = (tz != savedTZ);
     }
@@ -686,7 +761,7 @@ void KSystemTimeZoneSourcePrivate::restoreTZ()
 {
     if (TZIsSaved)
     {
-        if (!savedTZ)
+        if (savedTZ.isEmpty())
             ::unsetenv("TZ");
         else
             ::setenv("TZ", savedTZ, 1);
@@ -771,3 +846,4 @@ QList<int> KSystemTimeZoneData::utcOffsets() const
 {
     return QList<int>();
 }
+
