@@ -41,7 +41,7 @@ class KUrlPrivate;
 
 /**
  * \class KUrl kurl.h <KUrl>
- * 
+ *
  * Represents and parses a URL.
  *
  * A prototypical URL looks like:
@@ -65,12 +65,12 @@ class KUrlPrivate;
  * \endcode
  *
  * If you have the URL of a local file or directory and need the absolute path,
- * you would use path().
+ * you would use toLocalFile().
  * \code
  *    KUrl url( "file:///bar/%23foo%23" );
  *    ...
  *    if ( url.isLocalFile() )
- *       QString path = url.path();       // -> "/bar/#foo#"
+ *       QString path = url.toLocalFile();       // -> "/bar/#foo#"
  * \endcode
  *
  * This must also be considered when you have separated directory and file
@@ -108,7 +108,7 @@ class KUrlPrivate;
  * context.
  *
  */
-class KDECORE_EXPORT KUrl : public QUrl // krazy:exclude=dpointer (krazy can't deal with embedded classes)
+class KDECORE_EXPORT KUrl : public QUrl // krazy:exclude=dpointer,qclasses (krazy can't deal with embedded classes)
 {
 public:
   typedef QMap<QString, QString> MetaDataMap;
@@ -116,11 +116,11 @@ public:
 
   /**
    * \class List kurl.h <KUrl>
-   * 
+   *
    * KUrl::List is a QList that contains KUrls with a few
    * convenience methods.
    * @see KUrl
-   * @see QValueList
+   * @see QList
    */
   class KDECORE_EXPORT List : public QList<KUrl> //krazy:exclude=dpointer (just some convenience methods)
   {
@@ -179,6 +179,45 @@ public:
                              const KUrl::MetaDataMap& metaData = MetaDataMap(),
                              MimeDataFlags flags = DefaultMimeDataFlags ) const;
 
+      /**
+       * Adds URLs into the given QMimeData.
+       *
+       * This should add both the KDE-style URLs (eg: desktop:/foo) and
+       * the "most local" version of the URLs (eg:
+       * file:///home/jbloggs/Desktop/foo) to the mimedata.
+       *
+       * This method should be called on the KDE-style URLs.
+       *
+       * @code
+       * QMimeData* mimeData = new QMimeData();
+       *
+       * KUrl::List kdeUrls;
+       * kdeUrls << "desktop:/foo";
+       * kdeUrls << "desktop:/bar";
+       *
+       * KUrl::List normalUrls;
+       * normalUrls << "file:///home/jbloggs/Desktop/foo";
+       * normalUrls << "file:///home/jbloggs/Desktop/bar";
+       *
+       * kdeUrls.populateMimeData(normalUrls, mimeData);
+       * @endcode
+       *
+       * @param mostLocalUrls the "most local" urls
+       * @param mimeData      the mime data object to populate
+       * @param metaData      KIO metadata shipped in the mime data, which is
+       *                      used for instance to set a correct HTTP referrer
+       *                      (some websites require it for downloading e.g. an
+       *                      image)
+       * @param flags         set NoTextExport to prevent setting plain/text
+       *                      data into @p mimeData. In such a case,
+       *                      <code>setExportAsText(false)</code> should be called.
+       * @since 4.2
+       */
+      void populateMimeData(const KUrl::List& mostLocalUrls,
+                            QMimeData* mimeData,
+                            const KUrl::MetaDataMap& metaData = MetaDataMap(),
+                            MimeDataFlags flags = DefaultMimeDataFlags) const;
+
       /**
        * Return true if @p mimeData contains URI data
        */
@@ -199,6 +238,41 @@ public:
        */
       static KUrl::List fromMimeData( const QMimeData *mimeData, KUrl::MetaDataMap* metaData = 0 );
 
+      /**
+       * Flags to be used in fromMimeData.
+       * @since 4.2.3
+       */
+      enum DecodeOptions {
+        /**
+         * When the mimedata contains both KDE-style URLs (eg: desktop:/foo) and
+         * the "most local" version of the URLs (eg: file:///home/dfaure/Desktop/foo),
+         * decode it as local urls. Useful in paste/drop operations that end up calling KIO,
+         * so that urls from other users work as well.
+         */
+        PreferLocalUrls,
+        /**
+         * When the mimedata contains both KDE-style URLs (eg: desktop:/foo) and
+         * the "most local" version of the URLs (eg: file:///home/dfaure/Desktop/foo),
+         * decode it as the KDE-style URL. Useful in DnD code e.g. when moving icons,
+         * and the kde-style url is used as identifier for the icons.
+         */
+        PreferKdeUrls
+      };
+
+      /**
+       * Extract a list of KUrls from the contents of @p mimeData.
+       * Decoding will fail if @p mimeData does not contain any URLs, or if at
+       * least one extracted URL is not valid.
+       * @param mimeData the mime data to extract from; cannot be 0
+       * @param decodeOptions options for decoding
+       * @param metaData optional pointer to a map holding the metadata
+       * @return the list of urls
+       * @since 4.2.3
+       */
+      static KUrl::List fromMimeData( const QMimeData *mimeData,
+                                      DecodeOptions decodeOptions, // TODO KDE5: = PreferKdeUrls, and merge with above
+                                      KUrl::MetaDataMap* metaData = 0 );
+
   };
   /**
    * Constructs an empty URL.
@@ -239,7 +313,7 @@ public:
    * Converts from a QUrl.
    * @param u the QUrl
    */
-  KUrl( const QUrl &u );
+  KUrl( const QUrl &u ); //krazy:exclude=qclasses
   /**
    * Constructor allowing relative URLs.
    *
@@ -552,10 +626,10 @@ public:
 
   /**
    * Adds to the current path.
-   * Assumes that the current path is a directory. @p _txt is appended to the
+   * Assumes that the current path is a directory. @p txt is appended to the
    * current path. The function adds '/' if needed while concatenating.
    * This means it does not matter whether the current path has a trailing
-   * '/' or not. If there is none, it becomes appended. If @p _txt
+   * '/' or not. If there is none, it becomes appended. If @p txt
    * has a leading '/' then this one is stripped.
    *
    * @param txt The text to add. It is considered to be decoded.
@@ -652,6 +726,7 @@ public:
     AppendTrailingSlash = 0x04,
     /**
      * Opposite of ObeyTrailingSlash  (default)
+     * fileName("file:/foo/") and fileName("file:/foo") is "foo" in both cases.
      */
     IgnoreTrailingSlash = 0x01
 
@@ -738,6 +813,12 @@ public:
    * @return the new KUrl
    */
   QString pathOrUrl() const;
+  /**
+   * Overload with @p trailing parameter
+   * @param trailing use to add or remove a trailing slash to/from the path. see adjustPath.
+   * @since 4.2
+   */
+  QString pathOrUrl(AdjustPathOption trailing) const; // KDE5: merge with above
 
   /**
    * Returns the URL as a string, using the standard conventions for mime data
@@ -923,7 +1004,7 @@ public:
    * everything is 7 bit (ascii) after being percent-encoded.
    */
   static KDE_DEPRECATED QString encode_string(const QString &str) {
-      return QLatin1String( QUrl::toPercentEncoding( str ) );
+      return QLatin1String( QUrl::toPercentEncoding( str ) ); //krazy:exclude=qclasses
   }
 
   /**
@@ -938,7 +1019,7 @@ public:
    *
    */
   static KDE_DEPRECATED QString encode_string_no_slash(const QString &str) {
-      return QString::fromLatin1( QUrl::toPercentEncoding( str, "/" ) );
+      return QString::fromLatin1( QUrl::toPercentEncoding( str, "/" ) ); //krazy:exclude=qclasses
   }
 
   /**
@@ -952,7 +1033,7 @@ public:
    *
    */
   static KDE_DEPRECATED QString decode_string(const QString &str) {
-      return QUrl::fromPercentEncoding( str.toLatin1() );
+      return QUrl::fromPercentEncoding( str.toLatin1() ); //krazy:exclude=qclasses
   }
 
 
@@ -961,6 +1042,17 @@ public:
    *
    * Returns whether '_url' is likely to be a "relative" URL instead of
    * an "absolute" URL.
+   *
+   * This is mostly meant for KUrl(url, relativeUrl).
+   *
+   * If you are looking for the notion of "relative path" (foo) vs "absolute path" (/foo),
+   * use QUrl::isRelative() instead.
+   * Indeed, isRelativeUrl() returns true for the string "/foo" since it doesn't contain a protocol,
+   * while KUrl("/foo").isRelative() is false since the KUrl constructor turns it into file:///foo.
+   * The two methods basically test the same thing, but this one takes a string (which is faster)
+   * while the class method requires a QUrl/KUrl which gives a more expected result, given
+   * the "magic" in the KUrl constructor.
+   *
    * @param _url URL to examine
    * @return true when the URL is likely to be "relative", false otherwise.
    */
@@ -1033,10 +1125,6 @@ KDECORE_EXPORT bool urlcmp( const QString& _url1, const QString& _url2 );
  */
 KDECORE_EXPORT bool urlcmp( const QString& _url1, const QString& _url2, const KUrl::EqualsOptions& options );
 
-/*
- * MSVC needs it.
- */
-Q_CORE_EXPORT uint qHash(const QString &key);
-inline uint qHash(const KUrl& kurl){ return qHash(kurl.url()); }
+KDECORE_EXPORT uint qHash(const KUrl& kurl);
 
 #endif
