@@ -29,6 +29,7 @@
 
 #include <kprotocolinfo.h>
 #include <kdebug.h>
+#include <kservicegroup.h>
 #include <kservicetypetrader.h>
 #include <kservicetype.h>
 #include <kservicetypeprofile.h>
@@ -49,19 +50,12 @@ void KServiceTest::initTestCase()
 
 
     // Create fake service for some tests below.
-    bool mustUpdateKSycoca = false;
+    bool mustUpdateKSycoca = !KService::serviceByDesktopPath("fakeservice.desktop");
     const QString fakeService = KStandardDirs::locateLocal("services", "fakeservice.desktop");
     const bool mustCreateFakeService = !QFile::exists(fakeService);
     if (mustCreateFakeService) {
         mustUpdateKSycoca = true;
-        KDesktopFile file(fakeService);
-        KConfigGroup group = file.desktopGroup();
-        group.writeEntry("Name", "FakePlugin");
-        group.writeEntry("Type", "Service");
-        group.writeEntry("X-KDE-Library", "faketextplugin");
-        group.writeEntry("X-KDE-Protocols", "http,ftp");
-        group.writeEntry("ServiceTypes", "KTextEditor/Plugin");
-        group.writeEntry("MimeType", "text/plain;");
+        createFakeService();
     }
 
     if ( mustUpdateKSycoca ) {
@@ -82,21 +76,21 @@ void KServiceTest::testByName()
     QVERIFY( s0 );
     QCOMPARE( s0->name(), QString::fromLatin1("KParts/ReadOnlyPart") );
 
-    KService::Ptr kdeprintd = KService::serviceByDesktopPath("kded/kwalletd.desktop");
-    QCOMPARE( kdeprintd->name(), QString::fromLatin1("KWallet Daemon Module"));
+    KService::Ptr khtml = KService::serviceByDesktopPath("khtml.desktop");
+    QCOMPARE( khtml->name(), QString::fromLatin1("KHTML"));
 }
 
 void KServiceTest::testProperty()
 {
-    KService::Ptr kdeprintd = KService::serviceByDesktopPath("kded/kwalletd.desktop");
-    QVERIFY(kdeprintd);
-    QCOMPARE(kdeprintd->entryPath(), QString("kded/kwalletd.desktop"));
+    KService::Ptr kdedkcookiejar = KService::serviceByDesktopPath("kded/kcookiejar.desktop");
+    QVERIFY(kdedkcookiejar);
+    QCOMPARE(kdedkcookiejar->entryPath(), QString("kded/kcookiejar.desktop"));
 
-    QCOMPARE(kdeprintd->property("ServiceTypes").toStringList().join(","), QString("KDEDModule"));
-    QCOMPARE(kdeprintd->property("X-KDE-Kded-autoload").toBool(), false);
-    QCOMPARE(kdeprintd->property("X-KDE-Kded-load-on-demand").toBool(), true);
-    QVERIFY(!kdeprintd->property("Name").toString().isEmpty());
-    QVERIFY(!kdeprintd->property("Name[fr]", QVariant::String).isValid());
+    QCOMPARE(kdedkcookiejar->property("ServiceTypes").toStringList().join(","), QString("KDEDModule"));
+    QCOMPARE(kdedkcookiejar->property("X-KDE-Kded-autoload").toBool(), false);
+    QCOMPARE(kdedkcookiejar->property("X-KDE-Kded-load-on-demand").toBool(), true);
+    QVERIFY(!kdedkcookiejar->property("Name").toString().isEmpty());
+    QVERIFY(!kdedkcookiejar->property("Name[fr]", QVariant::String).isValid());
 
     KService::Ptr kjavaappletviewer = KService::serviceByDesktopPath("kjavaappletviewer.desktop");
     QVERIFY(kjavaappletviewer);
@@ -242,12 +236,11 @@ void KServiceTest::testServiceTypeTraderForReadOnlyPart()
     // Check ordering according to InitialPreference
     int lastPreference = -1;
     bool lastAllowedAsDefault = true;
-    KService::List::const_iterator it = offers.begin();
-    for ( ; it != offers.end() ; it++ ) {
-        const QString path = (*it)->entryPath();
-        const int preference = (*it)->initialPreference(); // ## might be wrong if we use per-servicetype preferences...
-        //qDebug( "%s has preference %d, allowAsDefault=%d", qPrintable( path ), preference, (*it)->allowAsDefault() );
-        if ( lastAllowedAsDefault && !(*it)->allowAsDefault() ) {
+    Q_FOREACH(KService::Ptr service, offers) {
+        const QString path = service->entryPath();
+        const int preference = service->initialPreference(); // ## might be wrong if we use per-servicetype preferences...
+        //qDebug( "%s has preference %d, allowAsDefault=%d", qPrintable( path ), preference, service->allowAsDefault() );
+        if ( lastAllowedAsDefault && !service->allowAsDefault() ) {
             // first "not allowed as default" offer
             lastAllowedAsDefault = false;
             lastPreference = -1; // restart
@@ -259,8 +252,8 @@ void KServiceTest::testServiceTypeTraderForReadOnlyPart()
 
     // Now look for any KTextEditor/Plugin
     offers = KServiceTypeTrader::self()->query("KTextEditor/Plugin");
-    QVERIFY( offerListHasService( offers, "ktexteditor_docwordcompletion.desktop" ) );
     QVERIFY( offerListHasService( offers, "ktexteditor_insertfile.desktop" ) );
+    QVERIFY( offerListHasService( offers, "ktexteditor_kdatatool.desktop" ) );
 }
 
 void KServiceTest::testTraderConstraints()
@@ -268,9 +261,14 @@ void KServiceTest::testTraderConstraints()
     if ( !KSycoca::isAvailable() )
         QSKIP( "ksycoca not available", SkipAll );
 
-    KService::List offers = KServiceTypeTrader::self()->query("KTextEditor/Plugin", "Library == 'ktexteditor_docwordcompletion'");
+    KService::List offers = KServiceTypeTrader::self()->query("KTextEditor/Plugin", "Library == 'ktexteditor_insertfile'");
     QCOMPARE(offers.count(), 1);
-    QVERIFY( offerListHasService( offers, "ktexteditor_docwordcompletion.desktop" ) );
+    QVERIFY( offerListHasService( offers, "ktexteditor_insertfile.desktop" ) );
+
+    // A test with an invalid query, to test for memleaks
+    offers = KServiceTypeTrader::self()->query("KTextEditor/Plugin", "A == B OR C == D AND OR Foo == 'Parse Error'");
+    // The constraint is ignored, in such cases, so we do get results.
+    QVERIFY(!offers.isEmpty());
 }
 
 void KServiceTest::testHasServiceType1() // with services constructed with a full path (rare)
@@ -281,11 +279,11 @@ void KServiceTest::testHasServiceType1() // with services constructed with a ful
     QVERIFY( katepart.hasServiceType( "KParts/ReadOnlyPart" ) );
     QVERIFY( katepart.hasServiceType( "KParts/ReadWritePart" ) );
 
-    QString ktexteditor_docwordcompletionPath = KStandardDirs::locate( "services", "ktexteditor_docwordcompletion.desktop" );
-    QVERIFY( !ktexteditor_docwordcompletionPath.isEmpty() );
-    KService ktexteditor_docwordcompletion( ktexteditor_docwordcompletionPath );
-    QVERIFY( ktexteditor_docwordcompletion.hasServiceType( "KTextEditor/Plugin" ) );
-    QVERIFY( !ktexteditor_docwordcompletion.hasServiceType( "KParts/ReadOnlyPart" ) );
+    QString ktexteditor_insertfilePath = KStandardDirs::locate( "services", "ktexteditor_insertfile.desktop" );
+    QVERIFY( !ktexteditor_insertfilePath.isEmpty() );
+    KService ktexteditor_insertfile( ktexteditor_insertfilePath );
+    QVERIFY( ktexteditor_insertfile.hasServiceType( "KTextEditor/Plugin" ) );
+    QVERIFY( !ktexteditor_insertfile.hasServiceType( "KParts/ReadOnlyPart" ) );
 }
 
 void KServiceTest::testHasServiceType2() // with services coming from ksycoca
@@ -295,10 +293,10 @@ void KServiceTest::testHasServiceType2() // with services coming from ksycoca
     QVERIFY( katepart->hasServiceType( "KParts/ReadOnlyPart" ) );
     QVERIFY( katepart->hasServiceType( "KParts/ReadWritePart" ) );
 
-    KService::Ptr ktexteditor_docwordcompletion = KService::serviceByDesktopPath( "ktexteditor_docwordcompletion.desktop" );
-    QVERIFY( !ktexteditor_docwordcompletion.isNull() );
-    QVERIFY( ktexteditor_docwordcompletion->hasServiceType( "KTextEditor/Plugin" ) );
-    QVERIFY( !ktexteditor_docwordcompletion->hasServiceType( "KParts/ReadOnlyPart" ) );
+    KService::Ptr ktexteditor_insertfile = KService::serviceByDesktopPath( "ktexteditor_insertfile.desktop" );
+    QVERIFY( !ktexteditor_insertfile.isNull() );
+    QVERIFY( ktexteditor_insertfile->hasServiceType( "KTextEditor/Plugin" ) );
+    QVERIFY( !ktexteditor_insertfile->hasServiceType( "KParts/ReadOnlyPart" ) );
 }
 
 void KServiceTest::testWriteServiceTypeProfile()
@@ -390,3 +388,111 @@ void KServiceTest::testActionsAndDataStream()
     QCOMPARE(loadedService.exec(), service.exec());
     QCOMPARE(loadedService.actions().count(), 3);
 }
+
+void KServiceTest::testServiceGroups()
+{
+    KServiceGroup::Ptr root = KServiceGroup::root();
+    QVERIFY(root);
+    qDebug() << root->groupEntries().count();
+
+    KServiceGroup::Ptr group = root;
+    QVERIFY(group);
+    const KServiceGroup::List list = group->entries(true /* sorted */,
+                                                   true /* exclude no display entries */,
+                                                   false /* allow separators */,
+                                                   true /* sort by generic name */);
+
+    kDebug() << list.count();
+    Q_FOREACH(KServiceGroup::SPtr s, list) {
+        qDebug() << s->name() << s->entryPath();
+    }
+
+    // No unit test here yet, but at least this can be valgrinded for errors.
+}
+
+void KServiceTest::testKSycocaUpdate()
+{
+    KService::Ptr fakeService = KService::serviceByDesktopPath("fakeservice.desktop");
+    QVERIFY(fakeService); // see initTestCase; it should be found.
+    const QString servPath = KStandardDirs::locateLocal("services", "fakeservice.desktop");
+    QVERIFY(QFile::exists(servPath));
+    QSignalSpy spy(KSycoca::self(), SIGNAL(databaseChanged(QStringList)));
+    QVERIFY(spy.isValid());
+    QFile::remove(servPath);
+    kDebug() << QThread::currentThread() << "executing kbuildsycoca";
+    QProcess::execute( KGlobal::dirs()->findExe(KBUILDSYCOCA_EXENAME) );
+    kDebug() << QThread::currentThread() << "done";
+    while (spy.isEmpty())
+        QTest::qWait(50);
+    QVERIFY(!spy.isEmpty());
+    QVERIFY(spy[0][0].toStringList().contains("services"));
+    kDebug() << QThread::currentThread() << "got signal ok";
+
+    spy.clear();
+    QVERIFY(fakeService); // the whole point of refcounting is that this KService instance is still valid.
+    QVERIFY(!QFile::exists(servPath));
+
+    // Recreate it, for future tests
+    createFakeService();
+    QVERIFY(QFile::exists(servPath));
+    kDebug() << QThread::currentThread() << "executing kbuildsycoca (2)";
+    QProcess::execute( KGlobal::dirs()->findExe(KBUILDSYCOCA_EXENAME) );
+    kDebug() << QThread::currentThread() << "done (2)";
+    while (spy.isEmpty())
+        QTest::qWait(50);
+    kDebug() << QThread::currentThread() << "got signal ok (2)";
+    QVERIFY(spy[0][0].toStringList().contains("services"));
+    if (QThread::currentThread() != QCoreApplication::instance()->thread())
+        m_sycocaUpdateDone.ref();
+}
+
+void KServiceTest::createFakeService()
+{
+    const QString fakeService = KStandardDirs::locateLocal("services", "fakeservice.desktop");
+    KDesktopFile file(fakeService);
+    KConfigGroup group = file.desktopGroup();
+    group.writeEntry("Name", "FakePlugin");
+    group.writeEntry("Type", "Service");
+    group.writeEntry("X-KDE-Library", "faketextplugin");
+    group.writeEntry("X-KDE-Protocols", "http,ftp");
+    group.writeEntry("ServiceTypes", "KTextEditor/Plugin");
+    group.writeEntry("MimeType", "text/plain;");
+}
+
+#include <QThreadPool>
+#include <qtconcurrentrun.h>
+
+// Testing for concurrent access to ksycoca from multiple threads
+// It's especially interesting to run this test as ./kservicetest testThreads
+// so that even the ksycoca initialization is happening from N threads at the same time.
+// Use valgrind --tool=helgrind to see the race conditions.
+
+void KServiceTest::testReaderThreads()
+{
+    QThreadPool::globalInstance()->setMaxThreadCount(10);
+    QList<QFuture<void> > futures;
+    futures << QtConcurrent::run(this, &KServiceTest::testAllServices);
+    futures << QtConcurrent::run(this, &KServiceTest::testAllServices);
+    futures << QtConcurrent::run(this, &KServiceTest::testAllServices);
+    futures << QtConcurrent::run(this, &KServiceTest::testHasServiceType1);
+    futures << QtConcurrent::run(this, &KServiceTest::testAllServices);
+    futures << QtConcurrent::run(this, &KServiceTest::testAllServices);
+    Q_FOREACH(QFuture<void> f, futures)
+        f.waitForFinished();
+    QThreadPool::globalInstance()->setMaxThreadCount(1); // delete those threads
+}
+
+void KServiceTest::testThreads()
+{
+    QThreadPool::globalInstance()->setMaxThreadCount(10);
+    QList<QFuture<void> > futures;
+    futures << QtConcurrent::run(this, &KServiceTest::testAllServices);
+    futures << QtConcurrent::run(this, &KServiceTest::testHasServiceType1);
+    futures << QtConcurrent::run(this, &KServiceTest::testKSycocaUpdate);
+    futures << QtConcurrent::run(this, &KServiceTest::testTraderConstraints);
+    while (m_sycocaUpdateDone == 0) // not using a bool, just to silence helgrind
+        QTest::qWait(100); // process dbus events!
+    kDebug() << "Joining all threads";
+    Q_FOREACH(QFuture<void> f, futures)
+        f.waitForFinished();
+}
