@@ -75,7 +75,7 @@ class KRockerGesture;
  * GUI element you want.  The KAction class will then take care of
  * correctly defining the menu item (with icons, accelerators, text,
  * etc), toolbar button, or other.  From then on, if you
- * manipulate the action at all, the effect will propogate through all
+ * manipulate the action at all, the effect will propagate through all
  * GUI representations of it.  Back to the "cut" example: if you want
  * to disable the Cut Action, you would simply call
  * 'cutAction->setEnabled(false)' and both the menuitem and button would
@@ -121,9 +121,9 @@ class KRockerGesture;
  *
  * \section kaction_general The kinds of shortcuts
  *
- * Local shortcuts are active
- * in their context, global shortcus are active everywhere, usually even if
- * another program has focus.
+ * Local shortcuts are active if their context has the focus, global shortcus
+ * are active even if the program does not have the focus. If a global
+ * shortcut and a local shortcut are ambiguous the global shortcut wins.
  *
  * @li Active shortcuts trigger a KAction if activated.
  * @li Default shortcuts are what the active shortcuts revert to if the user chooses
@@ -133,15 +133,30 @@ class KRockerGesture;
  *
  * Here is an example of enabling a "New [document]" action
  * \code
- * KAction *newAct = new KAction("filenew", i18n("&New"), actionCollection(), "new");
- * newAct->setShortcut(KStandardShortcut::shortcut(KStandardShortcut::New));
- * connect(newAct, SIGNAL(triggered(bool)), SLOT(fileNew()));
+ * KAction *newAct = actionCollection()->addAction(
+ *              KStandardAction::New,   //< see KStandardAction
+ *              this,                   //< Receiver
+ *              SLOT(fileNew())  );     //< SLOT
  * \endcode
  *
- * This section creates our action.  It says that wherever this action is
- * displayed, it will use "&New" as the text, the standard icon, and
- * the standard shortcut.  It further says that whenever this action
- * is invoked, it will use the fileNew() slot to execute it.
+ * This section creates our action. Text, Icon and Shortcut will be set from
+ * KStandardAction. KStandardAction ensures your application complies to the
+ * platform standards. When triggered the \c fileNew() slot will be called.
+ *
+ * @see KStandardAction for more information.
+ *
+ * If you want to create your own actions use
+ * \code
+ * KAction *newAct = actionCollection()->addAction("quick-connect");
+ * newAct->setText(i18n("Quick Connect"))
+ * newAct->setIcon(KIcon("quick-connect"));
+ * newAct->setShortcut(Qt::Key_F6);
+ * connect(newAct, SIGNAL(triggered()), this, SLOT(quickConnect()));
+ * \endcode
+ *
+ * This section creates our action. It displays the text "Quick Connect",
+ * uses the Icon "quick-connect" and pressing \c F6 will trigger it. When
+ * invoked, the slot quickConnect() is called.
  *
  * \code
  * QMenu *file = new QMenu;
@@ -170,15 +185,20 @@ class KRockerGesture;
  * when you have finished with it - the destructor takes care of all
  * of the cleanup.
  *
- * \note calling QAction::setShortcut() on a KAction may lead to unexpected
+ * \warning calling QAction::setShortcut() on a KAction may lead to unexpected
  * behavior. There is nothing we can do about it because QAction::setShortcut()
-*  is not virtual.
+ * is not virtual.
  *
  * \note if you are using a "standard" action like "new", "paste",
  * "quit", or any other action described in the KDE UI Standards,
  * please use the methods in the KStandardAction class rather than
  * defining your own.
  *
+ * \section Using QActions
+ *
+ * Mixing QActions and KActions in an application is not a
+ * good idea. KShortcutsEditor doesn't handle QActions at all.
+ *
  * \section kaction_xmlgui Usage Within the XML Framework
  *
  * If you are using KAction within the context of the XML menu and
@@ -196,6 +216,7 @@ class KDEUI_EXPORT KAction : public QWidgetAction
   Q_PROPERTY( bool shortcutConfigurable READ isShortcutConfigurable WRITE setShortcutConfigurable )
   Q_PROPERTY( KShortcut globalShortcut READ globalShortcut WRITE setGlobalShortcut )
   Q_PROPERTY( bool globalShortcutAllowed READ globalShortcutAllowed WRITE setGlobalShortcutAllowed )
+  Q_PROPERTY( bool globalShortcutEnabled READ isGlobalShortcutEnabled )
   Q_FLAGS( ShortcutType )
 
 public:
@@ -255,6 +276,25 @@ public:
      */
     virtual ~KAction();
 
+    /**
+     * Sets the help text for the action.
+     * This help text will be set for all help mechanisms:
+     *  - the status-bar help text
+     *  - the tooltip (for toolbar buttons)
+     *  - the "WhatsThis" help text (unless one was already set)
+     *
+     * This is more convenient than calling all three methods with the
+     * same text, and this level of abstraction can allow to change
+     * the default implementation of help one day more easily.
+     * Of course you can also call setStatusTip, setToolTip and setWhatsThis
+     * separately for more flexibility.
+     *
+     * This method is also the easiest way to port from KDE3's KAction::setToolTip.
+     *
+     * @since 4.3
+     */
+    void setHelpText(const QString& text);
+
     /**
      * Get the shortcut for this action.
      *
@@ -337,25 +377,38 @@ public:
 
     /**
      * Assign a global shortcut for this action. Global shortcuts
-     * allow your actions to respond to keys independently of the focused window.
-     * Unlike regular shortcuts, the application's window does not need focus
-     * for them to be activated.
+     * allow an action to respond to key shortcuts independently of the focused window,
+     * i.e. the action will trigger if the keys were pressed no matter where in the X session.
      *
-     * When an action, identified by main component name and text(), is assigned
+     * The action must have a per main component unique
+     * objectName() to enable cross-application bookeeping. If the objectName() is empty this method will
+     * do nothing, otherwise the isGlobalShortcutEnabled() property will be set to true and the
+     * shortcut will be enabled.
+     * It is mandatory that the objectName() doesn't change once isGlobalshortcutEnabled()
+     * has become true.
+     *
+     * \note KActionCollection::insert(name, action) will set action's objectName to name so you often
+     * don't have to set an objectName explicitly.
+     *
+     * When an action, identified by main component name and objectName(), is assigned
      * a global shortcut for the first time on a KDE installation the assignment will
-     * be saved. The shortcut will then be restored every time the action's
-     * globalShortcutAllowed flag becomes true.
-     * \e This \e includes \e calling \e setGlobalShortcut()!
-     * If you actually want to change the global shortcut you have to set
-     * @p loading to NoAutoloading. The new shortcut will be saved again.
+     * be saved. The shortcut will then be restored every time setGlobalShortcut() is
+     * called with @p loading == Autoloading.
      *
-     * \param shortcut global shortcut(s) to assign. Will be ignored unless \p loading is set to NoAutoloading.
+     * If you actually want to change the global shortcut you have to set
+     * @p loading to NoAutoloading. The new shortcut will be automatically saved again.
+     *
+     * \param shortcut global shortcut(s) to assign. Will be ignored unless \p loading is set to NoAutoloading or this is the first time ever you call this method (see above).
      * \param type the type of shortcut to be set, whether the active shortcut, the default shortcut,
      *             or both (the default).
-     * \param loading load the previous shortcut, which might have been changed by the user (Autoloading, the default)
-     *                 or really set \param shortcut as the new shortcut (NoAutoloading).
+     * \param loading if Autoloading, assign the global shortcut this action has previously had if any.
+     *                   That way user preferences and changes made to avoid clashes will be conserved.
+     *                if NoAutoloading the given shortcut will be assigned without looking up old values.
+     *                   You should only do this if the user wants to change the shortcut or if you have
+     *                   another very good reason. Key combinations that clash with other shortcuts will be
+     *                   dropped.
      *
-     * \sa KGlobalAccel
+     * \note the default shortcut will never be influenced by autoloading - it will be set as given.
      * \sa globalShortcut()
      */
     void setGlobalShortcut(const KShortcut& shortcut, ShortcutTypes type =
@@ -365,20 +418,37 @@ public:
     /**
      * Returns true if this action is permitted to have a global shortcut.
      * Defaults to false.
+     * Use isGlobalShortcutEnabled() instead.
      */
-    bool globalShortcutAllowed() const;
+    KDE_DEPRECATED bool globalShortcutAllowed() const;
 
     /**
      * Indicate whether the programmer and/or user may define a global shortcut for this action.
      * Defaults to false. Note that calling setGlobalShortcut() turns this on automatically.
      *
      * \param allowed set to \e true if this action may have a global shortcut, otherwise \e false.
-     * \param loading (see setGlobalShortcut). Ignore this parameter.
+     * \param loading if Autoloading, assign to this action the global shortcut it has previously had
+     *                if any.
      */
-    void setGlobalShortcutAllowed(bool allowed, GlobalShortcutLoading loading = Autoloading);
-    //^ TODO: NoAutoloading would make little sense in this method, right? We don't have the shortcut to set...
-    // KDE5: remove loading parameter.
+    KDE_DEPRECATED void setGlobalShortcutAllowed(bool allowed, GlobalShortcutLoading loading = Autoloading);
 
+    /**
+     * Returns true if this action is enabled to have a global shortcut.
+     * This will be respected by \class KGlobalShortcutsEditor.
+     * Defaults to false.
+     */
+    bool isGlobalShortcutEnabled() const;
+
+    /**
+     * Sets the globalShortcutEnabled property to false and sets the global shortcut to an
+     * empty shortcut.
+     * This will also wipe out knowlegde about the existence of this action's global shorctut
+     * so it will not be considered anymore for shortcut conflict resolution. It will also not be
+     * visible anymore in the shortcuts KControl module.
+     * This method should not be used unless these effects are explicitly desired.
+     * @since 4.1
+     */
+    void forgetGlobalShortcut();
 
     KShapeGesture shapeGesture(ShortcutTypes type = ActiveShortcut) const;
     KRockerGesture rockerGesture(ShortcutTypes type = ActiveShortcut) const;
@@ -386,6 +456,12 @@ public:
     void setShapeGesture(const KShapeGesture& gest, ShortcutTypes type = ShortcutTypes(ActiveShortcut | DefaultShortcut));
     void setRockerGesture(const KRockerGesture& gest, ShortcutTypes type = ShortcutTypes(ActiveShortcut | DefaultShortcut));
 
+    /**
+     * @reimp
+     */
+    bool event(QEvent*);
+
+
 Q_SIGNALS:
 #ifdef KDE3_SUPPORT
     /**
@@ -402,11 +478,20 @@ Q_SIGNALS:
      */
     void triggered(Qt::MouseButtons buttons, Qt::KeyboardModifiers modifiers);
 
+    /**
+     * Emitted when the global shortcut is changed. A global shortcut is
+     * subject to be changed by the global shortcuts kcm.
+     */
+    void globalShortcutChanged(const QKeySequence&);
+
 private:
-    friend class KGlobalAccelPrivate;
+    friend class KGlobalAccelPrivate; // Needs access to the component
+    friend class KActionCollectionPrivate; // Needs access to the component
+    friend class KShortcutsEditorDelegate; // Needs access to the component
     Q_PRIVATE_SLOT(d, void slotTriggered())
     class KActionPrivate* const d;
     friend class KActionPrivate;
+    friend class KGlobalShortcutTest;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(KAction::ShortcutTypes)
