@@ -123,10 +123,13 @@ void KColorButton::setDefaultColor( const QColor &c )
 
 void KColorButton::KColorButtonPrivate::initStyleOption(QStyleOptionButton* opt) const
 {
-    opt->init(q);
+    opt->initFrom(q);
+    opt->state |= q->isDown() ? QStyle::State_Sunken : QStyle::State_Raised;
+    opt->features = QStyleOptionButton::None;
+    if (q->isDefault())
+      opt->features |= QStyleOptionButton::DefaultButton;
     opt->text.clear();
     opt->icon = QIcon();
-    opt->features = QStyleOptionButton::None;
 }
 
 void KColorButton::paintEvent( QPaintEvent* )
@@ -235,7 +238,10 @@ void KColorButton::KColorButtonPrivate::_k_chooseColor()
   if ( m_bdefaultColor )
   {
       if( KColorDialog::getColor( c, m_defaultColor, q ) != QDialog::Rejected ) {
-          q->setColor( c );
+          if ( c.isValid() )
+              q->setColor( c );
+          else
+              q->setColor( m_defaultColor );
       }
   }
   else
