@@ -40,10 +40,7 @@ KAboutKdeDialog::KAboutKdeDialog(QWidget *parent)
     d( 0 )
 {
     setPlainCaption(i18n("About KDE"));
-    setButtons(KDialog::Help | KDialog::Close);
-    setDefaultButton(KDialog::Close);
-    setHelp("khelpcenter/main.html");
-    setModal(false);
+    setButtons(KDialog::Close);
 
     KTitleWidget *titleWidget = new KTitleWidget(this);
     titleWidget->setText(i18n("<html><font size=\"5\">K Desktop Environment</font><br /><b>Version %1</b></html>",
@@ -120,7 +117,7 @@ KAboutKdeDialog::KAboutKdeDialog(QWidget *parent)
         "contributing to KDE. You are encouraged to support KDE through a financial "
         "donation, using one of the ways described at "
         "<a href=\"http://www.kde.org/support/\">http://www.kde.org/support/</a>."
-        "<br /><br />Thank you very much in advance for your support.</html>") + "<br />"); // FIXME: ugly <br /> at the end...
+        "<br /><br />Thank you very much in advance for your support.</html>") + "<br /><br />"); // FIXME: ugly <br /> at the end...
 
     QTabWidget *tabWidget = new QTabWidget;
     tabWidget->setUsesScrollButtons(false);
@@ -130,10 +127,6 @@ KAboutKdeDialog::KAboutKdeDialog(QWidget *parent)
     tabWidget->addTab(support, i18n("&Support KDE"));
 
     QLabel *image = new QLabel;
-    image->setFrameShape(QFrame::StyledPanel);
-    image->setFrameShadow(QFrame::Plain);
-    image->setAutoFillBackground(true);
-    image->setBackgroundRole(QPalette::Base);
     image->setPixmap(KStandardDirs::locate("data", "kdeui/pics/aboutkde.png"));
 
     QHBoxLayout *midLayout = new QHBoxLayout;
