@@ -50,7 +50,7 @@ class KConfigDialogManager;
  *   if(KConfigDialog::showDialog("settings"))
  *     return;
  *   KConfigDialog *dialog = new KConfigDialog(this, "settings", MySettings::self());
- *   dialog->setFaceType(KPageDialog::IconList);
+ *   dialog->setFaceType(KPageDialog::List);
  *   dialog->addPage(new General(0, "General"), i18n("General") );
  *   dialog->addPage(new Appearance(0, "Style"), i18n("Appearance") );
  *   connect(dialog, SIGNAL(settingsChanged(const QString&)), mainWidget, SLOT(loadSettings()));
@@ -162,6 +162,7 @@ public:
    */
   static bool showDialog( const QString& name );
 
+
 protected Q_SLOTS:
   /**
    * Update the settings from the dialog.
@@ -188,11 +189,27 @@ protected Q_SLOTS:
    */
   virtual void updateWidgetsDefault();
 
+
+  /**
+   * Updates the Apply and Default buttons.
+   * Connect to this slot if you implement you own hasChanged()
+   * or isDefault() methods for widgets not managed by KConfig.
+   * @since 4.3
+   */
+  void updateButtons();
+
+  /**
+   * Some setting was changed. Emit the signal with the dialogs name.
+   * Connect to this slot if there are widgets not managed by KConfig.
+   * @since 4.3
+   */
+  void settingsChangedSlot();
+
 protected:
 
   /**
    * Returns whether the current state of the dialog is
-   * different from the current configutation.
+   * different from the current configuration.
    * Virtual function for custom additions.
    */
   virtual bool hasChanged();
