@@ -28,7 +28,7 @@
 #include <QMetaProperty>
 #include <QTimer>
 #include <QRadioButton>
-#include <QButtonGroup>
+//#include <QButtonGroup>
 
 #include <kconfigskeleton.h>
 #include <kdebug.h>
@@ -95,7 +95,7 @@ void KConfigDialogManager::initMaps()
     s_changedMap->insert("QRadioButton", SIGNAL(toggled(bool)));
     // We can only store one thing, so you can't have
     // a ButtonGroup that is checkable.
-    s_changedMap->insert("QButtonGroup", SIGNAL(buttonClicked(int)));
+//    s_changedMap->insert("QButtonGroup", SIGNAL(buttonClicked(int)));
     s_changedMap->insert("QGroupBox", SIGNAL(toggled(bool)));
     s_changedMap->insert("QComboBox", SIGNAL(activated (int)));
     //qsqlproperty map doesn't store the text, but the value!
@@ -196,7 +196,16 @@ void KConfigDialogManager::setupWidget(QWidget *widget, KConfigSkeletonItem *ite
       widget->setWhatsThis(whatsThis );
     }
   }
-  
+
+  if (widget->toolTip().isEmpty())
+  {
+    QString toolTip = item->toolTip();
+    if ( !toolTip.isEmpty() )
+    {
+      widget->setToolTip(toolTip);
+    }
+  }
+
   if(!item->isEqual( property(widget) ))
     setProperty( widget, item->property() );
 }
@@ -231,21 +240,21 @@ bool KConfigDialogManager::parseChildren(const QWidget *widget, bool trackChange
         setupWidget(childWidget, item);
 
         if ( d->trackChanges ) {
-	  QHash<QString, QByteArray>::const_iterator changedIt = s_changedMap->find(childWidget->metaObject()->className());
+          QHash<QString, QByteArray>::const_iterator changedIt = s_changedMap->constFind(childWidget->metaObject()->className());
 
-          if (changedIt == s_changedMap->end())
+          if (changedIt == s_changedMap->constEnd())
           {
 		   // If the class name of the widget wasn't in the monitored widgets map, then look for
 		   // it again using the super class name. This fixes a problem with using QtRuby/Korundum
 		   // widgets with KConfigXT where 'Qt::Widget' wasn't being seen a the real deal, even
 		   // though it was a 'QWidget'.
             if ( childWidget->metaObject()->superClass() )
-              changedIt = s_changedMap->find(childWidget->metaObject()->superClass()->className());
+              changedIt = s_changedMap->constFind(childWidget->metaObject()->superClass()->className());
             else
-              changedIt = s_changedMap->find(0);
+              changedIt = s_changedMap->constFind(0);
           }
 
-          if (changedIt == s_changedMap->end())
+          if (changedIt == s_changedMap->constEnd())
           {
             kWarning(178) << "Don't know how to monitor widget '" << childWidget->metaObject()->className() << "' for changes!";
           }
@@ -288,8 +297,8 @@ bool KConfigDialogManager::parseChildren(const QWidget *widget, bool trackChange
 #ifndef NDEBUG
     else if (!widgetName.isEmpty() && d->trackChanges)
     {
-      QHash<QString, QByteArray>::const_iterator changedIt = s_changedMap->find(childWidget->metaObject()->className());
-      if (changedIt != s_changedMap->end())
+      QHash<QString, QByteArray>::const_iterator changedIt = s_changedMap->constFind(childWidget->metaObject()->className());
+      if (changedIt != s_changedMap->constEnd())
       {
         if ((!d->insideGroupBox || !qobject_cast<QRadioButton*>(childWidget)) &&
             !qobject_cast<QGroupBox*>(childWidget) &&!qobject_cast<QTabWidget*>(childWidget) )
@@ -337,9 +346,7 @@ void KConfigDialogManager::updateWidgets()
      if (item->isImmutable())
      {
         widget->setEnabled(false);
-        QWidget *buddy = 0;
-        if(d->buddyWidget.contains(it.key()))
-           buddy = d->buddyWidget.value(it.key());
+        QWidget *buddy = d->buddyWidget.value(it.key(), 0);
         if (buddy)
            buddy->setEnabled(false);
      }
@@ -403,59 +410,80 @@ QByteArray KConfigDialogManager::getUserProperty(const QWidget *widget) const
   return s_propertyMap->value( widget->metaObject()->className() );
 }
 
+QByteArray KConfigDialogManager::getCustomProperty(const QWidget *widget) const
+{
+    QVariant prop(widget->property("kcfg_property"));
+    if (prop.isValid()) {
+        if (!prop.canConvert(QVariant::ByteArray)) {
+            kWarning(178) << "kcfg_property on" << widget->metaObject()->className()
+                          << "is not of type ByteArray";
+        } else {
+            return prop.toByteArray();
+        }
+    }
+    return QByteArray();
+}
+
 void KConfigDialogManager::setProperty(QWidget *w, const QVariant &v)
 {
-  QButtonGroup *bg = qobject_cast<QButtonGroup *>(w);
+/*  QButtonGroup *bg = qobject_cast<QButtonGroup *>(w);
   if (bg)
   {
     QAbstractButton *b = bg->button(v.toInt());
     if (b)
         b->setDown(true);
     return;
-  }
+  }*/
 
-  QByteArray userproperty = getUserProperty( w );
-  if ( userproperty.isEmpty() ) {
-    QComboBox *cb = qobject_cast<QComboBox *>(w);
-    if (cb) {
-        if ( cb->isEditable() )
-        {
-            int i = cb->findText(v.toString());
-            if (i != -1)
-                cb->setCurrentIndex(i);
-            else
-                cb->setEditText(v.toString());
-        } else {
-            cb->setCurrentIndex( v.toInt() );
+    QByteArray userproperty = getCustomProperty(w);
+    if (userproperty.isEmpty()) {
+        userproperty = getUserProperty(w);
+    }
+    if (userproperty.isEmpty()) {
+        QComboBox *cb = qobject_cast<QComboBox *>(w);
+        if (cb) {
+            if (cb->isEditable()) {
+                int i = cb->findText(v.toString());
+                if (i != -1) {
+                    cb->setCurrentIndex(i);
+                } else {
+                    cb->setEditText(v.toString());
+                }
+            } else {
+                cb->setCurrentIndex(v.toInt());
+            }
+            return;
         }
+        kWarning(178) << w->metaObject()->className() << " widget not handled!";
         return;
     }
-    kWarning(178) << w->metaObject()->className() << " widget not handled!";
-    return;
-  }
-  w->setProperty( userproperty, v );
+    w->setProperty(userproperty, v);
 }
 
 QVariant KConfigDialogManager::property(QWidget *w) const
 {
-  QButtonGroup *bg = qobject_cast<QButtonGroup *>(w);
+/*  QButtonGroup *bg = qobject_cast<QButtonGroup *>(w);
   if (bg && bg->checkedButton())
-    return QVariant(bg->id(bg->checkedButton()));
+    return QVariant(bg->id(bg->checkedButton()));*/
 
-  QByteArray userproperty = getUserProperty( w );
-  if ( userproperty.isEmpty() ) {
-    QComboBox *cb = qobject_cast<QComboBox *>(w);
-    if (cb) {
-        if ( cb->isEditable() )
-            return QVariant(cb->currentText());
-        else
-            return QVariant(cb->currentIndex());
+    QByteArray userproperty = getCustomProperty(w);
+    if (userproperty.isEmpty()) {
+        userproperty = getUserProperty(w);
+    }
+    if (userproperty.isEmpty()) {
+        QComboBox *cb = qobject_cast<QComboBox *>(w);
+        if (cb) {
+            if (cb->isEditable()) {
+                return QVariant(cb->currentText());
+            } else {
+                return QVariant(cb->currentIndex());
+            }
+        }
+        kWarning(178) << w->metaObject()->className() << " widget not handled!";
+        return QVariant();
     }
-    kWarning(178) << w->metaObject()->className() << " widget not handled!";
-    return QVariant();
-  }
 
-  return w->property( userproperty );
+    return w->property(userproperty);
 }
 
 bool KConfigDialogManager::hasChanged() const
