@@ -26,6 +26,7 @@
 #include <QtGui/QLayout>
 #include <QtGui/QListWidget>
 #include <QtGui/QScrollArea>
+#include <QtGui/QScrollBar>
 #include <QtGui/QTextDocumentFragment>
 
 #include <kapplication.h>
@@ -38,12 +39,8 @@
 #include <kiconloader.h>
 #include <kconfiggroup.h>
 #include <ktextedit.h>
-
-#ifdef Q_WS_X11
-#include <qx11info_x11.h>
-#include <X11/Xlib.h>
-#include <fixx11h.h>
-#endif
+#include <ksqueezedtextlabel.h>
+#include <kwindowsystem.h>
 
 // Some i18n filters, that standard button texts are piped through
 // (the new KGuiItem object with filtered text is created from the old one).
@@ -154,13 +151,6 @@ int KMessageBox::createKMessageBox(KDialog *dialog, QMessageBox::Icon icon,
                       ask, checkboxReturn, options, details, icon);
 }
 
-static int longest_line( const QFontMetrics & fm, const QString & text ) {
-    const QStringList lines = QTextDocumentFragment::fromHtml( text ).toPlainText().split( QLatin1String( "\n" ) );
-    int len = 0;
-    Q_FOREACH( const QString & line, lines )
-        len = qMax( len, fm.width( line ) );
-    return len;
-}
 
 int KMessageBox::createKMessageBox(KDialog *dialog, const QIcon &icon,
                              const QString &text, const QStringList &strlist,
@@ -169,14 +159,13 @@ int KMessageBox::createKMessageBox(KDialog *dialog, const QIcon &icon,
 {
     QWidget *mainWidget = new QWidget(dialog);
     QVBoxLayout *mainLayout = new QVBoxLayout(mainWidget);
-    mainLayout->setSpacing(KDialog::spacingHint() * 2);
+    mainLayout->setSpacing(KDialog::spacingHint() * 2); // provide extra spacing
     mainLayout->setMargin(0);
 
     QHBoxLayout *hLayout = new QHBoxLayout();
     hLayout->setMargin(0);
-    hLayout->setSpacing(KDialog::spacingHint());
-    mainLayout->addLayout(hLayout);
-    mainLayout->addStretch();
+    hLayout->setSpacing(-1); // use default spacing
+    mainLayout->addLayout(hLayout,5);
 
     QLabel *iconLabel = new QLabel(mainWidget);
 
@@ -189,10 +178,24 @@ int KMessageBox::createKMessageBox(KDialog *dialog, const QIcon &icon,
     iconLayout->addWidget(iconLabel);
     iconLayout->addStretch(5);
 
-    hLayout->addLayout(iconLayout);
+    hLayout->addLayout(iconLayout,0);
     hLayout->addSpacing(KDialog::spacingHint());
 
     QLabel *messageLabel = new QLabel(text, mainWidget);
+    QRect desktop = KGlobalSettings::desktopGeometry(dialog);
+    bool usingSqueezedTextLabel = false;
+    if (messageLabel->sizeHint().width() > desktop.width() * 0.5) {
+        // enable automatic wrapping of messages which are longer than 50% of screen width
+        messageLabel->setWordWrap(true);
+        // display a text widget with scrollbar if still too wide
+        usingSqueezedTextLabel = messageLabel->sizeHint().width() > desktop.width() * 0.85;
+        if (usingSqueezedTextLabel)
+        {
+            delete messageLabel;
+            messageLabel = new KSqueezedTextLabel(text, mainWidget);
+        }
+    }
+
     messageLabel->setOpenExternalLinks(options & KMessageBox::AllowLink );
     Qt::TextInteractionFlags flags = Qt::TextSelectableByMouse | Qt::TextSelectableByKeyboard;
     if ( options & KMessageBox::AllowLink )
@@ -202,32 +205,51 @@ int KMessageBox::createKMessageBox(KDialog *dialog, const QIcon &icon,
     messagePal.setColor(QPalette::Window, Qt::transparent);
     messageLabel->setPalette(messagePal);
 
-    QRect desktop = KGlobalSettings::desktopGeometry(dialog);
-    if (desktop.width() / 3 < longest_line(messageLabel->fontMetrics(), text)) {
-        // do only enable automatic wrapping of messages which are longer than one third of the current screen
-        messageLabel->setWordWrap(true);
+
+    bool usingScrollArea=desktop.height() / 3 < messageLabel->sizeHint().height();
+    if (usingScrollArea)
+    {
+        QScrollArea* messageScrollArea = new QScrollArea(mainWidget);
+        messageScrollArea->setWidget(messageLabel);
+        messageScrollArea->setFrameShape(QFrame::NoFrame);
+        messageScrollArea->setWidgetResizable(true);
+        QPalette scrollPal(messageScrollArea->palette());
+        scrollPal.setColor(QPalette::Window, Qt::transparent);
+        messageScrollArea->viewport()->setPalette(scrollPal);
+        hLayout->addWidget(messageScrollArea,5);
     }
+    else
+        hLayout->addWidget(messageLabel,5);
 
-    QScrollArea* messageScrollArea = new QScrollArea(mainWidget);
-    messageScrollArea->setWidget(messageLabel);
-    messageScrollArea->setFrameShape(QFrame::NoFrame);
-    messageScrollArea->setWidgetResizable(true);
-    QPalette scrollPal(messageScrollArea->palette());
-    scrollPal.setColor(QPalette::Window, Qt::transparent);
-    messageScrollArea->setPalette(scrollPal);
 
-    hLayout->addWidget(messageScrollArea);
-
-    QListWidget *listWidget = 0;
-    if (!strlist.isEmpty()) {
+    const bool usingListWidget=!strlist.isEmpty();
+    if (usingListWidget) {
         // enable automatic wrapping since the listwidget has already a good initial width
         messageLabel->setWordWrap(true);
-        listWidget = new QListWidget(mainWidget);
-        mainLayout->addWidget(listWidget);
+        QListWidget *listWidget = new QListWidget(mainWidget);
         listWidget->addItems(strlist);
+
+        QStyleOptionViewItem styleOption;
+        styleOption.initFrom(listWidget);
+        QFontMetrics fm(styleOption.font);
+        int w = listWidget->width();
+        Q_FOREACH(const QString &str, strlist) {
+            w = qMax(w, fm.width(str));
+        }
+        const int borderWidth = listWidget->width() - listWidget->viewport()->width() + listWidget->verticalScrollBar()->height();
+        w += borderWidth;
+        if (w > desktop.width() * 0.85) { // limit listWidget size to 85% of screen width
+            w = qRound(desktop.width() * 0.85);
+        }
+        listWidget->setMinimumWidth(w);
+
+        mainLayout->addWidget(listWidget,usingScrollArea?10:50);
         listWidget->setSelectionMode(QListWidget::NoSelection);
-        mainLayout->setStretchFactor(listWidget, 1);
+        messageLabel->setSizePolicy(QSizePolicy::Preferred,QSizePolicy::Minimum);
     }
+    else if (!usingScrollArea)
+        mainLayout->addStretch(15);
+
 
     QPointer<QCheckBox> checkbox = 0;
     if (!ask.isEmpty()) {
@@ -249,27 +271,31 @@ int KMessageBox::createKMessageBox(KDialog *dialog, const QIcon &icon,
                 flags |= Qt::LinksAccessibleByMouse | Qt::LinksAccessibleByKeyboard;;
             detailsLabel->setTextInteractionFlags(flags);
             detailsLabel->setWordWrap(true);
-            detailsLayout->addWidget(detailsLabel);
+            detailsLayout->addWidget(detailsLabel,50);
         } else {
             KTextEdit *detailTextEdit = new KTextEdit(details);
             detailTextEdit->setReadOnly(true);
             detailTextEdit->setMinimumHeight(detailTextEdit->fontMetrics().lineSpacing() * 11);
-            detailsLayout->addWidget(detailTextEdit);
+            detailsLayout->addWidget(detailTextEdit,50);
         }
+        if (!usingListWidget)
+            mainLayout->setStretchFactor(hLayout,10);
         dialog->setDetailsWidget(detailsGroup);
     }
 
     dialog->setMainWidget(mainWidget);
     dialog->showButtonSeparator(true);
-    if (!listWidget) {
-        int hfw = messageLabel->heightForWidth(messageScrollArea->sizeHint().width() - 2);
-        if (hfw != messageScrollArea->sizeHint().height() && hfw < desktop.height() / 2) {
-            messageScrollArea->setMinimumHeight(hfw);
-        }
-        if (details.isEmpty())
-            dialog->setFixedSize(dialog->sizeHint() + QSize( 10, 10 ));
+    if (!usingListWidget && !usingScrollArea && !usingSqueezedTextLabel && details.isEmpty())
+        dialog->setFixedSize(dialog->sizeHint() + QSize( 10, 10 ));
+    else if (!details.isEmpty() && dialog->minimumHeight()<iconLabel->sizeHint().height()*2)//strange bug...
+    {
+        if (!usingScrollArea)
+            dialog->setMinimumSize(300,qMax(150,qMax(iconLabel->sizeHint().height(),messageLabel->sizeHint().height())));
+        else
+            dialog->setMinimumSize(300,qMax(150,iconLabel->sizeHint().height()));
     }
 
+
     if ((options & KMessageBox::Dangerous)) {
         if (dialog->isButtonEnabled(KDialog::Cancel))
             dialog->setDefaultButton(KDialog::Cancel);
@@ -302,7 +328,7 @@ int KMessageBox::createKMessageBox(KDialog *dialog, const QIcon &icon,
     // In that case the QPointer will reset to 0.
     QPointer<KDialog> guardedDialog = dialog;
 
-    int result = guardedDialog->exec();
+    const int result = guardedDialog->exec();
     if (checkbox && checkboxReturn) {
         *checkboxReturn = checkbox->isChecked();
     }
@@ -340,7 +366,7 @@ bool KMessageBox::shouldBeShownYesNo(const QString &dontShowAgainName,
         return true;
     }
     KConfigGroup cg( KMessageBox_againConfig ? KMessageBox_againConfig : KGlobal::config().data(), "Notification Messages" );
-    QString dontAsk = cg.readEntry(dontShowAgainName, QString()).toLower();
+    const QString dontAsk = cg.readEntry(dontShowAgainName, QString()).toLower();
     if (dontAsk == "yes" || dontAsk == "true") {
         result = Yes;
         return false;
@@ -438,14 +464,12 @@ int KMessageBox::questionYesNoListWId(WId parent_id, const QString &text,
     if ( options & PlainCaption ) {
         dialog->setPlainCaption( caption );
     }
-#ifdef Q_WS_X11
     if ( parent == NULL && parent_id ) {
-        XSetTransientForHint( QX11Info::display(), dialog->winId(), parent_id );
+        KWindowSystem::setMainWindow( dialog, parent_id );
     }
-#endif
 
     bool checkboxResult = false;
-    int result = createKMessageBox(dialog, QMessageBox::Information, text, strlist,
+    const int result = createKMessageBox(dialog, QMessageBox::Information, text, strlist,
                        dontAskAgainName.isEmpty() ? QString() : i18n("Do not ask again"),
                        &checkboxResult, options);
     res = (result==KDialog::Yes ? Yes : No);
@@ -502,14 +526,12 @@ int KMessageBox::questionYesNoCancelWId(WId parent_id,
     if ( options & PlainCaption ) {
         dialog->setPlainCaption( caption );
     }
-#ifdef Q_WS_X11
     if ( parent == NULL && parent_id ) {
-        XSetTransientForHint( QX11Info::display(), dialog->winId(), parent_id );
+        KWindowSystem::setMainWindow( dialog, parent_id );
     }
-#endif
 
     bool checkboxResult = false;
-    int result = createKMessageBox(dialog, QMessageBox::Information,
+    const int result = createKMessageBox(dialog, QMessageBox::Information,
                        text, QStringList(),
                        dontAskAgainName.isEmpty() ? QString() : i18n("Do not ask again"),
                        &checkboxResult, options);
@@ -593,14 +615,12 @@ int KMessageBox::warningYesNoListWId(WId parent_id, const QString &text,
     if ( options & PlainCaption ) {
         dialog->setPlainCaption( caption );
     }
-#ifdef Q_WS_X11
     if ( parent == NULL && parent_id ) {
-        XSetTransientForHint( QX11Info::display(), dialog->winId(), parent_id );
+        KWindowSystem::setMainWindow( dialog, parent_id );
     }
-#endif
 
     bool checkboxResult = false;
-    int result = createKMessageBox(dialog, QMessageBox::Warning, text, strlist,
+    const int result = createKMessageBox(dialog, QMessageBox::Warning, text, strlist,
                        dontAskAgainName.isEmpty() ? QString() : i18n("Do not ask again"),
                        &checkboxResult, options);
     res = (result==KDialog::Yes ? Yes : No);
@@ -676,14 +696,12 @@ int KMessageBox::warningContinueCancelListWId(WId parent_id, const QString &text
     if ( options & PlainCaption ) {
         dialog->setPlainCaption( caption );
     }
-#ifdef Q_WS_X11
     if ( parent == NULL && parent_id ) {
-        XSetTransientForHint( QX11Info::display(), dialog->winId(), parent_id );
+        KWindowSystem::setMainWindow( dialog, parent_id );
     }
-#endif
 
     bool checkboxResult = false;
-    int result = createKMessageBox(dialog, QMessageBox::Warning, text, strlist,
+    const int result = createKMessageBox(dialog, QMessageBox::Warning, text, strlist,
                        dontAskAgainName.isEmpty() ? QString() : i18n("Do not ask again"),
                        &checkboxResult, options);
 
@@ -766,14 +784,12 @@ int KMessageBox::warningYesNoCancelListWId(WId parent_id, const QString &text,
     if ( options & PlainCaption ) {
         dialog->setPlainCaption( caption );
     }
-#ifdef Q_WS_X11
     if ( parent == NULL && parent_id ) {
-        XSetTransientForHint( QX11Info::display(), dialog->winId(), parent_id );
+        KWindowSystem::setMainWindow( dialog, parent_id );
     }
-#endif
 
     bool checkboxResult = false;
-    int result = createKMessageBox(dialog, QMessageBox::Warning, text, strlist,
+    const int result = createKMessageBox(dialog, QMessageBox::Warning, text, strlist,
                        dontAskAgainName.isEmpty() ? QString() : i18n("Do not ask again"),
                        &checkboxResult, options);
 
@@ -826,11 +842,9 @@ void KMessageBox::errorListWId(WId parent_id,  const QString &text, const QStrin
     if ( options & PlainCaption ) {
         dialog->setPlainCaption( caption );
     }
-#ifdef Q_WS_X11
     if ( parent == NULL && parent_id ) {
-        XSetTransientForHint( QX11Info::display(), dialog->winId(), parent_id );
+        KWindowSystem::setMainWindow( dialog, parent_id );
     }
-#endif
 
     createKMessageBox(dialog, QMessageBox::Critical, text, strlist, QString(), 0, options);
 }
@@ -860,11 +874,9 @@ void KMessageBox::detailedErrorWId(WId parent_id,  const QString &text,
     if( options & PlainCaption ) {
         dialog->setPlainCaption( caption );
     }
-#ifdef Q_WS_X11
     if ( parent == NULL && parent_id ) {
-        XSetTransientForHint( QX11Info::display(), dialog->winId(), parent_id );
+        KWindowSystem::setMainWindow( dialog, parent_id );
     }
-#endif
 
     createKMessageBox(dialog, QMessageBox::Critical, text, QStringList(), QString(), 0, options, details);
 }
@@ -908,11 +920,9 @@ void KMessageBox::sorryWId(WId parent_id, const QString &text,
     if ( options & PlainCaption ) {
         dialog->setPlainCaption( caption );
     }
-#ifdef Q_WS_X11
     if ( parent == NULL && parent_id ) {
-        XSetTransientForHint( QX11Info::display(), dialog->winId(), parent_id );
+        KWindowSystem::setMainWindow( dialog, parent_id );
     }
-#endif
 
     createKMessageBox(dialog, QMessageBox::Warning, text, QStringList(), QString(), 0, options);
 }
@@ -941,11 +951,9 @@ void KMessageBox::detailedSorryWId(WId parent_id, const QString &text,
     if ( options & PlainCaption ) {
         dialog->setPlainCaption( caption );
     }
-#ifdef Q_WS_X11
     if ( parent == NULL && parent_id ) {
-        XSetTransientForHint( QX11Info::display(), dialog->winId(), parent_id );
+        KWindowSystem::setMainWindow( dialog, parent_id );
     }
-#endif
 
     createKMessageBox(dialog, QMessageBox::Warning, text, QStringList(), QString(), 0, options, details);
 }
@@ -989,11 +997,9 @@ void KMessageBox::informationListWId(WId parent_id,const QString &text, const QS
     if ( options & PlainCaption ) {
         dialog->setPlainCaption( caption );
     }
-#ifdef Q_WS_X11
     if ( parent == NULL && parent_id ) {
-        XSetTransientForHint( QX11Info::display(), dialog->winId(), parent_id );
+        KWindowSystem::setMainWindow( dialog, parent_id );
     }
-#endif
 
     bool checkboxResult = false;
 
