@@ -4,6 +4,7 @@
     Copyright (C) 2006 Hamish Rodda <rodda@kde.org>
     Copyright (C) 2007 Roberto Raggi <roberto@kdevelop.org>
     Copyright (C) 2007 Andreas Hartmetz <ahartmetz@gmail.com>
+    Copyright (C) 2008 Michael Jansen <kde@michael-jansen.biz>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -28,6 +29,9 @@
 #include "kgesture.h"
 
 class KActionCollection;
+class KConfig;
+class KConfigBase;
+class KConfigGroup;
 class KGlobalAccel;
 class KShortcut;
 class KShortcutsEditorPrivate;
@@ -51,6 +55,7 @@ class KShortcutsEditorPrivate;
  * @see KShortcutsDialog
  * @author Nicolas Hadacek <hadacek@via.ecp.fr>
  * @author Hamish Rodda <rodda@kde.org> (KDE 4 porting)
+ * @author Michael Jansen <kde@michael-jansen.biz>
  */
 class KDEUI_EXPORT KShortcutsEditor : public QWidget
 {
@@ -107,6 +112,11 @@ public:
 	/// Destructor
 	virtual ~KShortcutsEditor();
 
+    /**
+     * Are the unsaved changes?
+     */
+    bool isModified() const;
+
 	/**
 	 * Removes all action collections from the editor
 	 */
@@ -119,51 +129,75 @@ public:
 	 */
 	void addCollection(KActionCollection *, const QString &title = QString());
 
-	/**
-	 * This function writes any shortcut changes back to the original
-	 * action set(s).
-	 */
-	//void commitChanges(); //we do it the other way around
 
 	/**
-	 * This function reverts any shortcut changes to the original
-	 * action set(s). Not implemented yet.
+     * Undo all change made since the last commit().
 	 */
 	void undoChanges();
 
-	/**
-	 * This saves the actions to disk.
-	 * Any KActionCollection objects with the xmlFile() value set
-	 * will be written to an XML file.  All other will be written
-	 * to the application's rc file.
-	 */
-	void save();
+    /**
+     * Save the changes.
+     *
+     * Before saving the changes are committed. This saves the actions to disk.
+     * Any KActionCollection objects with the xmlFile() value set will be
+     * written to an XML file.  All other will be written to the application's
+     * rc file.
+     */
+    void save();
 
-	/**
-	 * Checks whether the given shortcut conflicts with global keyboard shortcuts.
-	 * If yes, and the warnUser argument is true, warns the user and gives them a chance
-	 * to reassign the shortcut from the global shortcut.
-	 *
-	 * @return true if there was conflict (and the user didn't reassign the shortcut)
-	 * @param cut the shortcut that will be checked for conflicts
-	 * @param warnUser if true, the user will be warned about a conflict and given a chance
-	 *        to reassign the shortcut
-	 * @param parent parent widget for the warning dialog
-	 */
-	static bool checkGlobalShortcutsConflict( const KShortcut& cut, bool warnUser, QWidget* parent );
+    /**
+     * Commit the changes without saving.
+     *
+     * This commits the changes without saving.
+     *
+     * @since 4.2
+     */
+    void commit();
+
+
+    /**
+     * Removes all configured shortcuts.
+     */
+    void clearConfiguration();
+
+    /**
+     * Write the current settings to the \p config object.
+     *
+     * This does not initialize the \p config object. It adds the
+     * configuration. 
+     *
+     * @note this will not save the global configuration! globalaccel holds
+     * that part of the configuration.
+     * @see writeGlobalConfig()
+     *
+     * @param config Config object to save to or, or null to use the
+     *               applications config object
+     *
+     */
+    void writeConfiguration( KConfigGroup* config = 0 ) const;
+
+    /**
+     * Export the current setting to configuration @p config.
+     *
+     * This initializes the configuration object. This will export the global
+     * configuration too.
+     *
+     * @param config Config object
+     */
+    KDE_DEPRECATED void exportConfiguration( KConfig *config) const;
+    void exportConfiguration( KConfigBase *config) const;
+
+    /**
+     * Import the settings from configuration @p config.
+     *
+     * This will remove all current setting before importing. All shortcuts
+     * are set to KShortcut() prior to importing from @p config!
+     *
+     * @param config Config object
+     */
+    KDE_DEPRECATED void importConfiguration( KConfig *config);
+    void importConfiguration( KConfigBase *config);
 
-	/**
-	 * Checks whether the given shortcut conflicts with standard keyboard shortcuts.
-	 * If yes, and the warnUser argument is true, warns the user and gives them a chance
-	 * to reassign the shortcut from the standard shortcut.
-	 *
-	 * @return true if there was conflict (and the user didn't reassign the shortcut)
-	 * @param cut the shortcut that will be checked for conflicts
-	 * @param warnUser if true, the user will be warned about a conflict and given a chance
-	 *        to reassign the shortcut
-	 * @param parent parent widget for the warning dialog
-	 */
-	static bool checkStandardShortcutsConflict( const KShortcut& cut, bool warnUser, QWidget* parent );
 
 Q_SIGNALS:
 	/**
@@ -181,12 +215,15 @@ public Q_SLOTS:
 	 * Set all shortcuts to their default values (bindings).
 	 **/
 	void allDefault();
+        
+        /**
+         * Opens a printing dialog to print all the shortcuts
+         */
+        void printShortcuts() const;
 
 private:
 	Q_PRIVATE_SLOT(d, void capturedShortcut(QVariant, const QModelIndex &))
 
-	Q_PRIVATE_SLOT(d, void globalSettingsChangedSystemwide(int))
-
 private:
 	friend class KShortcutsDialog;
 	friend class KShortcutsEditorPrivate;
