@@ -30,15 +30,26 @@ class KGlobalShortcutTest : public QObject
 {
     Q_OBJECT
 private Q_SLOTS:
-    void initTestCase();
     void testSetShortcut();
     void testFindActionByKey();
     void testChangeShortcut();
     void testStealShortcut();
     void testSaveRestore();
     void testListActions();
-    void cleanupTestCase();
+    void testComponentAssignment();
+    void testConfigurationActions();
+    // This has to be the last before forgetGlobalShortcut
+    void testOverrideMainComponentData();
+    void testForgetGlobalShortcut();
+
+public:
+
+    KGlobalShortcutTest() : m_actionA(0) ,m_actionB(0)
+        {}
+
 private:
+    void setupTest(QString id);
+
     KAction *m_actionA;
     KAction *m_actionB;
 };
