@@ -97,10 +97,11 @@ void KCModule::showEvent(QShowEvent *ev)
 {
     if (d->_firstshow) {
         d->_firstshow = false;
-        load();
+        QMetaObject::invokeMethod(this, "load", Qt::QueuedConnection);
+        QMetaObject::invokeMethod(this, "changed", Qt::QueuedConnection, Q_ARG(bool, false));
     }
+
     QWidget::showEvent(ev);
-    emit( changed( false ));
 }
 
 KCModule::Buttons KCModule::buttons() const
