@@ -144,7 +144,7 @@ public:
        * KLineEdit as child-widget for example, so the KComboBox is used as
        * the representation widget.
        *
-       * @see KUrlRequester::customEditor()
+       * @see KUrlRequester::customEditor(), setCustomEditor
        */
       KEditListBox( const QString& title,
                     const CustomEditor &customEditor,
@@ -179,15 +179,15 @@ public:
       QPushButton* downButton() const;
 
       /**
-       * See QListBox::count()
+       * See Q3ListBox::count()
        */
       int count() const;
       /**
-       * See QListBox::insertStringList()
+       * See Q3ListBox::insertStringList()
        */
       void insertStringList(const QStringList& list, int index=-1);
       /**
-       * See QListBox::insertStrList()
+       * See Q3ListBox::insertItem()
        */
       void insertItem(const QString& text, int index=-1);
       /**
@@ -195,15 +195,15 @@ public:
        */
       void clear();
       /**
-       * See QListBox::text()
+       * See Q3ListBox::text()
        */
       QString text(int index) const;
       /**
-       * See QListBox::currentItem()
+       * See Q3ListBox::currentItem()
        */
       int currentItem() const;
       /**
-       * See QListBox::currentText()
+       * See Q3ListBox::currentText()
        */
       QString currentText() const;
 
@@ -245,6 +245,24 @@ public:
        */
       bool checkAtEntering();
 
+      /**
+       * Allows to use a custom editing widget
+       * instead of the standard KLineEdit widget. E.g. you can use a
+       * KUrlRequester or a KComboBox as input widget. The custom
+       * editor must consist of a lineedit and optionally another widget that
+       * is used as representation. A KComboBox or a KUrlRequester have a
+       * KLineEdit as child-widget for example, so the KComboBox is used as
+       * the representation widget.
+       *
+       * @since 4.1
+       */
+      void setCustomEditor( const CustomEditor& editor );
+
+      /**
+       * Reimplented for interal reasons. The API is not affected.
+       */
+      bool eventFilter( QObject* o, QEvent* e );
+
    Q_SIGNALS:
       void changed();
 
@@ -260,7 +278,7 @@ public:
        */
       void removed( const QString & text );
 
-   protected Q_SLOTS:
+   protected Q_SLOTS: // KDE5: make private?
       void moveItemUp();
       void moveItemDown();
       void addItem();
@@ -268,10 +286,8 @@ public:
       void enableMoveButtons(const QModelIndex&, const QModelIndex&);
       void typedSomething(const QString& text);
 
-   private:
-      //this is called in both ctors, to avoid code duplication
-      void init( bool checkAtEntering = false, Buttons buttons = All,
-                 QWidget *representationWidget = 0 );
+   private Q_SLOTS:
+      void slotSelectionChanged( const QItemSelection& selected, const QItemSelection& deselected );
 
    private:
       friend class KEditListBoxPrivate;
