@@ -1,6 +1,7 @@
 /* This file is part of the KDE libraries
     Copyright (C) 2003 Stephan Binner <binner@kde.org>
     Copyright (C) 2003 Zack Rusin <zack@kde.org>
+    Copyright (C) 2009 Urs Wolfer <uwolfer @ kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -28,7 +29,7 @@
 /**
  * A QTabBar with extended features.
  */
-class KDEUI_EXPORT KTabBar: public QTabBar
+class KDEUI_EXPORT KTabBar: public QTabBar //krazy:exclude=qclasses
 {
     Q_OBJECT
 
@@ -49,32 +50,79 @@ class KDEUI_EXPORT KTabBar: public QTabBar
      * Sets the tab reordering enabled or disabled. If enabled,
      * the user can reorder the tabs by drag and drop the tab
      * headers.
+     *
+     * @deprecated Use QTabBar::setMovable() instead.
      */
-    void setTabReorderingEnabled( bool enable );
+    KDE_DEPRECATED void setTabReorderingEnabled( bool enable );
 
     /**
      * Returns whether tab reordering is enabled.
+     *
+     * @deprecated Use QTabBar::isMovable() instead.
      */
-    bool isTabReorderingEnabled() const;
+    KDE_DEPRECATED bool isTabReorderingEnabled() const;
 
-    void setHoverCloseButton( bool );
-    bool hoverCloseButton() const;
+    /**
+     * If enabled, a close button is shown above the tab icon.
+     * The signal KTabBar::closeRequest() is emitted, if the
+     * close button has been clicked. Note that the tab must have
+     * an icon to use this feature.
+     *
+     * @deprecated Use QTabBar::setTabsClosable() instead.
+     */
+    KDE_DEPRECATED void setHoverCloseButton( bool );
 
-    void setHoverCloseButtonDelayed( bool );
-    bool hoverCloseButtonDelayed() const;
+    /** @deprecated Use QTabBar::tabsClosable() instead. */
+    KDE_DEPRECATED bool hoverCloseButton() const;
+
+    /**
+     * If enabled, the close button cannot get clicked until a
+     * minor delay has been passed. This prevents that user
+     * accidentally closes a tab.
+     *
+     * @deprecated Use QTabBar::setTabsClosable() instead.
+     */
+    KDE_DEPRECATED void setHoverCloseButtonDelayed( bool );
+
+    /** @deprecated Use QTabBar::tabsClosable() instead. */
+    KDE_DEPRECATED bool hoverCloseButtonDelayed() const;
+
+    /**
+     * If enabled, a close button is available for each tab. The
+     * signal KTabBar::closeRequest() is emitted, if the close button
+     * has been clicked.
+     *
+     * @since 4.1
+     *
+     * @deprecated Use QTabBar::setTabsClosable() instead.
+     */
+    KDE_DEPRECATED void setCloseButtonEnabled( bool );
+
+    /**
+     * Returns true if the close button is shown on tabs.
+     *
+     * @since 4.1
+     *
+     * @deprecated Use QTabBar::tabsClosable() instead.
+     */
+    KDE_DEPRECATED bool isCloseButtonEnabled() const;
 
     /**
      * Sets the 'activate previous tab on close' feature enabled
      * or disabled. If enabled, as soon as you close a tab, the
      * previously selected tab is activated again.
+     *
+     * @deprecated Use QTabBar::setSelectionBehaviorOnRemove() instead.
      */
-    void setTabCloseActivatePrevious( bool );
+    KDE_DEPRECATED void setTabCloseActivatePrevious( bool );
 
     /**
      * Returns whether the 'activate previous tab on close' feature
      * is enabled.
+     *
+     * @deprecated Use QTabBar::selectionBehaviorOnRemove() instead.
      */
-    bool tabCloseActivatePrevious() const;
+    KDE_DEPRECATED bool tabCloseActivatePrevious() const;
 
     /**
      * Selects the tab which has a tab header at
@@ -100,8 +148,10 @@ class KDEUI_EXPORT KTabBar: public QTabBar
     void initiateDrag( int );
     void testCanDecode( const QDragMoveEvent*, bool& );
     void receivedDropEvent( int, QDropEvent* );
-    void moveTab( int, int );
-    void closeRequest( int );
+    /** @deprecated Use QTabBar::tabMoved(int,int) instead.*/
+    QT_MOC_COMPAT void moveTab( int, int );
+    /** @deprecated Use QTabBar::tabCloseRequested(int) instead. */
+    QT_MOC_COMPAT void closeRequest( int );
 #ifndef QT_NO_WHEELEVENT
     void wheelDelta( int );
 #endif
@@ -119,14 +169,24 @@ class KDEUI_EXPORT KTabBar: public QTabBar
     virtual void dragMoveEvent( QDragMoveEvent *event );
     virtual void dropEvent( QDropEvent *event );
 
+    virtual void paintEvent( QPaintEvent *event );
+    virtual void leaveEvent( QEvent *event );
+    virtual QSize tabSizeHint( int index ) const;
+
   protected Q_SLOTS:
-    void closeButtonClicked();
-    void enableCloseButton();
+    /** @deprecated */
+    QT_MOC_COMPAT void closeButtonClicked();
+    /** @deprecated */
+    QT_MOC_COMPAT void enableCloseButton();
     virtual void activateDragSwitchTab();
 
   protected:
     virtual void tabLayoutChange();
 
+  private:
+    QPoint closeButtonPos( int tabIndex ) const;
+    QRect closeButtonRect( int tabIndex ) const;
+
   private:
     class Private;
     Private* const d;
