@@ -75,7 +75,16 @@
 
 class KXmlGuiWindowPrivate : public KMainWindowPrivate {
 public:
+    void _k_slotFactoryMakingChanges(bool b)
+    {
+        // While the GUI factory is adding/removing clients,
+        // don't let KMainWindow think those are changes made by the user
+        // #105525
+        letDirtySettings = !b;
+    }
+
     bool showHelpMenu:1;
+    QSize defaultSize;
 
     KDEPrivate::ToolBarHandler *toolBarHandler;
     KToggleAction *showStatusBarAction;
@@ -146,8 +155,11 @@ bool KXmlGuiWindow::isHelpMenuEnabled() const
 KXMLGUIFactory *KXmlGuiWindow::guiFactory()
 {
     K_D(KXmlGuiWindow);
-    if (!d->factory)
+    if (!d->factory) {
         d->factory = new KXMLGUIFactory( this, this );
+        connect(d->factory, SIGNAL(makingChanges(bool)),
+                this, SLOT(_k_slotFactoryMakingChanges(bool)));
+    }
     return d->factory;
 }
 
@@ -180,6 +192,8 @@ void KXmlGuiWindow::setupGUI( StandardWindowOptions options, const QString & xml
 }
 
 void KXmlGuiWindow::setupGUI( const QSize & defaultSize, StandardWindowOptions options, const QString & xmlfile ) {
+    K_D(KXmlGuiWindow);
+
     if( options & Keys ){
         KStandardAction::keyBindings(guiFactory(),
                     SLOT(configureShortcuts()), actionCollection());
@@ -195,32 +209,31 @@ void KXmlGuiWindow::setupGUI( const QSize & defaultSize, StandardWindowOptions o
                       SLOT(configureToolbars() ), actionCollection());
     }
 
+    d->defaultSize = defaultSize;
+
     if( options & Create ){
         createGUI(xmlfile);
     }
 
-    if( options & Save ){
-        // setupGUI() is typically called in the constructor before show(),
-        // so the default window size will be incorrect unless the application
-        // hard coded the size which they should try not to do (i.e. use
-        // size hints).
-        if(initialGeometrySet())
-        {
-          // Do nothing...
-        }
-        else if(defaultSize.isValid())
-        {
-          resize(defaultSize);
-        }
-        else if(isHidden())
-        {
-          adjustSize();
-        }
-        setAutoSaveSettings();
+    if (initialGeometrySet()) {
+        // Do nothing...
+    }
+    else if (d->defaultSize.isValid()) {
+        resize(d->defaultSize);
+    }
+    else if (isHidden()) {
+        adjustSize();
     }
 
+    if( options & Save ){
+        const KConfigGroup cg(autoSaveConfigGroup());
+        if (cg.isValid()) {
+            setAutoSaveSettings(cg);
+        } else {
+            setAutoSaveSettings();
+        }
+    }
 }
-
 void KXmlGuiWindow::createGUI( const QString &xmlfile )
 {
     K_D(KXmlGuiWindow);
@@ -244,17 +257,20 @@ void KXmlGuiWindow::createGUI( const QString &xmlfile )
         d->helpMenu = new KHelpMenu(this, componentData().aboutData(), true, actionCollection());
     }
 
+    const QString windowXmlFile = xmlfile.isNull() ? componentData().componentName() + "ui.rc" : xmlfile;
+
+    // Help beginners who call setXMLFile and then setupGUI...
+    if (!xmlFile().isEmpty() && xmlFile() != windowXmlFile) {
+        kWarning() << "You called setXMLFile(" << xmlFile() << ") and then createGUI or setupGUI,"
+                   << "which also calls setXMLFile and will overwrite the file you have previously set.\n"
+                   << "You should call createGUI("<<xmlFile()<<") or setupGUI(<options>,"<<xmlFile()<<") instead.";
+    }
+
     // we always want to load in our global standards file
     setXMLFile(KStandardDirs::locate("config", "ui/ui_standards.rc", componentData()));
 
-    // now, merge in our local xml file.  if this is null, then that
-    // means that we will be only using the global file
-    if ( !xmlfile.isNull() ) {
-        setXMLFile( xmlfile, true );
-    } else {
-        QString auto_file(componentData().componentName() + "ui.rc");
-        setXMLFile( auto_file, true );
-    }
+    // now, merge in our local xml file.
+    setXMLFile(windowXmlFile, true);
 
     // make sure we don't have any state saved already
     setXMLGUIBuildDocument( QDomDocument() );
@@ -263,7 +279,6 @@ void KXmlGuiWindow::createGUI( const QString &xmlfile )
     guiFactory()->addClient( this );
 
     //  setUpdatesEnabled( true );
-    updateGeometry();
 }
 
 void KXmlGuiWindow::slotStateChanged(const QString &newstate)
@@ -325,20 +340,13 @@ void KXmlGuiWindow::createStandardStatusBarAction(){
 
 void KXmlGuiWindow::finalizeGUI( bool /*force*/ )
 {
-    //kDebug(260) << "KXmlGuiWindow::finalizeGUI force=" << force;
-    // The whole reason for this is that moveToolBar relies on the indexes
-    // of the other toolbars, so in theory it should be called only once per
-    // toolbar, but in increasing order of indexes.
-    // Since we can't do that immediately, we move them, and _then_
-    // we call positionYourself again for each of them, but this time
-    // the toolbariterator should give them in the proper order.
-    // Both the XMLGUI and applySettings call this, hence "force" for the latter.
-    /* FIXME KAction port - not needed?
-    foreach (KToolBar* toolbar, toolBars()) {
-        toolbar->positionYourself( force );
-    }*/
-
-    //d->settingsDirty = false;
+    // FIXME: this really needs to be removed with a code more like the one we had on KDE3.
+    //        what we need to do here is to position correctly toolbars so they don't overlap.
+    //        Also, take in count plugins could provide their own toolbars and those also need to
+    //        be restored.
+    if (autoSaveSettings() && autoSaveConfigGroup().isValid()) {
+        applyMainWindowSettings(autoSaveConfigGroup());
+    }
 }
 
 void KXmlGuiWindow::applyMainWindowSettings(const KConfigGroup &config, bool force)
