@@ -1,6 +1,7 @@
 /*
  *  This file is part of the KDE libraries
  *  Copyright (C) 2006 Matt Broadstone (mbroadst@gmail.com)
+ *  Copyright (C) 2008 Maks Orlovich (maksim@kde.org)
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -32,6 +33,9 @@
 #include "debugwindow.h"
 #include "debugdocument.h"
 
+#include <kmimetype.h>
+#include <kicon.h>
+
 using namespace KJS;
 using namespace KJSDebugger;
 
@@ -54,23 +58,31 @@ ScriptsDock::~ScriptsDock()
 
 void ScriptsDock::documentDestroyed(DebugDocument *document)
 {
-    assert (m_documents.contains(document));
+    // We may be asked to remove a document held for reload repeatedly;
+    // ignore later attempts
+    if (!m_documents.contains(document))
+        return; 
 
-    QTreeWidgetItem *child = m_documents[document];
-    QTreeWidgetItem *parent = child->parent();
-
-    int childIdx = parent->indexOfChild(child);
-    parent->takeChild(childIdx);
-
-    if (parent->childCount() == 0)
-    {
-        int parentIdx = m_widget->indexOfTopLevelItem(parent);
-        parent = m_widget->takeTopLevelItem(parentIdx);
-        m_headers[parent->text(0)];
-    }
-
-    m_documents.remove(document);
+    QTreeWidgetItem* fragment = m_documents[document];
+    m_documents.remove(document);    
+        
+    QTreeWidgetItem* file      = fragment->parent();
+    QTreeWidgetItem* domain    = file->parent();
 
+    delete file->takeChild(file->indexOfChild(fragment));
+    
+    if (file->childCount())
+        return;
+      
+    // Need to clean up the file as well.
+    delete domain->takeChild(domain->indexOfChild(file));
+    
+    if (domain->childCount())
+      return;
+      
+    // ... and domain
+    m_headers.remove(domain->text(0));
+    delete m_widget->takeTopLevelItem(m_widget->indexOfTopLevelItem(domain));
 }
 
 void ScriptsDock::addDocument(DebugDocument *document)
@@ -79,9 +91,11 @@ void ScriptsDock::addDocument(DebugDocument *document)
 
     QString name = document->name();
     QString domain;
+    QString favicon;
 
     if (document->url().isEmpty())
         domain = "????"; // ### KDE4.1: proper i18n'able string
+
     else
     {
         KUrl kurl(document->url());
@@ -89,6 +103,8 @@ void ScriptsDock::addDocument(DebugDocument *document)
             domain = kurl.host();
         else
             domain = "localhost";
+
+        favicon = KMimeType::favIconForUrl(kurl);
     }
 
     QTreeWidgetItem *parent = 0;
@@ -97,12 +113,32 @@ void ScriptsDock::addDocument(DebugDocument *document)
     else
     {
         parent = new QTreeWidgetItem(QStringList() << domain);
-        m_headers[domain] = parent;
-    }
+        if (!favicon.isEmpty()) {
+            KIcon icon(favicon);
+            parent->setIcon(0, icon);
+        }
 
-    QTreeWidgetItem *child = new QTreeWidgetItem(parent, QStringList() << name);
+        m_headers[domain] = parent;
+        m_widget->invisibleRootItem()->addChild(parent);        
+    }
+    
+    // Now try to find a kid for the name
+    QTreeWidgetItem* file = 0;
+    for (int c = 0; c < parent->childCount(); ++c) {
+        QTreeWidgetItem* cand = parent->child(c);
+        if (cand->text(0) == name)
+            file = cand;
+    }
+    
+    if (!file)
+        file = new QTreeWidgetItem(parent, QStringList() << name);
+    
+    // Now add the fragment
+    QString lines = QString::number(1 + document->baseLine()) + "-" + 
+                    QString::number(1 + document->baseLine() + document->length() - 1);
+    
+    QTreeWidgetItem *child = new QTreeWidgetItem(file, QStringList() << lines);
     m_documents[document] = child;
-    m_widget->invisibleRootItem()->addChild(parent);
 }
 
 void ScriptsDock::scriptSelected(QTreeWidgetItem *item, int /*column*/)
@@ -112,41 +148,5 @@ void ScriptsDock::scriptSelected(QTreeWidgetItem *item, int /*column*/)
         emit displayScript(doc);
 }
 
-void ScriptsDock::updateModel()
-{
-/*    m_model->clear();
-
-    QHash<QString, QStandardItem*> parents;
-
-    QStandardItem *top = m_model->invisibleRootItem();
-    foreach (KJS::DebugDocument *document, m_documents)
-    {
-        QString domain = QUrl(document->url()).host();
-        if (domain.isEmpty())
-            domain = "unknown";
-
-        QString name = document->name();
-
-        kDebug() << "domain: " << domain << ", name: " << name;
-        QStandardItem *parent = parents[domain];
-        if (!parent)
-        {
-            kDebug() << "Couldn't find the domain, adding it"; 
-            parent = new QStandardItem(domain);
-            parent->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
-            parents[domain] = parent;
-            top->appendRow(parent);
-        }
-
-        QStandardItem *item = new QStandardItem(name);
-        item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
-
-        QVariant var;
-        var.setValue(document);
-        item->setData(var, Qt::UserRole);
-        parent->appendRow(item);
-    }
-
-    m_view->expandAll();*/
-}
 
+// kate: indent-width 4; replace-tabs on; tab-width 4; space-indent on;
