@@ -48,6 +48,7 @@
 #include <css/cssvalues.h>
 #include <xml/dom_textimpl.h>
 #include <xml/dom2_eventsimpl.h>
+#include <dom/dom_doc.h>
 
 #include <kdebug.h>
 #include <kglobal.h>
@@ -113,44 +114,6 @@ DOMString HTMLElementImpl::namespaceURI() const
         DOMString(XHTML_NAMESPACE) : DOMString();
 }
 
-
-DOMString HTMLElementImpl::localName() const
-{
-    // We only have a localName if we were created by createElementNS(), in which
-    // case we are an XHTML element. This also means we have a lowercase name.
-    if (!m_htmlCompat) // XHTML == not HTMLCompat
-    {
-        NodeImpl::Id _id = id();
-        DOMString tn;
-        if ( _id >= ID_LAST_TAG )
-            tn = getDocument()->getName(ElementId, _id);
-        else // HTML tag
-            tn = getTagName( _id );
-	return tn; // lowercase already
-    }
-    // createElement() always returns elements with a null localName.
-    else
-	return DOMString();
-}
-
-DOMString HTMLElementImpl::tagName() const
-{
-    DOMString tn;
-    NodeImpl::Id _id = id();
-    if ( _id >= ID_LAST_TAG )
-        tn = getDocument()->getName(ElementId, _id);
-    else // HTML tag
-        tn = getTagName( _id );
-
-    if ( m_htmlCompat )
-        tn = tn.upper();
-
-    if (m_prefix)
-        return DOMString(m_prefix) + ":" + tn;
-
-    return tn;
-}
-
 void HTMLElementImpl::parseAttribute(AttributeImpl *attr)
 {
     DOMString indexstring;
@@ -170,24 +133,27 @@ void HTMLElementImpl::parseAttribute(AttributeImpl *attr)
     case ATTR_ID:
         // unique id
         setHasID();
-        getDocument()->incDOMTreeVersion();
+        document()->incDOMTreeVersion(DocumentImpl::TV_IDNameHref);
         break;
     case ATTR_CLASS:
         if (attr->val()) {
-          DOMString v = attr->value();
-          const QChar* s = v.unicode();
-          int l = v.length();
-          while( l && !s->isSpace() )
-            l--,s++;
-          setHasClassList(l);
-          setHasClass(true);
+            DOMString v = attr->value();
+            const QChar* characters = v.unicode();
+            unsigned length = v.length();
+            unsigned i;
+            for (i = 0; i < length && characters[i].isSpace(); ++i) { }
+            setHasClass(i < length);
+            attributes()->setClass(v);
         } else {
-          setHasClassList(false);
-          setHasClass(false);
+            setHasClass(false);
         }
+        document()->incDOMTreeVersion(DocumentImpl::TV_Class);
         break;
     case ATTR_NAME:
-        getDocument()->incDOMTreeVersion();
+        document()->incDOMTreeVersion(DocumentImpl::TV_IDNameHref);
+        break;
+    case ATTR_CONTENTEDITABLE:
+        setContentEditable(attr);
         break;
     case ATTR_STYLE:
         if (inlineStyleDecls())
@@ -207,60 +173,59 @@ void HTMLElementImpl::parseAttribute(AttributeImpl *attr)
         break;
     case ATTR_DIR:
         addCSSProperty(CSS_PROP_DIRECTION, attr->value().lower());
-        addCSSProperty(CSS_PROP_UNICODE_BIDI, CSS_VAL_EMBED);
         break;
 // standard events
     case ATTR_ONCLICK:
 	setHTMLEventListener(EventImpl::KHTML_ECMA_CLICK_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onclick", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onclick", this));
         break;
     case ATTR_ONDBLCLICK:
 	setHTMLEventListener(EventImpl::KHTML_ECMA_DBLCLICK_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "ondblclick", this));
+	    document()->createHTMLEventListener(attr->value().string(), "ondblclick", this));
         break;
     case ATTR_ONMOUSEDOWN:
         setHTMLEventListener(EventImpl::MOUSEDOWN_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onmousedown", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onmousedown", this));
         break;
     case ATTR_ONMOUSEMOVE:
         setHTMLEventListener(EventImpl::MOUSEMOVE_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onmousemove", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onmousemove", this));
         break;
     case ATTR_ONMOUSEOUT:
         setHTMLEventListener(EventImpl::MOUSEOUT_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onmouseout", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onmouseout", this));
         break;
     case ATTR_ONMOUSEOVER:
         setHTMLEventListener(EventImpl::MOUSEOVER_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onmouseover", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onmouseover", this));
         break;
     case ATTR_ONMOUSEUP:
         setHTMLEventListener(EventImpl::MOUSEUP_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onmouseup", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onmouseup", this));
         break;
     case ATTR_ONKEYDOWN:
         setHTMLEventListener(EventImpl::KEYDOWN_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onkeydown", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onkeydown", this));
 	break;
     case ATTR_ONKEYPRESS:
         setHTMLEventListener(EventImpl::KEYPRESS_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onkeypress", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onkeypress", this));
 	break;
     case ATTR_ONKEYUP:
         setHTMLEventListener(EventImpl::KEYUP_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onkeyup", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onkeyup", this));
         break;
     case ATTR_ONFOCUS:
         setHTMLEventListener(EventImpl::FOCUS_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string(), "onfocus", this));
+            document()->createHTMLEventListener(attr->value().string(), "onfocus", this));
         break;
     case ATTR_ONBLUR:
         setHTMLEventListener(EventImpl::BLUR_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string(), "onblur", this));
+            document()->createHTMLEventListener(attr->value().string(), "onblur", this));
         break;
     case ATTR_ONSCROLL:
         setHTMLEventListener(EventImpl::SCROLL_EVENT,
-            getDocument()->createHTMLEventListener(attr->value().string(), "onscroll", this));
+            document()->createHTMLEventListener(attr->value().string(), "onscroll", this));
         break;
 // other misc attributes
     default:
@@ -446,7 +411,7 @@ void HTMLElementImpl::removeCSSProperty(int id)
 {
     if (!m_hasCombinedStyle)
         return;
-    nonCSSStyleDecls()->setParent(getDocument()->elementSheet());
+    nonCSSStyleDecls()->setParent(document()->elementSheet());
     nonCSSStyleDecls()->removeProperty(id);
     setChanged();
 }
@@ -494,28 +459,11 @@ DOMString HTMLElementImpl::innerText() const
 
 DocumentFragment HTMLElementImpl::createContextualFragment( const DOMString &html )
 {
-    // the following is in accordance with the definition as used by IE
-    if( endTagRequirement(id()) == FORBIDDEN )
-        return DocumentFragment();
-    // IE disallows innerHTML on inline elements.
-    // I don't see why we should have this restriction, as our
-    // dhtml engine can cope with it. Lars
-    //if ( isInline() ) return false;
-    switch( id() ) {
-        case ID_COL:
-        case ID_COLGROUP:
-        case ID_FRAMESET:
-        case ID_HEAD:
-        case ID_TABLE:
-        case ID_TBODY:
-        case ID_TFOOT:
-        case ID_THEAD:
-        case ID_TITLE:
-            return DocumentFragment();
-        default:
-            break;
-    }
-    if ( !getDocument()->isHTMLDocument() )
+    // IE originally restricted innerHTML to a small subset of elements; 
+    // and we largely matched that. Mozilla's embrace of innerHTML, however, extended 
+    // it to pretty much everything, and so the web (and HTML5) requires it now.
+    // For now, we accept everything, but do not do context-based recovery in the parser.
+    if ( !document()->isHTMLDocument() )
         return DocumentFragment();
 
     DocumentFragmentImpl* fragment = new DocumentFragmentImpl( docPtr() );
@@ -647,6 +595,44 @@ void HTMLElementImpl::addHTMLAlignment( DOMString alignment )
 	addCSSProperty( CSS_PROP_VERTICAL_ALIGN, propvalign );
 }
 
+DOMString HTMLElementImpl::contentEditable() const
+{
+    document()->updateRendering();
+
+    if (!renderer())
+        return "false";
+
+    switch (renderer()->style()->userInput()) {
+        case UI_ENABLED:
+            return "true";
+        case UI_DISABLED:
+        case UI_NONE:
+            return "false";
+        default: ;
+    }
+    return "inherit";
+}
+
+void HTMLElementImpl::setContentEditable(AttributeImpl* attr)
+{
+    const DOMString &enabled = attr->value();
+    if (enabled.isEmpty() || strcasecmp(enabled, "true") == 0)
+        addCSSProperty(CSS_PROP__KHTML_USER_INPUT, CSS_VAL_ENABLED);
+    else if (strcasecmp(enabled, "false") == 0)
+        addCSSProperty(CSS_PROP__KHTML_USER_INPUT, CSS_VAL_NONE);
+    else if (strcasecmp(enabled, "inherit") == 0)
+        addCSSProperty(CSS_PROP__KHTML_USER_INPUT, CSS_VAL_INHERIT);
+}
+
+void HTMLElementImpl::setContentEditable(const DOMString &enabled) {
+    if (enabled == "inherit") {
+        int exceptionCode;
+        removeAttribute(ATTR_CONTENTEDITABLE, exceptionCode);
+    }
+    else
+        setAttribute(ATTR_CONTENTEDITABLE, enabled.isEmpty() ? "true" : enabled);
+}
+
 DOMString HTMLElementImpl::toString() const
 {
     if (!hasChildNodes()) {
@@ -669,7 +655,7 @@ DOMString HTMLElementImpl::toString() const
 HTMLGenericElementImpl::HTMLGenericElementImpl(DocumentImpl *doc, ushort i)
     : HTMLElementImpl(doc)
 {
-    _id = i;
+    m_localName = LocalName::fromId(i);
 }
 
 HTMLGenericElementImpl::~HTMLGenericElementImpl()
