@@ -99,6 +99,9 @@ void HTMLPartContainerElementImpl::setNeedComputeContent()
 
 void HTMLPartContainerElementImpl::setWidget(QWidget* widget)
 {
+    if (widget == m_childWidget)
+        return; // The same part got navigated. Don't do anything
+        
     QWidget* oldWidget = m_childWidget;
     m_childWidget = widget;
     if (m_childWidget)
@@ -126,6 +129,31 @@ bool HTMLPartContainerElementImpl::mimetypeHandledInternally(const QString&)
     return false;
 }
 
+void HTMLPartContainerElementImpl::slotEmitLoadEvent()
+{
+    dispatchHTMLEvent(EventImpl::LOAD_EVENT, false, false);
+}
+
+void HTMLPartContainerElementImpl::postResizeEvent()
+{
+   QApplication::postEvent( this, new QEvent(static_cast<QEvent::Type>(DOMCFResizeEvent)) );
+}
+
+void HTMLPartContainerElementImpl::sendPostedResizeEvents()
+{
+    QApplication::sendPostedEvents(0, DOMCFResizeEvent);
+}
+
+bool HTMLPartContainerElementImpl::event(QEvent *e)
+{
+    if (e->type() == static_cast<QEvent::Type>(DOMCFResizeEvent)) {
+        dispatchWindowEvent(EventImpl::RESIZE_EVENT, false, false);
+        e->accept();
+        return true;
+    }
+    return QObject::event(e);
+}
+
 // -------------------------------------------------------------------------
 HTMLObjectBaseElementImpl::HTMLObjectBaseElementImpl(DocumentImpl *doc)
     : HTMLPartContainerElementImpl(doc)
@@ -169,8 +197,8 @@ void HTMLObjectBaseElementImpl::parseAttribute(AttributeImpl *attr)
             break;
         case ATTR_NAME:
             if (inDocument() && m_name != attr->value()) {
-                getDocument()->underDocNamedCache().remove(m_name.string(),        this);
-                getDocument()->underDocNamedCache().add   (attr->value().string(), this);
+                document()->underDocNamedCache().remove(m_name,        this);
+                document()->underDocNamedCache().add   (attr->value(), this);
             }
             m_name = attr->value();
             //fallthrough
@@ -210,25 +238,25 @@ void HTMLObjectBaseElementImpl::defaultEventHandler(EventImpl *e)
 
 void HTMLObjectBaseElementImpl::removedFromDocument()
 {
-    getDocument()->underDocNamedCache().remove(m_name.string(), this);
+    document()->underDocNamedCache().remove(m_name, this);
     HTMLElementImpl::removedFromDocument();
 }
 
 void HTMLObjectBaseElementImpl::insertedIntoDocument()
 {
-    getDocument()->underDocNamedCache().add(m_name.string(), this);
+    document()->underDocNamedCache().add(m_name, this);
     HTMLElementImpl::insertedIntoDocument();
 }
 
-void HTMLObjectBaseElementImpl::removeId(const QString& id)
+void HTMLObjectBaseElementImpl::removeId(const DOMString& id)
 {
-    getDocument()->underDocNamedCache().remove(id, this);
+    document()->underDocNamedCache().remove(id, this);
     HTMLElementImpl::removeId(id);
 }
 
-void HTMLObjectBaseElementImpl::addId   (const QString& id)
+void HTMLObjectBaseElementImpl::addId   (const DOMString& id)
 {
-    getDocument()->underDocNamedCache().add(id, this);
+    document()->underDocNamedCache().add(id, this);
     HTMLElementImpl::addId(id);
 }
 
@@ -272,19 +300,19 @@ void HTMLObjectBaseElementImpl::attach() {
         return;
     }
 
-    RenderStyle* _style = getDocument()->styleSelector()->styleForElement(this);
+    RenderStyle* _style = document()->styleSelector()->styleForElement(this);
     _style->ref();
 
     if (parentNode()->renderer() && parentNode()->renderer()->childAllowed() &&
         _style->display() != NONE)
     {
         if (m_imageLike) {
-            m_render = new (getDocument()->renderArena()) RenderImage(this);
+            m_render = new (document()->renderArena()) RenderImage(this);
             // make sure we don't attach the inner contents
             addCSSProperty(CSS_PROP_DISPLAY, CSS_VAL_NONE);
         }
         else {
-            m_render = new (getDocument()->renderArena()) RenderPartObject(this);
+            m_render = new (document()->renderArena()) RenderPartObject(this);
             // If we already have a widget, set it.
             if (childWidget())
                 static_cast<RenderFrame*>(m_render)->setWidget(childWidget());
@@ -298,14 +326,6 @@ void HTMLObjectBaseElementImpl::attach() {
 
     _style->deref();
     NodeBaseImpl::attach();
-
-    if (m_render) QTimer::singleShot( 0, this, SLOT( slotEmitLoadEvent() ) );
-}
-
-void HTMLObjectBaseElementImpl::slotEmitLoadEvent()
-{
-    // ### do this when we are actually finished loading instead
-    if (m_render) dispatchHTMLEvent(EventImpl::LOAD_EVENT, false, false);
 }
 
 HTMLEmbedElementImpl* HTMLObjectBaseElementImpl::relevantEmbed()
@@ -375,16 +395,16 @@ void HTMLObjectBaseElementImpl::computeContent()
     if (id() != ID_OBJECT) {
           NamedAttrMapImpl* a = attributes();
           if (a) {
-            for (unsigned long i = 0; i < a->length(); ++i) {
+            for (unsigned i = 0; i < a->length(); ++i) {
                 NodeImpl::Id id = a->idAt(i);
                 DOMString value = a->valueAt(i);
-                params.append(getDocument()->getName(NodeImpl::AttributeId, id).string() + "=\"" + value.string() + "\"");
+                params.append(LocalName::fromId(localNamePart(id)).toString().string() + "=\"" + value.string() + "\"");
             }
         }
     }
 
     params.append( QLatin1String("__KHTML__PLUGINEMBED=\"YES\"") );
-    params.append( QString::fromLatin1("__KHTML__PLUGINBASEURL=\"%1\"").arg(getDocument()->baseURL().url()));
+    params.append( QString::fromLatin1("__KHTML__PLUGINBASEURL=\"%1\"").arg(document()->baseURL().url()));
 
     // Non-embed elements parse a bunch of attributes and inherit things off <embed>, if any
     HTMLEmbedElementImpl* embed = relevantEmbed();
@@ -409,7 +429,8 @@ void HTMLObjectBaseElementImpl::computeContent()
             effectiveURL = embed->url;
             if (!embed->serviceType.isEmpty())
                 effectiveServiceType = embed->serviceType;
-        } else if (effectiveURL.isEmpty() && classId.startsWith("java:")) {
+        } else if (effectiveURL.isEmpty() &&
+                   classId.startsWith(QLatin1String("java:"))) {
             effectiveServiceType = "application/x-java-applet";
             effectiveURL         = classId.mid(5);
         }
@@ -437,7 +458,8 @@ void HTMLObjectBaseElementImpl::computeContent()
         }
     }
 
-    if (effectiveServiceType.isEmpty() && effectiveURL.startsWith("data:")) {
+    if (effectiveServiceType.isEmpty() &&
+        effectiveURL.startsWith(QLatin1String("data:"))) {
         // Extract the MIME type from the data URL.
         int index = effectiveURL.indexOf(';');
         if (index == -1)
@@ -455,7 +477,7 @@ void HTMLObjectBaseElementImpl::computeContent()
     // but may need to do a detach/attach
     QStringList supportedImageTypes = khtmlImLoad::ImageManager::loaderDatabase()->supportedMimeTypes();
 
-    bool newImageLike = effectiveServiceType.startsWith("image/") && supportedImageTypes.contains(effectiveServiceType);
+    bool newImageLike = effectiveServiceType.startsWith(QLatin1String("image/")) && supportedImageTypes.contains(effectiveServiceType);
 
     if (newImageLike != m_imageLike) {
         m_imageLike = newImageLike;
@@ -469,13 +491,13 @@ void HTMLObjectBaseElementImpl::computeContent()
     bool newRenderAlternative = false;
 
     // If we aren't permitted to load this by security policy, render alternative content instead.
-    if (!getDocument()->isURLAllowed(effectiveURL))
+    if (!document()->isURLAllowed(effectiveURL))
         newRenderAlternative = true;
 
     // If Java is off, render alternative as well...
     if (effectiveServiceType == "application/x-java-applet") {
-        KHTMLView* w = getDocument()->view();
-        if (!w || !w->part()->javaEnabled())
+        KHTMLPart* p = document()->part();
+        if (!p || !p->javaEnabled())
             newRenderAlternative = true;
     }
 
@@ -492,7 +514,7 @@ void HTMLObjectBaseElementImpl::computeContent()
     if (m_renderAlternative)
         return;
 
-    KHTMLPart* part = getDocument()->part();
+    KHTMLPart* part = document()->part();
     clearChildWidget();
 
     kDebug(6031) << effectiveURL << effectiveServiceType << params;
@@ -550,13 +572,13 @@ void HTMLObjectBaseElementImpl::slotPartLoadingErrorNotify()
 
     serviceType = embed->serviceType;
 
-    KHTMLPart* part = getDocument()->part();
+    KHTMLPart* part = document()->part();
     KParts::BrowserExtension *ext = part->browserExtension();
 
     if(!embed->pluginPage.isEmpty() && ext) {
         // Prepare the mimetype to show in the question (comment if available, name as fallback)
         QString mimeName = serviceType;
-        KMimeType::Ptr mime = KMimeType::mimeType(serviceType);
+        KMimeType::Ptr mime = KMimeType::mimeType(serviceType, KMimeType::ResolveAliases);
         if ( mime && mime->name() != KMimeType::defaultMimeType() )
             mimeName = mime->comment();
 
@@ -708,9 +730,9 @@ void HTMLEmbedElementImpl::parseAttribute(AttributeImpl *attr)
 
 void HTMLEmbedElementImpl::attach()
 {
-    KHTMLView* w = getDocument()->view();
+    KHTMLPart* p = document()->part();
 
-    if (!w || !w->part()->pluginsEnabled())
+    if (!p || !p->pluginsEnabled())
         m_renderAlternative = true;
 
     if (parentNode()->id() == ID_OBJECT)
@@ -760,11 +782,11 @@ void HTMLObjectElementImpl::parseAttribute(AttributeImpl *attr)
       break;
     case ATTR_ONLOAD: // ### support load/unload on object elements
         setHTMLEventListener(EventImpl::LOAD_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onload", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onload", this));
         break;
     case ATTR_ONUNLOAD:
         setHTMLEventListener(EventImpl::UNLOAD_EVENT,
-	    getDocument()->createHTMLEventListener(attr->value().string(), "onunload", this));
+	    document()->createHTMLEventListener(attr->value().string(), "onunload", this));
         break;
      case ATTR_VSPACE:
         addCSSLength(CSS_PROP_MARGIN_TOP, attr->value());
@@ -787,9 +809,7 @@ void HTMLObjectElementImpl::parseAttribute(AttributeImpl *attr)
 
 DocumentImpl* HTMLObjectElementImpl::contentDocument() const
 {
-    if ( !m_render ) return 0;
-    if ( !m_render->isWidget() ) return 0;
-    QWidget* widget = static_cast<RenderWidget*>( m_render )->widget();
+    QWidget* widget = childWidget();
     if( widget && qobject_cast<KHTMLView*>( widget ) )
         return static_cast<KHTMLView*>( widget )->part()->xmlDocImpl();
     return 0;
@@ -797,9 +817,9 @@ DocumentImpl* HTMLObjectElementImpl::contentDocument() const
 
 void HTMLObjectElementImpl::attach()
 {
-    KHTMLView* w = getDocument()->view();
+    KHTMLPart* p = document()->part();
 
-    if (!w || !w->part()->pluginsEnabled())
+    if (!p || !p->pluginsEnabled())
         m_renderAlternative = true;
 
     HTMLObjectBaseElementImpl::attach();
@@ -820,7 +840,7 @@ void HTMLParamElementImpl::parseAttribute(AttributeImpl *attr)
         m_value = attr->value().string();
         break;
     case ATTR_ID:
-        if (getDocument()->htmlMode() != DocumentImpl::XHtml) break;
+        if (document()->htmlMode() != DocumentImpl::XHtml) break;
         // fall through
     case ATTR_NAME:
         m_name = attr->value().string();
