@@ -210,7 +210,7 @@ static inline HTMLTableSectionElementImpl* processSection(HTMLTableSectionElemen
     lastSection = section;
     if ( index < 0 ) //append/last mode
         return 0;
-    
+
     long rows   = section->numRows();
     if ( index >= rows ) {
         section =  0;
@@ -395,7 +395,16 @@ void HTMLTableElementImpl::removeChild ( NodeImpl *oldChild, int &exceptioncode
     handleChildRemove( oldChild );
     HTMLElementImpl::removeChild( oldChild, exceptioncode);
 }
- 
+
+void HTMLTableElementImpl::removeChildren()
+{
+    HTMLElementImpl::removeChildren();
+    tCaption.clear();
+    head.clear();
+    foot.clear();
+    firstBody.clear();
+}
+
 static inline bool isTableCellAncestor(NodeImpl* n)
 {
     return n->id() == ID_THEAD || n->id() == ID_TBODY ||
@@ -419,7 +428,7 @@ static bool setTableCellsChanged(NodeImpl* n)
 
     return cellChanged;
 }
- 
+
 void HTMLTableElementImpl::parseAttribute(AttributeImpl *attr)
 {
     // ### to CSS!!
@@ -440,6 +449,7 @@ void HTMLTableElementImpl::parseAttribute(AttributeImpl *attr)
     case ATTR_BORDER:
     {
         int border;
+        bool ok = true;
         // ### this needs more work, as the border value is not only
         //     the border of the box, but also between the cells
         if(!attr->val())
@@ -447,21 +457,28 @@ void HTMLTableElementImpl::parseAttribute(AttributeImpl *attr)
         else if(attr->val()->l == 0)
             border = 1;
         else
-            border = attr->val()->toInt();
+            border = attr->val()->toInt(&ok);
+        if (!ok)
+            border = 1;
 #ifdef DEBUG_DRAW_BORDER
         border=1;
 #endif
         DOMString v = QString::number( border );
         addCSSLength(CSS_PROP_BORDER_WIDTH, v );
 
+        attr->rewriteValue( v );
+
         // wanted by HTML4 specs
         if(!border)
             frame = Void, rules = None;
         else
             frame = Box, rules = All;
 
-
-        if (attached()) updateFrame();
+        if (attached()) {
+            updateFrame();
+            if (tFirstBody())
+                setTableCellsChanged(tFirstBody());
+        }
         break;
     }
     case ATTR_BGCOLOR:
@@ -891,16 +908,16 @@ void HTMLTableCellElementImpl::parseAttribute(AttributeImpl *attr)
     case ATTR_ROWSPAN:
         rSpan = attr->val() ? attr->val()->toInt() : 1;
         // limit this to something not causing an overflow with short int
-        if(rSpan < 1 || rSpan > 1024) rSpan = 1;
+        if(rSpan < 0 || rSpan > 1024) rSpan = 1;
         if (renderer())
-            renderer()->updateFromElement();                    
+            renderer()->updateFromElement();
         break;
     case ATTR_COLSPAN:
         cSpan = attr->val() ? attr->val()->toInt() : 1;
         // limit this to something not causing an overflow with short int
-        if(cSpan < 1 || cSpan > 1024) cSpan = 1;
+        if(cSpan < 0 || cSpan > 1024) cSpan = 1;
         if (renderer())
-            renderer()->updateFromElement();        
+            renderer()->updateFromElement();
         break;
     case ATTR_NOWRAP:
         if (attr->val() != 0)
