@@ -218,9 +218,8 @@ void KHTMLPartBrowserExtension::copy()
     if ( !m_editableFormWidget )
     {
         // get selected text and paste to the clipboard
-        QString text= m_part->selectedText();
-	text.replace( QChar( 0xa0 ), ' ' );
-
+        QString text = m_part->selectedText();
+        text.replace( QChar( 0xa0 ), ' ' );
 
         QClipboard *cb = QApplication::clipboard();
         disconnect( cb, SIGNAL( selectionChanged() ), m_part, SLOT( slotClearSelection() ) );
@@ -262,15 +261,16 @@ void KHTMLPartBrowserExtension::searchProvider()
     // action name is of form "previewProvider[<searchproviderprefix>:]"
     const QString searchProviderPrefix = QString( sender()->objectName() ).mid( 14 );
 
+    const QString text = m_part->simplifiedSelectedText();
     KUriFilterData data;
     QStringList list;
-    data.setData( searchProviderPrefix + m_part->selectedText() );
+    data.setData( searchProviderPrefix + text );
     list << "kurisearchfilter" << "kuriikwsfilter";
 
     if( !KUriFilter::self()->filterUri(data, list) )
     {
         KDesktopFile file("services", "searchproviders/google.desktop");
-        QString encodedSearchTerm = QUrl::toPercentEncoding(m_part->selectedText());
+        QString encodedSearchTerm = QUrl::toPercentEncoding(text);
         KConfigGroup cg(file.desktopGroup());
         data.setData(cg.readEntry("Query").replace("\\{@}", encodedSearchTerm));
     }
@@ -281,14 +281,6 @@ void KHTMLPartBrowserExtension::searchProvider()
     emit m_part->browserExtension()->openUrlRequest( data.uri(), KParts::OpenUrlArguments(), browserArgs );
 }
 
-void KHTMLPartBrowserExtension::openSelection()
-{
-    KParts::BrowserArguments browserArgs;
-    browserArgs.frameName = "_blank";
-
-    emit m_part->browserExtension()->openUrlRequest( m_part->selectedText(), KParts::OpenUrlArguments(), browserArgs );
-}
-
 void KHTMLPartBrowserExtension::paste()
 {
     if ( m_extensionProxy )
@@ -434,97 +426,18 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
 
         editActions.append(khtml->actionCollection()->action("selectAll"));
 
+        addSearchActions(editActions);
 
-        // Fill search provider entries
-        KConfig config("kuriikwsfilterrc");
-        KConfigGroup cg = config.group("General");
-        const QString defaultEngine = cg.readEntry("DefaultSearchEngine", "google");
-        const char keywordDelimiter = cg.readEntry("KeywordDelimiter", static_cast<int>(':'));
-
-        // search text
-        QString selectedText = khtml->selectedText();
-        selectedText.replace("&", "&&");
-        if ( selectedText.length()>18 ) {
-            selectedText.truncate(15);
-            selectedText+="...";
-        }
-
-        // default search provider
-        KService::Ptr service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(defaultEngine));
-
-        // search provider icon
-        QPixmap icon;
-        KUriFilterData data;
-        QStringList list;
-        data.setData( QString("some keyword") );
-        list << "kurisearchfilter" << "kuriikwsfilter";
-
-        QString name;
-        if ( KUriFilter::self()->filterUri(data, list) )
-        {
-            QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
-            if ( iconPath.isEmpty() )
-                icon = SmallIcon("edit-find");
-            else
-                icon = QPixmap( iconPath );
-            name = service->name();
-        }
-        else
-        {
-            icon = SmallIcon("google");
-            name = "Google";
-        }
-
-        KAction *action = new KAction( i18n( "Search for '%1' with %2", selectedText, name ), this );
-        d->m_actionCollection->addAction( "searchProvider", action );
-        editActions.append(action);
-        static_cast<QAction*>( action )->setIcon( QIcon( icon ) );
-        connect( action, SIGNAL(triggered(bool)), d->m_khtml->browserExtension(), SLOT( searchProvider() ) );
-
-        // favorite search providers
-        QStringList favoriteEngines;
-        favoriteEngines << "google" << "google_groups" << "google_news" << "webster" << "dmoz" << "wikipedia";
-        favoriteEngines = cg.readEntry("FavoriteSearchEngines", favoriteEngines);
-
-        if ( !favoriteEngines.isEmpty()) {
-            KActionMenu* providerList = new KActionMenu( i18n( "Search for '%1' with" ,  selectedText ), this );
-            d->m_actionCollection->addAction( "searchProviderList", providerList );
-            editActions.append(providerList);
-
-            QStringList::ConstIterator it = favoriteEngines.begin();
-            for ( ; it != favoriteEngines.end(); ++it ) {
-                if (*it==defaultEngine)
-                    continue;
-                service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(*it));
-                if (!service)
-                    continue;
-                const QString searchProviderPrefix = *(service->property("Keys").toStringList().begin()) + keywordDelimiter;
-                data.setData( searchProviderPrefix + "some keyword" );
-
-                if ( KUriFilter::self()->filterUri(data, list) )
-                {
-                    QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
-                    if ( iconPath.isEmpty() )
-                        icon = SmallIcon("edit-find");
-                    else
-                        icon = QPixmap( iconPath );
-                    name = service->name();
-
-                    KAction *action = new KAction( name, this  );
-                    d->m_actionCollection->addAction( QString( "searchProvider" + searchProviderPrefix ).toLatin1().constData(), action );
-                    static_cast<QAction*>( action )->setIcon( QIcon( icon ) );
-                    connect( action, SIGNAL(triggered(bool)), d->m_khtml->browserExtension(), SLOT( searchProvider() ) );
-
-                    providerList->addAction(action);
-                }
+        QString selectedTextURL = selectedTextAsOneLine();
+        if ( selectedTextURL.contains("://") && KUrl(selectedTextURL).isValid() ) {
+            if (selectedTextURL.length() > 18) {
+                selectedTextURL.truncate(15);
+                selectedTextURL += "...";
             }
-        }
-
-        if ( selectedText.contains("://") && KUrl(selectedText).isValid() ) {
-            KAction *action = new KAction( i18n( "Open '%1'", selectedText ), this );
+            KAction *action = new KAction(i18n("Open '%1'", selectedTextURL), this);
             d->m_actionCollection->addAction( "openSelection", action );
             action->setIcon( KIcon( "window-new" ) );
-            connect( action, SIGNAL(triggered(bool)), d->m_khtml->browserExtension(), SLOT( openSelection() ) );
+            connect( action, SIGNAL(triggered(bool)), this, SLOT( openSelection() ) );
             editActions.append(action);
         }
 
@@ -538,7 +451,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
     if (!url.isEmpty()) {
         QList<QAction *> linkActions;
         if (url.protocol() == "mailto") {
-            KAction *action = new KAction( i18n( "Copy Email Address" ), this );
+            KAction *action = new KAction( i18n( "&Copy Email Address" ), this );
             d->m_actionCollection->addAction( "copylinklocation", action );
             connect( action, SIGNAL(triggered(bool)), this, SLOT(slotCopyLinkLocation()) );
             linkActions.append(action);
@@ -548,7 +461,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
             connect( action, SIGNAL(triggered(bool)), this, SLOT(slotSaveLinkAs()) );
             linkActions.append(action);
 
-            action = new KAction( i18n( "Copy &Link Address" ), this );
+            action = new KAction( i18n( "&Copy Link Address" ), this );
             d->m_actionCollection->addAction( "copylinklocation", action );
             connect( action, SIGNAL(triggered(bool)), this, SLOT( slotCopyLinkLocation() ) );
             linkActions.append(action);
@@ -700,7 +613,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
         separator->setSeparator(true);
         partActions.append(separator);
     }
-    if (!hasSelection || isImage || !url.isEmpty()) {
+    if (!hasSelection && url.isEmpty()) { // only when right-clicking on the page itself
         partActions.append(khtml->actionCollection()->action("viewDocumentSource"));
     }
     if (!hasSelection && url.isEmpty() && !isImage) {
@@ -715,6 +628,113 @@ KHTMLPopupGUIClient::~KHTMLPopupGUIClient()
     delete d;
 }
 
+void KHTMLPopupGUIClient::addSearchActions(QList<QAction *>& editActions)
+{
+    // Fill search provider entries
+    KConfig config("kuriikwsfilterrc");
+    KConfigGroup cg = config.group("General");
+    const QString defaultEngine = cg.readEntry("DefaultSearchEngine", "google");
+    const char keywordDelimiter = cg.readEntry("KeywordDelimiter", static_cast<int>(':'));
+
+    // search text
+    QString selectedText = d->m_khtml->simplifiedSelectedText();
+    if (selectedText.isEmpty())
+        return;
+
+    selectedText.replace("&", "&&");
+    if (selectedText.length() > 18) {
+        selectedText.truncate(15);
+        selectedText += "...";
+    }
+
+    // default search provider
+    KService::Ptr service;
+    if( !defaultEngine.isEmpty())
+        service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(defaultEngine));
+
+    // search provider icon
+    KIcon icon;
+    KUriFilterData data;
+    QStringList list;
+    data.setData(QString("some keyword"));
+    list << "kurisearchfilter" << "kuriikwsfilter";
+
+    QString name;
+    if (KUriFilter::self()->filterUri(data, list)) {
+        QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
+        if (iconPath.isEmpty())
+            icon = KIcon("edit-find");
+        else
+            icon = KIcon(QPixmap(iconPath));
+        name = service->name();
+    } else {
+        icon = KIcon("google");
+        name = "Google";
+    }
+
+    KAction *action = new KAction(i18n("Search for '%1' with %2", selectedText, name), this);
+    d->m_actionCollection->addAction("searchProvider", action);
+    editActions.append(action);
+    action->setIcon(icon);
+    connect(action, SIGNAL(triggered(bool)), d->m_khtml->browserExtension(), SLOT(searchProvider()));
+
+    // favorite search providers
+    QStringList favoriteEngines;
+    favoriteEngines << "google" << "google_groups" << "google_news" << "webster" << "dmoz" << "wikipedia";
+    favoriteEngines = cg.readEntry("FavoriteSearchEngines", favoriteEngines);
+
+    if (!favoriteEngines.isEmpty()) {
+        KActionMenu* providerList = new KActionMenu(i18n("Search for '%1' with", selectedText), this);
+        d->m_actionCollection->addAction("searchProviderList", providerList);
+        editActions.append(providerList);
+
+        QStringList::ConstIterator it = favoriteEngines.constBegin();
+        for (; it != favoriteEngines.constEnd(); ++it) {
+            if (*it==defaultEngine)
+                continue;
+            service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(*it));
+            if (!service)
+                continue;
+            const QString searchProviderPrefix = *(service->property("Keys").toStringList().begin()) + keywordDelimiter;
+            data.setData(searchProviderPrefix + "some keyword");
+
+            if (KUriFilter::self()->filterUri(data, list)) {
+                const QString iconPath = KStandardDirs::locate("cache", KMimeType::favIconForUrl(data.uri()) + ".png");
+                if (iconPath.isEmpty())
+                    icon = KIcon("edit-find");
+                else
+                    icon = KIcon(iconPath);
+                name = service->name();
+
+                KAction *action = new KAction(name, this);
+                d->m_actionCollection->addAction(QString("searchProvider" + searchProviderPrefix).toLatin1().constData(), action);
+                action->setIcon(icon);
+                connect(action, SIGNAL(triggered(bool)), d->m_khtml->browserExtension(), SLOT(searchProvider()));
+
+                providerList->addAction(action);
+            }
+        }
+    }
+}
+
+QString KHTMLPopupGUIClient::selectedTextAsOneLine() const
+{
+    QString text = d->m_khtml->simplifiedSelectedText();
+    // in addition to what simplifiedSelectedText does,
+    // remove linefeeds and any whitespace surrounding it (#113177),
+    // to get it all in a single line.
+    text.remove(QRegExp("[\\s]*\\n+[\\s]*"));
+    return text;
+}
+
+void KHTMLPopupGUIClient::openSelection()
+{
+    KParts::BrowserArguments browserArgs;
+    browserArgs.frameName = "_blank";
+
+    emit d->m_khtml->browserExtension()->openUrlRequest(selectedTextAsOneLine(), KParts::OpenUrlArguments(), browserArgs);
+}
+
 KParts::BrowserExtension::ActionGroupMap KHTMLPopupGUIClient::actionGroups() const
 {
     return d->actionGroups;
@@ -904,7 +924,7 @@ void KHTMLPopupGUIClient::saveURL( QWidget *parent, const QString &caption,
     destURL = KFileDialog::getSaveUrl( name, filter, parent, caption );
       if( destURL.isLocalFile() )
       {
-        QFileInfo info( destURL.path() );
+        QFileInfo info( destURL.toLocalFile() );
         if( info.exists() ) {
           // TODO: use KIO::RenameDlg (shows more information)
           query = KMessageBox::warningContinueCancel( parent, i18n( "A file named \"%1\" already exists. " "Are you sure you want to overwrite it?" ,  info.fileName() ), i18n( "Overwrite File?" ), KGuiItem(i18n( "Overwrite" )) );
@@ -927,7 +947,7 @@ void KHTMLPopupGUIClient::saveURL( QWidget* parent, const KUrl &url, const KUrl
         {
             if (destURL.isLocalFile())
             {
-                KSaveFile destFile(destURL.path());
+                KSaveFile destFile(destURL.toLocalFile());
                 if (destFile.open())
                 {
                     QDataStream stream ( &destFile );
