@@ -30,6 +30,10 @@
 
 #include "render_flow.h"
 
+namespace DOM {
+    class Position;
+}
+
 namespace khtml {
 
 class RenderBlock : public RenderFlow
@@ -110,6 +114,7 @@ public:
     void insertPositionedObject(RenderObject *o);
     void removePositionedObject(RenderObject *o);
 
+    QRegion visibleFloatingRegion(int x, int y) const;
     // Called to lay out the legend for a fieldset.
     virtual RenderObject* layoutLegend(bool /*relayoutChildren*/) { return 0; }
 
@@ -142,7 +147,7 @@ public:
     // called from lineWidth, to position the floats added in the last line.
     void positionNewFloats();
     void clearFloats();
-    int getClearDelta(RenderObject *child);
+    int getClearDelta(RenderObject *child, int yPos);
     virtual void markAllDescendantsWithFloatsForLayout(RenderObject* floatToRemove = 0);
 
     // FIXME: containsFloats() should not return true if the floating objects list
@@ -181,6 +186,8 @@ public:
 
     bool isPointInScrollbar(int x, int y, int tx, int ty);
 
+    virtual DOM::Position positionForCoordinates(int x, int y);
+
     virtual void calcMinMaxWidth();
     void calcInlineMinMaxWidth();
     void calcBlockMinMaxWidth();
@@ -188,12 +195,14 @@ public:
     virtual void close();
 
     virtual int getBaselineOfFirstLineBox();
-    virtual InlineFlowBox* getFirstLineBox();
 
     RootInlineBox* firstRootBox() { return static_cast<RootInlineBox*>(m_firstLineBox); }
     RootInlineBox* lastRootBox() { return static_cast<RootInlineBox*>(m_lastLineBox); }
 
+    virtual InlineFlowBox* getFirstLineBox();
+
     bool inRootBlockContext() const;
+    void deleteLineBoxTree();
 
 #ifdef ENABLE_DUMP
     virtual void printTree(int indent=0) const;
@@ -203,6 +212,10 @@ public:
 protected:
     void newLine();
 
+private:
+    DOM::Position positionForBox(InlineBox *box, bool start=true) const;
+    DOM::Position positionForRenderer(RenderObject *renderer, bool start=true) const;
+
 protected:
     struct FloatingObject {
         enum Type {
@@ -342,8 +355,8 @@ protected:
     RenderObject* handlePositionedChild(RenderObject* child, const MarginInfo& marginInfo, bool& handled);
     RenderObject* handleCompactChild(RenderObject* child, CompactInfo& compactInfo, const MarginInfo& marginInfo, bool& handled);
     RenderObject* handleRunInChild(RenderObject* child, bool& handled);
-    void collapseMargins(RenderObject* child, MarginInfo& marginInfo, int yPosEstimate);
-    void clearFloatsIfNeeded(RenderObject* child, MarginInfo& marginInfo, int oldTopPosMargin, int oldTopNegMargin);
+    int collapseMargins(RenderObject* child, MarginInfo& marginInfo, int yPos);
+    int clearFloatsIfNeeded(RenderObject* child, MarginInfo& marginInfo, int oldTopPosMargin, int oldTopNegMargin, int yPos);
     void adjustSizeForCompactIfNeeded(RenderObject* child, CompactInfo& compactInfo);
     void insertCompactIfNeeded(RenderObject* child, CompactInfo& compactInfo);
     int estimateVerticalPosition(RenderObject* child, const MarginInfo& info);
@@ -351,7 +364,6 @@ protected:
     void handleBottomOfBlock(int top, int bottom, MarginInfo& marginInfo);
     void setCollapsedBottomMargin(const MarginInfo& marginInfo);
     void clearChildOfPageBreaks(RenderObject* child, PageBreakInfo &pageBreakInfo, MarginInfo &marginInfo);
-    void deleteLineBoxTree();
     // End helper functions and structs used by layoutBlockChildren.
 
 protected:
