@@ -41,8 +41,16 @@
 #include "rendering/render_style.h"
 #include <QtCore/QTextIStream>
 
+// svg
+#include "FloatRect.h"
+#include "AffineTransform.h"
+
+using WebCore::FloatRect;
+using WebCore::AffineTransform;
+
 class QPainter;
 class QTextStream;
+template<class Key, class T> class QCache;
 
 #ifndef NDEBUG
 #define KHTMLAssert( x ) if( !(x) ) { \
@@ -108,6 +116,8 @@ namespace DOM {
     class DocumentImpl;
     class ElementImpl;
     class EventImpl;
+    class Position;
+    class Selection;
 }
 
 namespace khtml {
@@ -133,6 +143,7 @@ class RenderObject : public CachedObjectClient
     RenderObject(const RenderObject&);
     RenderObject& operator=(const RenderObject&);
 public:
+    KHTML_EXPORT static void cleanup();
 
     RenderObject(DOM::NodeImpl* node);
     virtual ~RenderObject();
@@ -148,6 +159,12 @@ public:
     RenderObject *nextRenderer() const; 
     RenderObject *previousRenderer() const; 
 
+    RenderObject *nextEditable() const;
+    RenderObject *previousEditable() const;
+
+    RenderObject *firstLeafChild() const;
+    RenderObject *lastLeafChild() const;
+
     virtual bool childAllowed() const { return false; }
     virtual int borderTopExtra() const { return 0; }
     virtual int borderBottomExtra() const { return 0; }
@@ -180,8 +197,8 @@ public:
 
     // Whether or not a positioned element requires normal flow x/y to be computed
     // to determine its position.
-    bool hasStaticX() const { return style()->left().isVariable() && style()->right().isVariable(); }
-    bool hasStaticY() const { return style()->top().isVariable() && style()->bottom().isVariable(); }
+    bool hasStaticX() const { return style()->left().isAuto() && style()->right().isAuto(); }
+    bool hasStaticY() const { return style()->top().isAuto() && style()->bottom().isAuto(); }
     bool isPosWithStaticDim()   const { return isPositioned() && (hasStaticX() || hasStaticY()); }
 
     // Linear tree traversal
@@ -278,6 +295,21 @@ public:
     virtual bool isFrameSet() const { return false; }
     virtual bool isApplet() const { return false; }
 
+    virtual bool isEditable() const;
+
+    // svg
+    virtual bool isSVGRoot() const { return false; }
+    virtual bool isRenderPath() const { return false; }
+    virtual bool isSVGContainer() const { return false; }
+    virtual bool isSVGText() const { return false; }
+    virtual bool isSVGHiddenContainer() const { return false; }
+
+    virtual FloatRect relativeBBox(bool includeStroke = false) const;
+
+    virtual AffineTransform localTransform() const;
+    virtual AffineTransform absoluteTransform() const;
+    // end svg
+
     bool isHTMLMarquee() const;
     bool isWordBreak() const;
 
@@ -294,6 +326,7 @@ public:
     bool isCompact() const { return style()->display() == COMPACT; } // compact
     bool isRunIn() const { return style()->display() == RUN_IN; } // run-in object
     bool mouseInside() const;
+    bool isDragging() const;
     bool isReplaced() const { return m_replaced; }
     bool isReplacedBlock() const { return isInline() && isReplaced() && isRenderBlock(); }
     bool shouldPaintBackgroundOrBorder() const { return m_paintBackground; }
@@ -304,7 +337,6 @@ public:
     bool posChildNeedsLayout() const { return m_posChildNeedsLayout; }
     bool normalChildNeedsLayout() const { return m_normalChildNeedsLayout; }
     bool minMaxKnown() const{ return m_minMaxKnown; }
-    bool overhangingContents() const { return m_overhangingContents; }
     bool hasFirstLine() const { return m_hasFirstLine; }
     bool isSelectionBorder() const { return m_isSelectionBorder; }
     bool recalcMinMax() const { return m_recalcMinMax; }
@@ -323,7 +355,6 @@ public:
      */
     RenderObject *container() const;
 
-    void setOverhangingContents(bool p=true);
     void markContainingBlocksForLayout();
     void dirtyFormattingContext( bool checkContainer );
     void repaintDuringLayout();
@@ -333,10 +364,8 @@ public:
 	m_minMaxKnown = b;
 	if ( !b ) {
 	    RenderObject *o = this;
-	    RenderObject *root = this;
 	    while( o ) { // ### && !o->m_recalcMinMax ) {
 		o->m_recalcMinMax = true;
-		root = o;
 		o = o->m_parent;
 	    }
 	}
@@ -363,6 +392,7 @@ public:
     virtual InlineBox* createInlineBox(bool makePlaceHolderBox, bool isRootLineBox);
     virtual void removeInlineBox(InlineBox* /*box*/) {}
 
+    virtual InlineBox *inlineBox(long offset=0);
     virtual short lineHeight( bool firstLine ) const;
     virtual short verticalPositionHint( bool firstLine ) const;
     virtual short baselinePosition( bool firstLine ) const;
@@ -383,6 +413,12 @@ public:
     };
     virtual void paint( PaintInfo& i, int tx, int ty);
 
+    void adjustBorderRadii(BorderRadii &tl, BorderRadii &tr, BorderRadii &bl, BorderRadii &br, int w, int h) const;
+
+    void drawBorderArc(QPainter *p, int x, int y, float horThickness, float vertThickness,
+                       const BorderRadii &radius, int angleStart, int angleSpan, const QBrush &brush,
+                       const QColor &textColor, EBorderStyle style, qreal *dashOffset = 0) const;
+
     void paintBorder(QPainter *p, int _tx, int _ty, int w, int h, const RenderStyle* style, bool begin=true, bool end=true);
     void paintOutline(QPainter *p, int _tx, int _ty, int w, int h, const RenderStyle* style);
 
@@ -536,12 +572,16 @@ public:
 	{}
     };
 
+#if 0
     virtual FindSelectionResult checkSelectionPoint( int _x, int _y, int _tx, int _ty,
                                                      DOM::NodeImpl*&, int & offset,
 						     SelPointState & );
+#endif
     virtual bool nodeAtPoint(NodeInfo& info, int x, int y, int tx, int ty, HitTestAction, bool inside = false);
     void setInnerNode(NodeInfo& info);
 
+    virtual DOM::Position positionForCoordinates(int x, int y);
+
     // set the style of the object.
     virtual void setStyle(RenderStyle *style);
 
@@ -549,9 +589,9 @@ public:
     RenderBlock *containingBlock() const;
 
     // return just the width of the containing block
-    virtual short containingBlockWidth() const;
+    virtual short containingBlockWidth(RenderObject* providedCB=0) const;
     // return just the height of the containing block
-    virtual int containingBlockHeight() const;
+    virtual int containingBlockHeight(RenderObject* providedCB=0) const;
 
     // size of the content area (box size minus padding/border)
     virtual short contentWidth() const { return 0; }
@@ -678,7 +718,7 @@ public:
     };
     void drawBorder(QPainter *p, int x1, int y1, int x2, int y2, BorderSide s,
                     QColor c, const QColor& textcolor, EBorderStyle style,
-                    int adjbw1, int adjbw2, bool invalidisInvert = false);
+                    int adjbw1, int adjbw2, bool invalidisInvert = false, qreal *dashOffset = 0);
 
     // Used by collapsed border tables.
     virtual void collectBorders(QList<CollapsedBorderValue>& borderStyles);
@@ -718,6 +758,10 @@ public:
 
     virtual SelectionState selectionState() const { return SelectionNone;}
     virtual void setSelectionState(SelectionState) {}
+    bool shouldSelect() const;
+    virtual bool isPointInsideSelection(int x, int y, const DOM::Selection &) const;
+
+    DOM::NodeImpl* draggableNode(bool dhtmlOK, bool uaOK, bool& dhtmlWillDrag) const;
 
     /**
      * Flags which influence the appearance and position
@@ -739,7 +783,7 @@ public:
      * @param width returns the caret's width
      * @param height returns the caret's height
      */
-    virtual void caretPos(int offset, int flags, int &_x, int &_y, int &width, int &height);
+    virtual void caretPos(int offset, int flags, int &_x, int &_y, int &width, int &height) const;
 
     // returns the lowest position of the lowest object in that particular object.
     // This 'height' is relative to the topleft of the margin box of the object.
@@ -761,6 +805,8 @@ public:
     virtual void dirtyInlineBoxes(bool /*fullLayout*/, bool /*isRootLineBox*/ = false) {}
     virtual void dirtyLinesFromChangedChild(RenderObject*) {}
     virtual void detach( );
+
+    bool documentBeingDestroyed() const { return !document()->renderer(); }
     
     void setDoNotDelete(bool b) { m_doNotDelete = b; }
     bool doNotDelete() const { return m_doNotDelete; }
@@ -778,19 +824,22 @@ public:
      *
      * Returns 0 by default.
      */
-    virtual long minOffset() const { return 0; }
+    virtual long caretMinOffset() const;
     /** returns the highest possible value the caret offset may have to
      * still point to a valid position.
      *
      * Returns 0 by default, as generic elements are considered to have no
      * width.
      */
-    virtual long maxOffset() const { return 0; }
+    virtual long caretMaxOffset() const;
+    virtual unsigned long caretMaxRenderedOffset() const;
 
     virtual void updatePixmap(const QRect&, CachedImage *);
 
     QRegion visibleFlowRegion(int x, int y) const;
 
+    virtual void removeSuperfluousAnonymousBlockChild( RenderObject* ) {}
+
 protected:
     virtual void selectionStartEnd(int& spos, int& epos);
 
@@ -801,8 +850,6 @@ protected:
     bool attemptDirectLayerTranslation();
     void updateWidgetMasks();
 
-    virtual void removeLeftoverAnonymousBoxes();
-
     void arenaDelete(RenderArena *arena);
 
 private:
@@ -823,7 +870,6 @@ private:
     bool m_floating                  : 1;
 
     bool m_positioned                : 1;
-    bool m_overhangingContents       : 1;
     bool m_relPositioned             : 1;
     bool m_paintBackground           : 1; // if the box has something to paint in the
                                           // background painting phase (background, border, etc)
@@ -852,8 +898,9 @@ private:
 
     bool m_doNotDelete 	             : 1; // This object should not be auto-deleted
 
-    // ### we have 16 + 27 bits.
+    // ### we have 16 + 26 bits.
 
+    static QCache<quint64, QPixmap> *s_dashedLineCache;
 
     void arenaDelete(RenderArena *arena, void *objectBase);
 
