@@ -4,7 +4,8 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
- *           (C) 2003-2006 Apple Computer, Inc.
+ *           (C) 2004, 2005, 2006, 2007 Apple Inc. All rights reserved.
+ *           (C) 2005 Maksim Orlovich (maksim@kde.org)
  *           (C) 2006 Allan Sandfeld Jensen (kde@carewolf.com)
  *
  * This library is free software; you can redistribute it and/or
@@ -32,10 +33,11 @@
 #include "dom_textimpl.h"
 #include "dom2_eventsimpl.h"
 #include "dom_docimpl.h"
+#include "dom_nodelistimpl.h"
 #include "xml/dom_position.h"
 #include "xml/dom_selection.h"
 #include "dom_restyler.h"
-#include "ClassNodeList.h"
+#include "html/html_objectimpl.h"
 
 #include <kglobal.h>
 #include <kdebug.h>
@@ -77,15 +79,15 @@ NodeImpl::NodeImpl(DocumentImpl *doc)
       m_changedAscendentAttribute( false ),
       m_inDocument( false ),
       m_hasAnchor( false ),
-      m_specified( false ),
+      m_elementHasRareData( false ),
       m_hovered( false ),
       m_focused( false ),
       m_active( false ),
       m_implicit( false ),
       m_htmlCompat( false ),
-      m_hasClassList( false ),
       m_hasClass( false ),
-      m_hasCombinedStyle( false )
+      m_hasCombinedStyle( false ),
+      m_hasHoverDependency(false)
 {
 }
 
@@ -314,17 +316,18 @@ NodeImpl *NodeImpl::addChild(NodeImpl *)
   return 0;
 }
 
-void NodeImpl::getCaret(int offset, bool override, int &_x, int &_y, int &width, int &height)
+void NodeImpl::getCaret(int /*offset*/, bool override, int &_x, int &_y, int &width, int &height)
 {
     if (m_render) {
-        RenderObject *r;
-	long r_ofs;
+        RenderObject *r = 0;
+	long r_ofs = 0;
 	bool outside, outsideEnd;
 #if 0
 kDebug(6200) << "getCaret: node " << this << " " << nodeName().string() << " offset: " << offset;
 #endif
 //         mapDOMPosToRenderPos(this, offset, r, r_ofs, outside, outsideEnd);
 	outside = false;
+	outsideEnd = false;
 #if 0
 kDebug(6200) << "getCaret: r " << r << " " << (r?r->renderName():QString()) << " r_ofs: " << r_ofs << " outside " << outside << " outsideEnd " << outsideEnd;
 #endif
@@ -564,25 +567,11 @@ void NodeImpl::dispatchWindowEvent(int _id, bool canBubbleArg, bool cancelableAr
     if (!evt->defaultPrevented())
 	doc->defaultEventHandler(evt);
 
-    if (_id == EventImpl::LOAD_EVENT && !evt->propagationStopped()) {
-        // For onload events, send them to the enclosing frame only.
-        // This is a DOM extension and is independent of bubbling/capturing rules of
-        // the DOM.  You send the event only to the enclosing frame.  It does not
-        // bubble through the parent document.
-        DOM::ElementImpl* elt = doc->ownerElement();
-        if (elt && (elt->document()->domain().isNull() ||
-                    elt->document()->domain() == doc->domain())) {
-            // We also do a security check, since we don't want to allow the enclosing
-            // iframe to see loads of child documents in other domains.
-            evt->setCurrentTarget(elt);
-
-            // Capturing first.
-            elt->handleLocalEvents(evt,true);
-
-            // Bubbling second.
-            if (!evt->propagationStopped())
-                elt->handleLocalEvents(evt,false);
-        }
+    if (_id == EventImpl::LOAD_EVENT) {
+        // Trigger Load Event on the enclosing frame if there is one
+        DOM::HTMLPartContainerElementImpl* elt = doc->ownerElement();
+        if (elt)
+            elt->slotEmitLoadEvent();
     }
 
     doc->deref();
@@ -679,15 +668,17 @@ void NodeImpl::dispatchUIEvent(int _id, int detail)
 void NodeImpl::dispatchSubtreeModifiedEvent()
 {
     childrenChanged();
-    document()->incDOMTreeVersion();
+    document()->incDOMTreeVersion(DocumentImpl::TV_Structural);
     if (!document()->hasListenerType(DocumentImpl::DOMSUBTREEMODIFIED_LISTENER))
         return;
     int exceptioncode = 0;
+    ref();
     MutationEventImpl* const evt = new MutationEventImpl(EventImpl::DOMSUBTREEMODIFIED_EVENT,true,
                                                          false,0,DOMString(),DOMString(),DOMString(),0);
     evt->ref();
     dispatchEvent(evt,exceptioncode,true);
     evt->deref();
+    derefOnly();
 }
 
 bool NodeImpl::dispatchKeyEvent(QKeyEvent *key, bool keypress)
@@ -813,7 +804,7 @@ void NodeImpl::checkSetPrefix(const DOMString &_prefix, int &exceptioncode)
     // - if this node is an attribute and the specified prefix is "xmlns" and
     //   the namespaceURI of this node is different from "http://www.w3.org/2000/xmlns/",
     // - or if this node is an attribute and the qualifiedName of this node is "xmlns" [Namespaces].
-    if (Element::khtmlMalformedPrefix(_prefix) || (namespacePart(id()) == defaultNamespace && id() > ID_LAST_TAG) ||
+    if (Element::khtmlMalformedPrefix(_prefix) || /*FIXME: use IDString somehow here (namespacePart(id()) == defaultNamespace && id() > ID_LAST_TAG) ||*/
         (_prefix == "xml" && namespaceURI() != "http://www.w3.org/XML/1998/namespace")) {
         exceptioncode = DOMException::NAMESPACE_ERR;
         return;
@@ -898,8 +889,8 @@ NodeImpl::StyleChange NodeImpl::diff( khtml::RenderStyle *s1, khtml::RenderStyle
 
     EDisplay display1 = s1 ? s1->display() : NONE;
     EDisplay display2 = s2 ? s2->display() : NONE;
-    EPosition position1 = s1 ? s1->position() : STATIC;
-    EPosition position2 = s2 ? s2->position() : STATIC;
+    EPosition position1 = s1 ? s1->position() : PSTATIC;
+    EPosition position2 = s2 ? s2->position() : PSTATIC;
 
     if (display1 != display2 || position1 != position2)
         ch = Detach;
@@ -972,7 +963,7 @@ void NodeImpl::attach()
         if (closed()) m_render->close();
         if (hovered()) m_render->setMouseInside();
     }
-    document()->incDOMTreeVersion();
+    document()->incDOMTreeVersion(DocumentImpl::TV_Structural);
     m_attached = true;
 }
 
@@ -984,7 +975,7 @@ void NodeImpl::detach()
         m_render->detach();
 
     m_render = 0;
-    document()->incDOMTreeVersion();
+    document()->incDOMTreeVersion(DocumentImpl::TV_Structural);
     m_attached = false;
 }
 
@@ -1112,6 +1103,11 @@ RenderObject *NodeImpl::createRenderer(RenderArena* /*arena*/, RenderStyle* /*st
     return 0;
 }
 
+RenderStyle *NodeImpl::computedStyle()
+{
+    return parentNode() ? parentNode()->computedStyle() : 0;
+}
+
 NodeImpl *NodeImpl::previousLeafNode() const
 {
     NodeImpl *node = traversePreviousNode();
@@ -1143,7 +1139,7 @@ long NodeImpl::caretMinOffset() const
 {
     return renderer() ? renderer()->caretMinOffset() : 0;
 }
- 
+
 long NodeImpl::caretMaxOffset() const
 {
     return renderer() ? renderer()->caretMaxOffset() : 1;
@@ -1161,7 +1157,7 @@ bool NodeImpl::isBlockFlow() const
 
 bool NodeImpl::isEditableBlock() const
 {
-    return isContentEditable() && isBlockFlow();
+    return isBlockFlow() && isContentEditable();
 }
 
 ElementImpl *NodeImpl::enclosingBlockFlowElement() const
@@ -1246,24 +1242,26 @@ bool NodeImpl::isPointInsideSelection(int x, int y, const Selection &sel) const
 
 }
 
-NodeListImpl* NodeImpl::getElementsByTagName( const DOMString &tagName )
+NodeListImpl* NodeImpl::getElementsByTagName(const DOMString &tagName)
 {
-    NodeImpl::Id id;
-    if ( tagName == "*" )
-        id = 0;
-    else
-        id = document()->getId(NodeImpl::ElementId, tagName.implementation(), false, true);
-    return new TagNodeListImpl( this, id );
+    LocalName localname;
+    PrefixName prefixname;
+    if (tagName == "*") {
+        localname = LocalName::fromId(anyLocalName);
+        prefixname = PrefixName::fromId(emptyPrefix);
+    } else
+        splitPrefixLocalName(tagName, prefixname, localname, m_htmlCompat);
+    return new TagNodeListImpl(this, NamespaceName::fromId(0), localname, prefixname);
 }
 
-NodeListImpl* NodeImpl::getElementsByTagNameNS( const DOMString &namespaceURI, const DOMString &localName )
+NodeListImpl* NodeImpl::getElementsByTagNameNS(const DOMString &namespaceURI, const DOMString &localName)
 {
-    return new TagNodeListImpl( this, namespaceURI, localName );
+    return new TagNodeListImpl(this, namespaceURI, localName);
 }
 
 NodeListImpl* NodeImpl::getElementsByClassName(const DOMString &name)
 {
-    return new ClassNodeList(this, name);
+    return new ClassNodeListImpl(this, name);
 }
 
 bool NodeImpl::hasAttributes() const
@@ -1289,6 +1287,105 @@ DocumentImpl* NodeImpl::ownerDocument() const
         return doc;
 }
 
+// Helper for compareDocumentPosition --- this extends the notion of a parent node
+// beyond structural to also include elements containing attributes, etc.
+static const NodeImpl* logicalParentNode(const DOM::NodeImpl* node)
+{
+    NodeImpl* parent = node->parentNode();
+    if (parent)
+        return parent;
+        
+    switch (node->nodeType()) {
+        case Node::ATTRIBUTE_NODE:
+            return static_cast<const AttrImpl*>(node)->ownerElement();
+        
+        case Node::ENTITY_NODE:
+        case Node::NOTATION_NODE:
+            return node->ownerDocument()->doctype();
+            
+        default:
+            return 0;
+    }
+}
+
+unsigned NodeImpl::compareDocumentPosition(const DOM::NodeImpl* other)
+{
+    if (other == this)
+        return 0; 
+        
+    // First, collect paths of the parents of this and other to the root of their subtrees.
+    // Root goes first, hence the use of QList, with its fast prepends
+    QList<const NodeImpl*> thisPath;
+    for (const NodeImpl* cur = this; cur; cur = logicalParentNode(cur))
+        thisPath.prepend(cur);
+        
+    QList<const NodeImpl*> otherPath;
+    for (const NodeImpl* cur = other; cur; cur = logicalParentNode(cur))
+        otherPath.prepend(cur);
+        
+    // if the roots aren't the same, we're disconnected. We're also supposed to 
+    // return IMPLEMENTATION_SPECIFIC here, and, reading tea leaves, make some 
+    // sort of a stable decision to get a total order.
+    if (thisPath[0] != otherPath[0]) {
+        return Node::DOCUMENT_POSITION_DISCONNECTED | Node::DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC |
+               (this > other ? Node::DOCUMENT_POSITION_PRECEDING : Node::DOCUMENT_POSITION_FOLLOWING);
+    }
+        
+    // Now find our common container.
+    const NodeImpl* common = 0;
+    int   diffPos = -1;
+    for (int pos = 0; pos < thisPath.size() && pos < otherPath.size(); ++pos) {
+        if (thisPath[pos] == otherPath[pos]) {
+            common = thisPath[pos];
+        } else {
+            diffPos = pos;
+            break;
+        }
+    }
+    
+    // Do we have direct containment? 
+    if (common == this)
+        return Node::DOCUMENT_POSITION_CONTAINED_BY | Node::DOCUMENT_POSITION_FOLLOWING;
+    else if (common == other)
+        return Node::DOCUMENT_POSITION_CONTAINS | Node::DOCUMENT_POSITION_PRECEDING;
+        
+    // OK, so now we are not nested, so there are ancestors of both nodes
+    // below common that are different. Since some of those may be logically and not 
+    // physically contained in common, we have to treat the logical containment case specially.
+    const NodeImpl* thisAnc  = thisPath [diffPos];
+    const NodeImpl* otherAnc = otherPath[diffPos];
+    
+    bool thisAncLogical  = thisAnc->parentNode() == 0;
+    bool otherAncLogical = otherAnc->parentNode() == 0;
+    kDebug() << thisAncLogical << otherAncLogical;
+    
+    if (thisAncLogical && otherAncLogical) {
+        // First, try to order by nodeType.
+        if (thisAnc->nodeType() != otherAnc->nodeType())
+            return (thisAnc->nodeType() < otherAnc->nodeType()) ? 
+                Node::DOCUMENT_POSITION_FOLLOWING : Node::DOCUMENT_POSITION_PRECEDING;
+                
+        // If not, another implementation-specific order.
+        return Node::DOCUMENT_POSITION_IMPLEMENTATION_SPECIFIC |
+               (this > other ? Node::DOCUMENT_POSITION_PRECEDING : Node::DOCUMENT_POSITION_FOLLOWING);
+    }
+    
+    if (thisAncLogical)
+        return Node::DOCUMENT_POSITION_FOLLOWING;
+        
+    if (otherAncLogical)
+        return Node::DOCUMENT_POSITION_PRECEDING;
+        
+    // Uff. And now the normal case -- just order thisAnc and otherAnc based on their tree order
+    // see if otherAnc follows thisAnc)
+    for (const NodeImpl* cur = thisAnc; cur; cur = cur->nextSibling()) {
+        if (cur == otherAnc)
+            return Node::DOCUMENT_POSITION_FOLLOWING;
+    }
+    
+    return Node::DOCUMENT_POSITION_PRECEDING;
+}
+
 void NodeImpl::setDocument(DocumentImpl* doc)
 {
     if (m_document == doc)
@@ -1847,7 +1944,7 @@ void NodeBaseImpl::setFocus(bool received)
     // note that we need to recalc the style
         setChanged(); // *:focus is a default style, so we just assume personal dependency
     if (isElementNode()) {
-        document()->dynamicDomRestyler().restyleDepedent(static_cast<ElementImpl*>(this), OtherStateDependency);
+        document()->dynamicDomRestyler().restyleDependent(static_cast<ElementImpl*>(this), OtherStateDependency);
     }
 }
 
@@ -1859,7 +1956,7 @@ void NodeBaseImpl::setActive(bool down)
 
     // note that we need to recalc the style
     if (isElementNode())
-        document()->dynamicDomRestyler().restyleDepedent(static_cast<ElementImpl*>(this), ActiveDependency);
+        document()->dynamicDomRestyler().restyleDependent(static_cast<ElementImpl*>(this), ActiveDependency);
 }
 
 void NodeBaseImpl::setHovered(bool hover)
@@ -1870,7 +1967,7 @@ void NodeBaseImpl::setHovered(bool hover)
 
     // note that we need to recalc the style
     if (isElementNode())
-        document()->dynamicDomRestyler().restyleDepedent(static_cast<ElementImpl*>(this), HoverDependency);
+        document()->dynamicDomRestyler().restyleDependent(static_cast<ElementImpl*>(this), HoverDependency);
 }
 
 unsigned long NodeBaseImpl::childNodeCount()
@@ -1894,10 +1991,14 @@ NodeImpl *NodeBaseImpl::childNode(unsigned long index)
 void NodeBaseImpl::dispatchChildInsertedEvents( NodeImpl *child, int &exceptioncode )
 {
     if (document()->hasListenerType(DocumentImpl::DOMNODEINSERTED_LISTENER)) {
+        // We need to weak-guard ourselves since 'this' may be a fresh node, so
+        // we don't want the mutation event to delete it.
+        ref();
         MutationEventImpl* const evt = new MutationEventImpl(EventImpl::DOMNODEINSERTED_EVENT,true,false,this,DOMString(),DOMString(),DOMString(),0);
         evt->ref();
         child->dispatchEvent(evt,exceptioncode,true);
         evt->deref();
+        derefOnly();
         if (exceptioncode)
             return;
     }
@@ -1912,10 +2013,12 @@ void NodeBaseImpl::dispatchChildInsertedEvents( NodeImpl *child, int &exceptionc
             c->insertedIntoDocument();
 
             if (hasInsertedListeners) {
+                ref();
                 MutationEventImpl* const evt = new MutationEventImpl(EventImpl::DOMNODEINSERTEDINTODOCUMENT_EVENT,false,false,0,DOMString(),DOMString(),DOMString(),0);
                 evt->ref();
                 c->dispatchEvent(evt,exceptioncode,true);
                 evt->deref();
+                derefOnly();
                 if (exceptioncode)
                     return;
             }
@@ -1928,10 +2031,12 @@ void NodeBaseImpl::dispatchChildRemovalEvents( NodeImpl *child, int &exceptionco
     // Dispatch pre-removal mutation events
     document()->notifyBeforeNodeRemoval(child); // ### use events instead
     if (document()->hasListenerType(DocumentImpl::DOMNODEREMOVED_LISTENER)) {
+        ref();
         MutationEventImpl* const evt = new MutationEventImpl(EventImpl::DOMNODEREMOVED_EVENT,true,false,this,DOMString(),DOMString(),DOMString(),0);
         evt->ref();
         child->dispatchEvent(evt,exceptioncode,true);
         evt->deref();
+        derefOnly();
         if (exceptioncode)
             return;
     }
@@ -1945,10 +2050,12 @@ void NodeBaseImpl::dispatchChildRemovalEvents( NodeImpl *child, int &exceptionco
     if (p->nodeType() == Node::DOCUMENT_NODE) {
         for (NodeImpl *c = child; c; c = c->traverseNextNode(child)) {
             if (hasRemovalListeners) {
+                ref();
                 MutationEventImpl* const evt = new MutationEventImpl(EventImpl::DOMNODEREMOVEDFROMDOCUMENT_EVENT,false,false,0,DOMString(),DOMString(),DOMString(),0);
                 evt->ref();
                 c->dispatchEvent(evt,exceptioncode,true);
                 evt->deref();
+                derefOnly();
                 if (exceptioncode)
                     return;
             }
@@ -1956,225 +2063,6 @@ void NodeBaseImpl::dispatchChildRemovalEvents( NodeImpl *child, int &exceptionco
     }
 }
 
-// ---------------------------------------------------------------------------
-NodeImpl *NodeListImpl::item( unsigned long index ) const
-{
-    unsigned long requestIndex = index;
-
-    m_cache->updateNodeListInfo(m_refNode->document());
-
-    NodeImpl* n;
-    bool usedCache = false;
-    if (m_cache->current.node) {
-        //Compute distance from the requested index to the cache node
-        unsigned long cacheDist = qAbs(long(index) - long(m_cache->position));
-
-        if (cacheDist < (unsigned long)index) { //Closer to the cached position
-            usedCache = true;
-            if (index >= m_cache->position) { //Go ahead
-                unsigned long relIndex = index - m_cache->position;
-                n = recursiveItem(m_refNode, m_cache->current.node, relIndex);
-            } else { //Go backwards
-                unsigned long relIndex = m_cache->position - index;
-                n = recursiveItemBack(m_refNode, m_cache->current.node, relIndex);
-            }
-        }
-    }
-
-    if (!usedCache)
-        n = recursiveItem(m_refNode, m_refNode->firstChild(), index);
-
-    //We always update the cache state, to make starting iteration
-    //where it was left off easy.
-    m_cache->current.node  = n;
-    m_cache->position = requestIndex;
-    return n;
-}
-
-unsigned long NodeListImpl::length() const
-{
-    m_cache->updateNodeListInfo(m_refNode->document());
-    if (!m_cache->hasLength) {
-        m_cache->length    = calcLength( m_refNode );
-        m_cache->hasLength = true;
-    }
-    return m_cache->length;
-}
-
-unsigned long NodeListImpl::calcLength(NodeImpl *start) const
-{
-    unsigned long len = 0;
-    for(NodeImpl *n = start->firstChild(); n != 0; n = n->nextSibling()) {
-        bool recurse = true;
-        if (nodeMatches(n, recurse))
-                len++;
-        if (recurse)
-            len+= NodeListImpl::calcLength(n);
-        }
-
-    return len;
-}
-
-NodeListImpl::NodeListImpl( NodeImpl *n, int type, CacheFactory* factory )
-{
-    m_refNode = n;
-    m_refNode->ref();
-
-    m_cache = m_refNode->document()->acquireCachedNodeListInfo(
-                  factory ? factory : Cache::make,
-                  n, type );
-}
-
-NodeListImpl::~NodeListImpl()
-{
-    m_refNode->document()->releaseCachedNodeListInfo(m_cache);
-    m_refNode->deref();
-}
-
-
-/**
- Next item in the pre-order walk of tree from node, but not going outside
- absStart
-*/
-static NodeImpl* helperNext(NodeImpl* node, NodeImpl* absStart)
-{
-    //Walk up until we wind a sibling to go to.
-    while (!node->nextSibling() && node != absStart)
-        node = node->parentNode();
-
-    if (node != absStart)
-        return node->nextSibling();
-    else
-        return 0;
-}
-
-NodeImpl *NodeListImpl::recursiveItem ( NodeImpl* absStart, NodeImpl *start, unsigned long &offset ) const
-{
-    for(NodeImpl *n = start; n != 0; n = helperNext(n, absStart)) {
-        bool recurse = true;
-        if (nodeMatches(n, recurse))
-            if (!offset--)
-                return n;
-
-        NodeImpl *depthSearch = recurse ? recursiveItem(n, n->firstChild(), offset) : 0;
-        if (depthSearch)
-            return depthSearch;
-    }
-
-    return 0; // no matching node in this subtree
-}
-
-
-NodeImpl *NodeListImpl::recursiveItemBack ( NodeImpl* absStart, NodeImpl *start, unsigned long &offset ) const
-{
-    //### it might be cleaner/faster to split nodeMatches and recursion
-    //filtering.
-    bool dummy   = true;
-    NodeImpl* n  = start;
-
-    do {
-        bool recurse = true;
-
-        //Check whether the current node matches.
-        if (nodeMatches(n, dummy))
-            if (!offset--)
-                return n;
-
-        if (n->previousSibling()) {
-            //Move to the last node of this whole subtree that we should recurse into
-            n       = n->previousSibling();
-            recurse = true;
-
-            while (n->lastChild()) {
-                (void)nodeMatches(n, recurse);
-                if (!recurse)
-                   break; //Don't go there
-                n = n->lastChild();
-            }
-        } else {
-            //We're done with this whole subtree, so move up
-            n = n->parentNode();
-        }
-    }
-    while (n && n != absStart);
-
-    return 0;
-}
-
-
-NodeListImpl::Cache::~Cache()
-{}
-
-void NodeListImpl::Cache::clear(DocumentImpl* doc)
-{
-   hasLength = false;
-   current.node = 0;
-   version   = doc->domTreeVersion();
-}
-
-void NodeListImpl::Cache::updateNodeListInfo(DocumentImpl* doc)
-{
-    //If version doesn't match, clear
-    if (doc->domTreeVersion() != version)
-        clear(doc);
-}
-
-ChildNodeListImpl::ChildNodeListImpl( NodeImpl *n ): NodeListImpl(n, CHILD_NODES)
-{}
-
-bool ChildNodeListImpl::nodeMatches( NodeImpl* /*testNode*/, bool& doRecurse ) const
-{
-    doRecurse = false;
-    return true;
-}
-
-TagNodeListImpl::TagNodeListImpl( NodeImpl *n, NodeImpl::Id id )
-  : NodeListImpl(n, UNCACHEABLE),
-    m_id(id),
-    m_namespaceAware(false)
-{
-    // An id of 0 here means "*" (match all nodes)
-    m_matchAllNames = (id == 0);
-    m_matchAllNamespaces = false;
-}
-
-TagNodeListImpl::TagNodeListImpl( NodeImpl *n, const DOMString &namespaceURI, const DOMString &localName )
-  : NodeListImpl(n, UNCACHEABLE),
-    m_id(0),
-    m_namespaceURI(namespaceURI),
-    m_localName(localName),
-    m_namespaceAware(true)
-{
-    m_matchAllNames = (localName == "*");
-    m_matchAllNamespaces = (namespaceURI == "*");
-}
-
-
-bool TagNodeListImpl::nodeMatches( NodeImpl *testNode, bool& /*doRecurse*/ ) const
-{
-    if ( testNode->nodeType() != Node::ELEMENT_NODE ) return false;
-    if (m_namespaceAware)
-	return (m_matchAllNamespaces || testNode->namespaceURI() == m_namespaceURI) &&
-	       (m_matchAllNames || testNode->localName() == m_localName);
-    else {
-        NodeImpl::Id testId = testNode->id();
-        //we have to strip the namespaces if we compare in a namespace unaware fashion
-        if ( !m_namespaceAware ) testId = localNamePart(testId);
-	return (m_id == 0 || m_id == testId);
-    }
-}
-
-NameNodeListImpl::NameNodeListImpl(NodeImpl *n, const DOMString &t )
-  : NodeListImpl(n, UNCACHEABLE),
-    nodeName(t)
-{}
-
-bool NameNodeListImpl::nodeMatches( NodeImpl *testNode, bool& /*doRecurse*/ ) const
-{
-    if ( testNode->nodeType() != Node::ELEMENT_NODE ) return false;
-    return static_cast<ElementImpl *>(testNode)->getAttribute(ATTR_NAME) == nodeName;
-}
-
 // ---------------------------------------------------------------------------
 
 NamedNodeMapImpl::NamedNodeMapImpl()
@@ -2187,9 +2075,11 @@ NamedNodeMapImpl::~NamedNodeMapImpl()
 
 NodeImpl* NamedNodeMapImpl::getNamedItem( const DOMString &name )
 {
-    NodeImpl::Id nid = mapId(0, name.implementation(), true);
-    if (!nid) return 0;
-    return getNamedItem(nid, false, name.implementation());
+    PrefixName prefix;
+    LocalName  localName;
+    splitPrefixLocalName(name, prefix, localName, htmlCompat());
+
+    return getNamedItem(localName.id(), prefix, false);
 }
 
 Node NamedNodeMapImpl::setNamedItem( const Node &arg, int& exceptioncode )
@@ -2199,38 +2089,39 @@ Node NamedNodeMapImpl::setNamedItem( const Node &arg, int& exceptioncode )
       return 0;
     }
 
-    Node r = setNamedItem(arg.handle(), false,
-                       arg.handle()->nodeName().implementation(), exceptioncode);
+    Node r = setNamedItem(arg.handle(), emptyPrefixName, false, exceptioncode);
     return r;
 }
 
 Node NamedNodeMapImpl::removeNamedItem( const DOMString &name, int& exceptioncode )
 {
-    Node r = removeNamedItem(mapId(0, name.implementation(), false),
-                                   false, name.implementation(), exceptioncode);
+    PrefixName prefix;
+    LocalName  localName;
+    splitPrefixLocalName(name, prefix, localName, htmlCompat());
+
+    Node r = removeNamedItem(localName.id(), prefix, false, exceptioncode);
     return r;
 }
 
 Node NamedNodeMapImpl::getNamedItemNS( const DOMString &namespaceURI, const DOMString &localName )
 {
-    NodeImpl::Id nid = mapId( namespaceURI.implementation(), localName.implementation(), true );
-    return getNamedItem(nid, true);
+    LocalName localname = LocalName::fromString(localName);
+    NamespaceName namespacename = NamespaceName::fromString(namespaceURI);
+    return getNamedItem(makeId(namespacename.id(), localname.id()), emptyPrefixName, true);
 }
 
 Node NamedNodeMapImpl::setNamedItemNS( const Node &arg, int& exceptioncode )
 {
-    Node r = setNamedItem(arg.handle(), true, 0, exceptioncode);
-    return r;
+    return setNamedItem(arg.handle(), emptyPrefixName, true, exceptioncode);
 }
 
 Node NamedNodeMapImpl::removeNamedItemNS( const DOMString &namespaceURI, const DOMString &localName, int& exceptioncode )
 {
-    NodeImpl::Id nid = mapId( namespaceURI.implementation(), localName.implementation(), false );
-    Node r = removeNamedItem(nid, true, 0, exceptioncode);
-    return r;
+    LocalName localname = LocalName::fromString(localName);
+    NamespaceName namespacename = NamespaceName::fromString(namespaceURI);
+    return removeNamedItem(makeId(namespacename.id(), localname.id()), emptyPrefixName, true, exceptioncode);
 }
 
-
 // ----------------------------------------------------------------------------
 
 GenericRONamedNodeMapImpl::GenericRONamedNodeMapImpl(DocumentImpl* doc)
@@ -2248,7 +2139,7 @@ GenericRONamedNodeMapImpl::~GenericRONamedNodeMapImpl()
     delete m_contents;
 }
 
-NodeImpl *GenericRONamedNodeMapImpl::getNamedItem ( NodeImpl::Id id, bool /*nsAware*/, DOMStringImpl* /*qName*/ ) const
+NodeImpl *GenericRONamedNodeMapImpl::getNamedItem(NodeImpl::Id id, const PrefixName& /*prefix*/, bool /*nsAware*/)
 {
     // ## do we need namespace support in this class?
     QListIterator<NodeImpl*> it(*m_contents);
@@ -2258,7 +2149,7 @@ NodeImpl *GenericRONamedNodeMapImpl::getNamedItem ( NodeImpl::Id id, bool /*nsAw
     return 0;
 }
 
-Node GenericRONamedNodeMapImpl::setNamedItem ( NodeImpl* /*arg*/, bool /*nsAware*/, DOMStringImpl* /*qName*/, int &exceptioncode )
+Node GenericRONamedNodeMapImpl::setNamedItem(NodeImpl* /*arg*/, const PrefixName& /*prefix*/, bool /*nsAware*/, int &exceptioncode)
 {
     // can't modify this list through standard DOM functions
     // NO_MODIFICATION_ALLOWED_ERR: Raised if this map is readonly
@@ -2266,7 +2157,7 @@ Node GenericRONamedNodeMapImpl::setNamedItem ( NodeImpl* /*arg*/, bool /*nsAware
     return 0;
 }
 
-Node GenericRONamedNodeMapImpl::removeNamedItem ( NodeImpl::Id /*id*/, bool /*nsAware*/, DOMStringImpl* /*qName*/, int &exceptioncode )
+Node GenericRONamedNodeMapImpl::removeNamedItem(NodeImpl::Id /*id*/, const PrefixName& /*prefix*/, bool /*nsAware*/, int &exceptioncode)
 {
     // can't modify this list through standard DOM functions
     // NO_MODIFICATION_ALLOWED_ERR: Raised if this map is readonly
@@ -2274,7 +2165,7 @@ Node GenericRONamedNodeMapImpl::removeNamedItem ( NodeImpl::Id /*id*/, bool /*ns
     return 0;
 }
 
-NodeImpl *GenericRONamedNodeMapImpl::item ( unsigned long index ) const
+NodeImpl *GenericRONamedNodeMapImpl::item(unsigned index)
 {
     if (index >= (unsigned int) m_contents->count())
         return 0;
@@ -2282,7 +2173,7 @@ NodeImpl *GenericRONamedNodeMapImpl::item ( unsigned long index ) const
     return m_contents->at(index);
 }
 
-unsigned long GenericRONamedNodeMapImpl::length(  ) const
+unsigned GenericRONamedNodeMapImpl::length() const
 {
     return m_contents->count();
 }
@@ -2290,24 +2181,13 @@ unsigned long GenericRONamedNodeMapImpl::length(  ) const
 void GenericRONamedNodeMapImpl::addNode(NodeImpl *n)
 {
     // The spec says that in the case of duplicates we only keep the first one
-    if (getNamedItem(n->id(), false, 0))
+    if (getNamedItem(n->id(), emptyPrefixName, false))
         return;
 
     n->ref();
     m_contents->append(n);
 }
 
-NodeImpl::Id GenericRONamedNodeMapImpl::mapId(DOMStringImpl* namespaceURI,
-                                              DOMStringImpl* localName, bool readonly)
-{
-    if (!m_doc)
-	return 0;
-
-    return m_doc->getId(NodeImpl::ElementId,
-                        namespaceURI, 0, localName, readonly,
-                        false /*don't lookupHTML*/);
-}
-
 // -----------------------------------------------------------------------------
 
 void RegisteredListenerList::addEventListener(EventName id, EventListener *listener, const bool useCapture)
@@ -2422,3 +2302,5 @@ bool RegisteredListenerList::stillContainsListener(const RegisteredEventListener
 RegisteredListenerList::~RegisteredListenerList() {
     delete listeners; listeners = 0;
 }
+
+// kate: indent-width 4; replace-tabs on; tab-width 4; space-indent on; 
