@@ -5,7 +5,7 @@
                   1998 Daniel Grana <grana@ie.iwi.unibe.ch>
                   1999,2000,2001,2002,2003 Carsten Pfeiffer <pfeiffer@kde.org>
                   2003 Clarence Dang <dang@kde.org>
-                  2008 Jaroslaw Staniek <js@iidea.pl>
+                  2008 Jarosław Staniek <staniek@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -38,16 +38,11 @@
 #include <kpluginloader.h>
 #include <kpluginfactory.h>
 #include <kdebug.h>
+#include <kwindowsystem.h>
 #include "kabstractfilewidget.h"
 #include "kabstractfilemodule.h"
 #include "krecentdirs.h"
 
-#ifdef Q_WS_X11
-#include <qx11info_x11.h>
-#include <X11/Xlib.h>
-#include <fixx11h.h>
-#endif
-
 /** File dialogs are native by default on Windows. */
 #ifdef Q_WS_WIN
 const bool NATIVE_FILEDIALOGS_BY_DEFAULT = true;
@@ -55,12 +50,34 @@ const bool NATIVE_FILEDIALOGS_BY_DEFAULT = true;
 const bool NATIVE_FILEDIALOGS_BY_DEFAULT = false;
 #endif
 
+static QStringList mime2KdeFilter( const QStringList &mimeTypes, QString *allExtensions = 0 )
+{
+  const KUrl emptyUrl;
+  QStringList kdeFilter;
+  QStringList allExt;
+  foreach( const QString& mimeType, mimeTypes ) {
+    KMimeType::Ptr mime( KMimeType::mimeType(mimeType) );
+    if (mime) {
+      allExt += mime->patterns();
+      kdeFilter.append(mime->patterns().join(QLatin1String(" ")) +
+                       QLatin1Char('|') +
+                       mime->comment(emptyUrl));
+    }
+  }
+  if (allExtensions) {
+      allExt.sort();
+      *allExtensions = allExt.join(QLatin1String(" "));
+  }
+  return kdeFilter;
+}
 /** @return File dialog filter in Qt format for @a filters
  *          or "All files (*)" for empty list.
  */
-static QString qtFilter(const QStringList& filters)
+static QString qtFilter(const QStringList& _filters)
 {
     QString converted;
+    QStringList filters = _filters;
+
     foreach (const QString& current, filters) {
         QString new_f; //filter part
         QString new_name; //filter name part
@@ -73,37 +90,25 @@ static QString qtFilter(const QStringList& filters)
             new_f = current.left(p);
             new_name = current.mid(p+1);
         }
-        // remove (.....) from name
-        p = new_name.indexOf('(');
-        int p2 = new_name.lastIndexOf(')');
-        QString new_name1, new_name2;
-        if (p!=-1)
-            new_name1 = new_name.left(p);
-        if (p2!=-1)
-            new_name2 = new_name.mid(p2+1);
-        if (!new_name1.isEmpty() || !new_name2.isEmpty())
-            new_name = new_name1.trimmed() + QLatin1Char(' ') + new_name2.trimmed();
-        new_name.remove('(');
-        new_name.remove(')');
-        new_name = new_name.trimmed();
+	//Qt filters assume anything in () is the file extension list
+	new_name = new_name.replace('(', '[').replace(')',']').trimmed();
 
-        // make filters unique: remove uppercase extensions (case doesn't matter on win32, BTW)
+        //convert everything to lower case and remove dupes (doesn't matter on win32)
         QStringList allfiltersUnique;
         const QStringList origList( new_f.split(' ', QString::SkipEmptyParts) );
         foreach (const QString& origFilter, origList) {
-            if (origFilter == origFilter.toLower())
-                allfiltersUnique += origFilter;
+	if (!allfiltersUnique.contains(origFilter, Qt::CaseInsensitive))
+                allfiltersUnique += origFilter.toLower();
         }
 
         if (!converted.isEmpty())
             converted += ";;";
 
         converted += (new_name + " (" + allfiltersUnique.join(" ") + QLatin1Char(')'));
-    } // foreach
+    }
 
     // Strip escape characters from escaped '/' characters.
-    for (int pos = 0; (pos = converted.indexOf("\\/", pos)) != -1; ++pos)
-      converted.remove(pos, 1);
+    converted.replace("\\/","/");
 
     return converted;
 }
@@ -118,8 +123,14 @@ static QString qtFilter(const QString& filter)
     QStringList filters;
     if (filter.isEmpty())
         filters += i18n("*|All files");
-    else
-        filters = filter.split('\n', QString::SkipEmptyParts);
+    else {
+      // check if it's a mimefilter
+      int pos = filter.indexOf('/');
+      if (pos > 0 && filter[pos - 1] != '\\')
+          filters = mime2KdeFilter(filter.split(QLatin1Char(' '), QString::SkipEmptyParts));
+      else
+          filters = filter.split('\n', QString::SkipEmptyParts);
+    }
     return qtFilter(filters);
 }
 
@@ -153,7 +164,7 @@ public:
             operationMode(KAbstractFileWidget::Opening)
         {
         }
-        /** @return previously set (global) start dir or the first url 
+        /** @return previously set (global) start dir or the first url
          selected using setSelection() or setUrl() if the start dir is empty. */
         KUrl startDir() const
         {
@@ -172,6 +183,7 @@ public:
             return s_startDir;
         }
         static KUrl s_startDir;
+        static bool s_allowNative;  // as fallback when we can't use natie dialog
         QString filter;
         QStringList mimeTypes;
         KUrl::List selectedUrls;
@@ -184,12 +196,15 @@ public:
         w(0),
         cfgGroup(KGlobal::config(), ConfigGroup)
     {
-        if (cfgGroup.readEntry("Native", NATIVE_FILEDIALOGS_BY_DEFAULT))
+        if (cfgGroup.readEntry("Native", NATIVE_FILEDIALOGS_BY_DEFAULT) &&
+            KFileDialogPrivate::Native::s_allowNative)
             native = new Native;
     }
 
     static bool isNative()
     {
+        if(!KFileDialogPrivate::Native::s_allowNative)
+            return false;
         KConfigGroup cfgGroup(KGlobal::config(), ConfigGroup);
         return cfgGroup.readEntry("Native", NATIVE_FILEDIALOGS_BY_DEFAULT);
     }
@@ -205,6 +220,7 @@ public:
 };
 
 KUrl KFileDialogPrivate::Native::s_startDir;
+bool KFileDialogPrivate::Native::s_allowNative = true;
 
 KFileDialog::KFileDialog( const KUrl& startDir, const QString& filter,
                           QWidget *parent, QWidget* customWidget)
@@ -216,21 +232,27 @@ KFileDialog::KFileDialog( const KUrl& startDir, const QString& filter,
       d( new KFileDialogPrivate )
 
 {
-    if (!d->native) {
-        setButtons( KDialog::None );
-        restoreDialogSize(d->cfgGroup); // call this before the fileQWidget is set as the main widget.
-                                        // otherwise the sizes for the components are not obeyed (ereslibre)
-    }
-
+    // It would be nice to have this behind d->native but it doesn't work
+    // because of derived classes like KEncodingDialog...
     // Dlopen the file widget from libkfilemodule
     QWidget* fileQWidget = fileModule()->createFileWidget(startDir, this);
     d->w = ::qobject_cast<KAbstractFileWidget *>(fileQWidget);
 
     if (d->native) {
-        d->native->filter = filter;
+        KFileDialogPrivate::Native::s_startDir = startDir;
+        // check if it's a mimefilter
+        int pos = filter.indexOf('/');
+        if (pos > 0 && filter[pos - 1] != '\\')
+          setMimeFilter(filter.split(QLatin1Char(' '), QString::SkipEmptyParts));
+        else
+          setFilter(filter);
         return;
     }
 
+    setButtons( KDialog::None );
+    restoreDialogSize(d->cfgGroup); // call this before the fileQWidget is set as the main widget.
+                                   // otherwise the sizes for the components are not obeyed (ereslibre)
+
     d->w->setFilter(filter);
     setMainWidget(fileQWidget);
 
@@ -258,7 +280,7 @@ KFileDialog::KFileDialog( const KUrl& startDir, const QString& filter,
     //connect(fileQWidget, SIGNAL(canceled()), SLOT(slotCancel()));
 
     if (customWidget)
-     d->w->setCustomWidget(customWidget);
+     d->w->setCustomWidget(QString(), customWidget);
 }
 
 
@@ -296,13 +318,12 @@ void KFileDialog::setMimeFilter( const QStringList& mimeTypes,
     d->w->setMimeFilter(mimeTypes, defaultType);
 
     if (d->native) {
-        QString kdeFilter;
-        foreach( const QString& mimeType, mimeTypes ) {
-            KMimeType::Ptr mime( KMimeType::mimeType(mimeType) );
-            if (mime)
-               kdeFilter += (mime->patterns().join(" ") + QLatin1Char('|') + mime->comment());
+        QString allExtensions;
+        QStringList filters = mime2KdeFilter(mimeTypes, &allExtensions);
+        if (defaultType.isEmpty() && (mimeTypes.count() > 1)) {
+            filters.prepend(allExtensions + QLatin1Char('|') + i18n("All Supported Files"));
         }
-        d->native->filter = kdeFilter;
+        d->native->filter = filters.join(QLatin1String("\n"));
     }
 }
 
@@ -332,9 +353,17 @@ void KFileDialog::setPreviewWidget(KPreviewWidgetBase *w)
     d->w->setPreviewWidget(w);
 }
 
+void KFileDialog::setInlinePreviewShown(bool show)
+{
+    if (d->native) {
+        return;
+    }
+    d->w->setInlinePreviewShown(show);
+}
+
 QSize KFileDialog::sizeHint() const
 {
-    return QSize(640, 400);
+    return QSize(700, 450);
 }
 
 // This slot still exists mostly for compat purposes; for subclasses which reimplement slotOk
@@ -380,7 +409,7 @@ void KFileDialog::setSelection(const QString& name)
 {
     if (d->native) {
          d->native->selectedUrls.clear();
-         d->native->selectedUrls.append( KUrl::fromPath(name) );
+         d->native->selectedUrls.append( KUrl(name) );
          return;
     }
     d->w->setSelection(name);
@@ -391,17 +420,17 @@ QString KFileDialog::getOpenFileName(const KUrl& startDir,
                                      QWidget *parent, const QString& caption)
 {
     if (KFileDialogPrivate::isNative() && (!startDir.isValid() || startDir.isLocalFile())) {
-        return QFileDialog::getOpenFileName( 
+        return QFileDialog::getOpenFileName(
             parent,
             caption.isEmpty() ? i18n("Open") : caption,
-            KFileDialogPrivate::Native::staticStartDir( startDir ).path(),
+            KFileDialogPrivate::Native::staticStartDir( startDir ).toLocalFile(),
             qtFilter(filter) );
 // TODO use extra args?     QString * selectedFilter = 0, Options options = 0
     }
     KFileDialog dlg(startDir, filter, parent);
     dlg.setOperationMode( Opening );
 
-    dlg.setMode( KFile::File | KFile::LocalOnly );
+    dlg.setMode( KFile::File | KFile::LocalOnly | KFile::ExistingOnly );
     dlg.setCaption(caption.isEmpty() ? i18n("Open") : caption);
 
     //dlg.d->ops->clearHistory();
@@ -417,17 +446,14 @@ QString KFileDialog::getOpenFileNameWId(const KUrl& startDir,
     if (KFileDialogPrivate::isNative() && (!startDir.isValid() || startDir.isLocalFile()))
         return KFileDialog::getOpenFileName(startDir, filter, 0, caption); // everything we can do...
     QWidget* parent = QWidget::find( parent_id );
+    KFileDialogPrivate::Native::s_allowNative = false;
     KFileDialog dlg(startDir, filter, parent);
-#ifdef Q_WS_X11
     if( parent == NULL && parent_id != 0 )
-        XSetTransientForHint( QX11Info::display(), dlg.winId(), parent_id );
-#else
-    // TODO
-#endif
+        KWindowSystem::setMainWindow( &dlg, parent_id );
 
     dlg.setOperationMode( KFileDialog::Opening );
 
-    dlg.setMode( KFile::File | KFile::LocalOnly );
+    dlg.setMode( KFile::File | KFile::LocalOnly | KFile::ExistingOnly );
     dlg.setCaption(caption.isEmpty() ? i18n("Open") : caption);
 
     //dlg.d->ops->clearHistory();
@@ -445,15 +471,16 @@ QStringList KFileDialog::getOpenFileNames(const KUrl& startDir,
         return QFileDialog::getOpenFileNames(
             parent,
             caption.isEmpty() ? i18n("Open") : caption,
-            KFileDialogPrivate::Native::staticStartDir( startDir ).path(),
-            qtFilter(filter) );
+            KFileDialogPrivate::Native::staticStartDir( startDir ).toLocalFile(),
+            qtFilter( filter ) );
 // TODO use extra args?  QString * selectedFilter = 0, Options options = 0
     }
+    KFileDialogPrivate::Native::s_allowNative = false;
     KFileDialog dlg(startDir, filter, parent);
     dlg.setOperationMode( Opening );
 
     dlg.setCaption(caption.isEmpty() ? i18n("Open") : caption);
-    dlg.setMode(KFile::Files | KFile::LocalOnly);
+    dlg.setMode(KFile::Files | KFile::LocalOnly | KFile::ExistingOnly);
     //dlg.d->ops->clearHistory();
     dlg.exec();
 
@@ -468,11 +495,12 @@ KUrl KFileDialog::getOpenUrl(const KUrl& startDir, const QString& filter,
             startDir, filter, parent, caption) );
         return fileName.isEmpty() ? KUrl() : KUrl::fromPath(fileName);
     }
+    KFileDialogPrivate::Native::s_allowNative = false;
     KFileDialog dlg(startDir, filter, parent);
     dlg.setOperationMode( Opening );
 
     dlg.setCaption(caption.isEmpty() ? i18n("Open") : caption);
-    dlg.setMode( KFile::File );
+    dlg.setMode( KFile::File | KFile::ExistingOnly );
     //dlg.d->ops->clearHistory();
     dlg.exec();
 
@@ -489,24 +517,32 @@ KUrl::List KFileDialog::getOpenUrls(const KUrl& startDir,
             startDir, filter, parent, caption) );
         return KUrl::List(fileNames);
     }
+    KFileDialogPrivate::Native::s_allowNative = false;
     KFileDialog dlg(startDir, filter, parent);
     dlg.setOperationMode( Opening );
 
     dlg.setCaption(caption.isEmpty() ? i18n("Open") : caption);
-    dlg.setMode(KFile::Files);
+    dlg.setMode(KFile::Files | KFile::ExistingOnly);
     //dlg.d->ops->clearHistory();
     dlg.exec();
 
     return dlg.selectedUrls();
 }
 
+void KFileDialog::setConfirmOverwrite(bool enable)
+{
+    if (operationMode() == KFileDialog::Saving) {
+        d->w->setConfirmOverwrite(enable);
+    }
+}
+
 KUrl KFileDialog::getExistingDirectoryUrl(const KUrl& startDir,
                                           QWidget *parent,
                                           const QString& caption)
 {
     if (KFileDialogPrivate::isNative() && (!startDir.isValid() || startDir.isLocalFile())) {
         QString result( QFileDialog::getExistingDirectory(parent, caption,
-            KFileDialogPrivate::Native::staticStartDir( startDir ).path(),
+            KFileDialogPrivate::Native::staticStartDir( startDir ).toLocalFile(),
             QFileDialog::ShowDirsOnly) );
         return result.isEmpty() ? KUrl() : KUrl::fromPath(result);
     }
@@ -519,7 +555,7 @@ QString KFileDialog::getExistingDirectory(const KUrl& startDir,
 {
     if (KFileDialogPrivate::isNative() && (!startDir.isValid() || startDir.isLocalFile())) {
         return QFileDialog::getExistingDirectory(parent, caption,
-            KFileDialogPrivate::Native::staticStartDir( startDir ).path(),
+            KFileDialogPrivate::Native::staticStartDir( startDir ).toLocalFile(),
             QFileDialog::ShowDirsOnly);
     }
     KUrl url = fileModule()->selectDirectory(startDir, true, parent, caption);
@@ -536,15 +572,15 @@ KUrl KFileDialog::getImageOpenUrl( const KUrl& startDir, QWidget *parent,
         return KFileDialog::getOpenUrl(startDir, mimetypes.join(" "), parent, caption);
     }
     QStringList mimetypes = KImageIO::mimeTypes( KImageIO::Reading );
+    KFileDialogPrivate::Native::s_allowNative = false;
     KFileDialog dlg(startDir,
                     mimetypes.join(" "),
                     parent);
     dlg.setOperationMode( Opening );
     dlg.setCaption( caption.isEmpty() ? i18n("Open") : caption );
     dlg.setMode( KFile::File );
+    dlg.setInlinePreviewShown( true );
 
-    KImageFilePreview *ip = new KImageFilePreview( &dlg );
-    dlg.setPreviewWidget( ip );
     dlg.exec();
 
     return dlg.selectedUrl();
@@ -567,7 +603,7 @@ KUrl::List KFileDialog::selectedUrls() const
 QString KFileDialog::selectedFile() const
 {
     if (d->native)
-        return selectedUrl().path();
+        return selectedUrl().toLocalFile();
     return d->w->selectedFile();
 }
 
@@ -606,7 +642,7 @@ QString KFileDialog::getSaveFileName(const KUrl& dir, const QString& filter,
         const QString result = QFileDialog::getSaveFileName(
             parent,
             caption.isEmpty() ? i18n("Save As") : caption,
-            KFileDialogPrivate::Native::staticStartDir( startDir ).path(),
+            KFileDialogPrivate::Native::staticStartDir( startDir ).toLocalFile(),
             qtFilter(filter) );
 // TODO use extra args?     QString * selectedFilter = 0, Options options = 0
         if (!result.isEmpty()) {
@@ -622,7 +658,7 @@ QString KFileDialog::getSaveFileName(const KUrl& dir, const QString& filter,
     if ( !specialDir && !defaultDir ) {
         if (!dir.isLocalFile())
             kWarning() << "KFileDialog::getSaveFileName called with non-local start dir " << dir;
-        dlg.setSelection( dir.path() ); // may also be a filename
+        dlg.setSelection( dir.toLocalFile() ); // may also be a filename
     }
 
     dlg.setOperationMode( Saving );
@@ -649,17 +685,13 @@ QString KFileDialog::getSaveFileNameWId(const KUrl& dir, const QString& filter,
     bool specialDir = !defaultDir && dir.protocol() == "kfiledialog";
     QWidget* parent = QWidget::find( parent_id );
     KFileDialog dlg( specialDir ? dir : KUrl(), filter, parent);
-#ifdef Q_WS_X11
     if( parent == NULL && parent_id != 0 )
-        XSetTransientForHint(QX11Info::display(), dlg.winId(), parent_id);
-#else
-    // TODO
-#endif
+        KWindowSystem::setMainWindow( &dlg, parent_id);
 
     if ( !specialDir && !defaultDir ) {
         if (!dir.isLocalFile())
             kWarning() << "KFileDialog::getSaveFileNameWId called with non-local start dir " << dir;
-        dlg.setSelection( dir.path() ); // may also be a filename
+        dlg.setSelection( dir.toLocalFile() ); // may also be a filename
     }
 
     dlg.setOperationMode( Saving );
@@ -680,13 +712,14 @@ KUrl KFileDialog::getSaveUrl(const KUrl& dir, const QString& filter,
 {
     if (KFileDialogPrivate::isNative() && (!dir.isValid() || dir.isLocalFile())) {
         const QString fileName( KFileDialog::getSaveFileName(
-            dir.path(), filter, parent, caption) );
+            dir, filter, parent, caption) );
         return fileName.isEmpty() ? KUrl() : KUrl::fromPath(fileName);
     }
     bool defaultDir = dir.isEmpty();
     bool specialDir = !defaultDir && dir.protocol() == "kfiledialog";
+    KFileDialogPrivate::Native::s_allowNative = false;
     KFileDialog dlg(specialDir ? dir : KUrl(), filter, parent);
-    if ( !specialDir )
+    if ( !specialDir && !defaultDir )
         dlg.setSelection( dir.url() ); // may also be a filename
 
     dlg.setCaption(caption.isEmpty() ? i18n("Save As") : caption);
@@ -822,10 +855,13 @@ KAbstractFileWidget* KFileDialog::fileWidget()
 #ifdef Q_WS_WIN
 int KFileDialog::exec()
 {
-    if (!d->native)
-      return QDialog::exec();
+    if (!d->native || !KFileDialogPrivate::Native::s_allowNative) {
+        KFileDialogPrivate::Native::s_allowNative = true;
+        return KDialog::exec();
+    }
 
-//    d->native->selectedUrls.clear();
+// not clear here to let KFileDialogPrivate::Native::startDir() return a useful value
+// d->native->selectedUrls.clear();
     switch (d->native->operationMode) {
     case KAbstractFileWidget::Opening:
     case KAbstractFileWidget::Other:
@@ -834,6 +870,7 @@ int KFileDialog::exec()
                d->native->startDir(), d->native->filter, parentWidget(), windowTitle()) );
             if (url.isEmpty() || !url.isValid())
                 return QDialog::Rejected;
+            d->native->selectedUrls.clear();
             d->native->selectedUrls.append(url);
             return QDialog::Accepted;
         }
@@ -850,6 +887,7 @@ int KFileDialog::exec()
                 d->native->startDir(), parentWidget(), windowTitle()) );
             if (url.isEmpty() || !url.isValid())
                 return QDialog::Rejected;
+            d->native->selectedUrls.clear();
             d->native->selectedUrls.append(url);
             return QDialog::Accepted;
         }
@@ -860,6 +898,7 @@ int KFileDialog::exec()
                 d->native->startDir(), d->native->filter, parentWidget(), windowTitle()) );
             if (url.isEmpty() || !url.isValid())
                 return QDialog::Rejected;
+            d->native->selectedUrls.clear();
             d->native->selectedUrls.append(url);
             return QDialog::Accepted;
         }
@@ -868,6 +907,7 @@ int KFileDialog::exec()
                 d->native->startDir(), parentWidget(), windowTitle()) );
             if (url.isEmpty() || !url.isValid())
                 return QDialog::Rejected;
+            d->native->selectedUrls.clear();
             d->native->selectedUrls.append(url);
             return QDialog::Accepted;
         }
@@ -927,6 +967,17 @@ public:
             qt_filedialog_save_filename_hook=&getSaveFileName;
     }
 
+    ~KFileDialogQtOverride() {
+        if(qt_filedialog_existing_directory_hook == &getExistingDirectory)
+            qt_filedialog_existing_directory_hook = 0;
+        if(qt_filedialog_open_filename_hook == &getOpenFileName)
+            qt_filedialog_open_filename_hook = 0;
+        if(qt_filedialog_open_filenames_hook == &getOpenFileNames)
+            qt_filedialog_open_filenames_hook=0;
+        if(qt_filedialog_save_filename_hook == &getSaveFileName)
+            qt_filedialog_save_filename_hook=0;
+    }
+
     /*
      * Map a Qt filter string into a KDE one.
      */
@@ -934,8 +985,8 @@ public:
     {
         QString               filter;
         QTextStream           str(&filter, QIODevice::WriteOnly);
-        QStringList           list(f.split(";;"));
-        QStringList::Iterator it(list.begin()),
+        QStringList           list(f.split(";;").replaceInStrings("/", "\\/"));
+        QStringList::const_iterator it(list.begin()),
                               end(list.end());
         bool                  first=true;
 
@@ -965,7 +1016,7 @@ public:
         if(sel)
         {
             QStringList           list(orig.split(";;"));
-            QStringList::Iterator it(list.begin()),
+            QStringList::const_iterator it(list.begin()),
                                   end(list.end());
             int                   pos;
 
