@@ -18,8 +18,6 @@
 
 #include "global.h"
 #include "job.h"
-#include "thumbcreator.h"   // needed to create a reference to ThumbCreator class in the export lib
-                            // other solution: don't use KIO_EXPORT there...
 
 #include <config.h>
 
@@ -28,10 +26,8 @@
 #include <kglobal.h>
 #include <kiconloader.h>
 #include <kprotocolmanager.h>
-#include <kde_file.h>
 #include <kmimetype.h>
-#include <kwidgetjobtracker.h>
-#include <kuiserverjobtracker.h>
+#include <kdynamicjobtracker_p.h>
 
 #include <QtCore/QByteArray>
 #include <QtCore/QDate>
@@ -48,7 +44,7 @@
 #include <unistd.h>
 #include <stdio.h>
 
-K_GLOBAL_STATIC(KWidgetJobTracker, globalJobTracker)
+K_GLOBAL_STATIC(KDynamicJobTracker, globalJobTracker)
 
 // If someone wants the SI-standard prefixes kB/MB/GB/TB, I would recommend
 // a hidden kconfig option and getting the code from #57240 into the same
@@ -120,14 +116,29 @@ KIO_EXPORT QTime KIO::calculateRemaining( KIO::filesize_t totalSize, KIO::filesi
 
 KIO_EXPORT QString KIO::itemsSummaryString(uint items, uint files, uint dirs, KIO::filesize_t size, bool showSize)
 {
-    const QString itemsText = items == 0 ? i18n( "No Items" ) : i18np( "One Item", "%1 Items", items );
-    const QString filesText = files == 0 ? i18n( "No Files" ) : i18np( "One File", "%1 Files", files );
-    const QString foldersText = dirs == 0 ? i18n( "No Folders" ) : i18np("One Folder", "%1 Folders", dirs);
-    if ( showSize && files > 0 ) {
-        const QString sizeText = i18n("(%1 Total)", KIO::convertSize( size ) );
-        return i18nc("Items (Folders, Files), Size", "%1 (%2, %3), %4", itemsText, foldersText, filesText, sizeText);
+    if ( files == 0 && dirs == 0 && items == 0 ) {
+        return i18np( "%1 Item", "%1 Items", 0 );
     }
-    return i18nc("Items (Folders, Files)", "%1 (%2, %3)", itemsText, foldersText, filesText);
+    
+    QString summary;
+    const QString foldersText = i18np( "1 Folder", "%1 Folders", dirs );
+    const QString filesText = i18np( "1 File", "%1 Files", files );
+    if ( files > 0 && dirs > 0 ) {
+        summary = showSize ?
+                  i18nc( "folders, files (size)", "%1, %2 (%3)", foldersText, filesText, KIO::convertSize( size ) ) :
+                  i18nc( "folders, files", "%1, %2", foldersText, filesText );
+    } else if ( files > 0 ) {
+        summary = showSize ? i18nc( "files (size)", "%1 (%2)", filesText, KIO::convertSize( size ) ) : filesText;
+    } else if ( dirs > 0 ) {
+        summary = foldersText;
+    }
+    
+    if ( items > dirs + files ) {
+        const QString itemsText = i18np( "%1 Item", "%1 Items", items );
+        summary = summary.isEmpty() ? itemsText : i18nc( "items: folders, files (size)", "%1: %2", itemsText, summary );
+    }
+    
+    return summary;
 }
 
 KIO_EXPORT QString KIO::encodeFileName( const QString & _str )
@@ -457,7 +468,7 @@ KIO_EXPORT QStringList KIO::Job::detailedErrorStrings( const KUrl *reqUrl /*= 0L
 KIO_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorText,
                                const KUrl *reqUrl /*= 0L*/, int /*method = -1*/ )
 {
-  QString url, host, protocol, datetime, domain, path, dir, filename;
+  QString url, host, protocol, datetime, domain, path, filename;
   bool isSlaveNetwork = false;
   if ( reqUrl ) {
     url = reqUrl->prettyUrl();
@@ -469,39 +480,14 @@ KIO_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorTex
     else
       domain = host;
 
-    path = reqUrl->path(KUrl::AddTrailingSlash);
     filename = reqUrl->fileName();
-    dir =  path + filename;
+    path = reqUrl->path();
 
     // detect if protocol is a network protocol...
-    // add your hacks here...
-    if ( protocol == "http" ||
-         protocol == "https" ||
-         protocol == "ftp" ||
-         protocol == "sftp" ||
-         protocol == "webdav" ||
-         protocol == "webdavs" ||
-         protocol == "finger" ||
-         protocol == "fish" ||
-         protocol == "gopher" ||
-         protocol == "imap" ||
-         protocol == "imaps" ||
-         protocol == "lan" ||
-         protocol == "ldap" ||
-         protocol == "mailto" ||
-         protocol == "news" ||
-         protocol == "nntp" ||
-         protocol == "pop3" ||
-         protocol == "pop3s" ||
-         protocol == "smtp" ||
-         protocol == "smtps" ||
-         protocol == "telnet"
-        ) {
-      isSlaveNetwork = false;
-    }
+    isSlaveNetwork = KProtocolInfo::protocolClass(protocol) == ":internet";
   } else {
     // assume that the errorText has the location we are interested in
-    url = host = domain = path = filename = dir = errorText;
+    url = host = domain = path = filename = errorText;
     protocol = i18nc("@info protocol", "(unknown)" );
   }
 
@@ -562,7 +548,7 @@ KIO_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorTex
       errorName = i18n( "Cannot Open Resource For Reading" );
       description = i18n( "This means that the contents of the requested file "
         "or folder <strong>%1</strong> could not be retrieved, as read "
-        "access could not be obtained." ,  dir );
+        "access could not be obtained.", path );
       causes << i18n( "You may not have permissions to read the file or open "
         "the folder.") << cLocked << cHardware;
       solutions << sAccess << sQuerylock << sSysadmin;
@@ -653,7 +639,7 @@ KIO_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorTex
     case  KIO::ERR_IS_DIRECTORY:
       errorName = i18n( "File Expected" );
       description = i18n( "The request expected a file, however the "
-        "folder <strong>%1</strong> was found instead." ,  dir );
+        "folder <strong>%1</strong> was found instead." , path );
       causes << i18n( "This may be an error on the server side." ) << cBug;
       solutions << sUpdate << sSysadmin;
       break;
@@ -661,7 +647,7 @@ KIO_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorTex
     case  KIO::ERR_IS_FILE:
       errorName = i18n( "Folder Expected" );
       description = i18n( "The request expected a folder, however "
-        "the file <strong>%1</strong> was found instead." ,  filename );
+        "the file <strong>%1</strong> was found instead." , filename );
       causes << cBug;
       solutions << sUpdate << sSysadmin;
       break;
@@ -669,7 +655,7 @@ KIO_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorTex
     case  KIO::ERR_DOES_NOT_EXIST:
       errorName = i18n( "File or Folder Does Not Exist" );
       description = i18n( "The specified file or folder <strong>%1</strong> "
-        "does not exist." ,  dir );
+        "does not exist." , path );
       causes << cBug;
       solutions << sUpdate << sSysadmin;
       break;
@@ -730,7 +716,7 @@ KIO_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorTex
       errorName = i18n( "Unable to Enter Folder" );
       description = i18n( "This means that an attempt to enter (in other words, "
         "to open) the requested folder <strong>%1</strong> was rejected." ,
-          dir );
+          path );
       causes << cAccess << cLocked;
       solutions << sAccess << sQuerylock << sSysadmin;
       break;
@@ -818,7 +804,7 @@ KIO_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorTex
         "<strong>L</strong>ocator (URL) that you entered did not refer to "
         "a valid mechanism of accessing the specific resource, "
         "<strong>%1%2</strong>." ,
-          !host.isNull() ? host + '/' : QString() ,  dir );
+          !host.isNull() ? host + '/' : QString() , path );
       causes << i18n( "KDE is able to communicate through a protocol within a "
         "protocol. This request specified a protocol be used as such, however "
         "this protocol is not capable of such an action. This is a rare event, "
@@ -963,7 +949,7 @@ KIO_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorTex
     case  KIO::ERR_COULD_NOT_RMDIR:
       errorName = i18n( "Could Not Remove Folder" );
       description = i18n( "An attempt to remove the specified folder, "
-        "<strong>%1</strong>, failed." ,  dir );
+        "<strong>%1</strong>, failed." , path );
       causes << i18n( "The specified folder may not exist." )
         << i18n( "The specified folder may not be empty." )
         << cAccess;
@@ -1090,7 +1076,7 @@ KIO_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorTex
         "<li>Timeout for receiving a response: %2 seconds</li>"
         "<li>Timeout for accessing proxy servers: %3 seconds</li></ul>"
         "Please note that you can alter these timeout settings in the KDE "
-        "Control Center, by selecting Network -> Preferences." ,
+        "System Settings, by selecting Network Settings -> Connection Preferences." ,
           KProtocolManager::connectTimeout() ,
           KProtocolManager::responseTimeout() ,
           KProtocolManager::proxyConnectTimeout() );
@@ -1258,7 +1244,45 @@ KJobTrackerInterface *KIO::getJobTracker()
 }
 
 
+/***************************************************************
+ *
+ * KIO::MetaData
+ *
+ ***************************************************************/
+KIO::MetaData::MetaData(const QMap<QString,QVariant>& map)
+{
+  *this = map;
+}
 
+KIO::MetaData & KIO::MetaData::operator += ( const QMap<QString,QVariant> &metaData )
+{
+  QMap<QString,QVariant>::ConstIterator it;
+  
+  for(it = metaData.constBegin(); it !=  metaData.constEnd(); ++it)
+     insert(it.key(), it.value().toString());
 
+  return *this;
+}
 
+KIO::MetaData & KIO::MetaData::operator = ( const QMap<QString,QVariant> &metaData )
+{
+  clear();
 
+  QMap<QString,QVariant>::ConstIterator it;
+  for(it = metaData.constBegin(); it !=  metaData.constEnd(); ++it)
+     insert(it.key(), it.value().toString());
+
+  return *this;
+}
+
+QVariant KIO::MetaData::toVariant() const
+{
+ QMap<QString, QVariant> map;
+ QMap<QString,QString>::ConstIterator it;
+ QMap<QString,QString>::ConstIterator itEnd = constEnd();
+
+ for(it = constBegin(); it != itEnd; ++it)
+   map.insert(it.key(), it.value());
+
+ return QVariant(map);
+}
