@@ -1,7 +1,9 @@
 /*
- * Copyright (C) 2000 Alex Zepeda <zipzippy@sonic.net
+ * Copyright (C) 2000 Alex Zepeda <zipzippy@sonic.net>
  * Copyright (C) 2001-2003 George Staikos <staikos@kde.org>
  * Copyright (C) 2001 Dawit Alemayehu <adawit@kde.org>
+ * Copyright (C) 2007,2008 Andreas Hartmetz <ahartmetz@gmail.com>
+ * Copyright (C) 2008 Roland Harnau <tau@gmx.eu>
  *
  * This file is part of the KDE project
  *
@@ -45,6 +47,7 @@
 
 #include <klocale.h>
 #include <QtCore/QDataStream>
+#include <QtCore/QTime>
 #include <QtNetwork/QTcpSocket>
 #include <QtNetwork/QHostInfo>
 #include <QtDBus/QtDBus>
@@ -60,6 +63,10 @@ using namespace KIO;
 typedef QMap<QString, QString> StringStringMap;
 Q_DECLARE_METATYPE(StringStringMap)
 
+namespace KIO {
+Q_DECLARE_OPERATORS_FOR_FLAGS(TCPSlaveBase::SslResult)
+}
+
 //TODO Proxy support whichever way works; KPAC reportedly does *not* work.
 //NOTE kded_proxyscout may or may not be interesting
 
@@ -71,15 +78,12 @@ Q_DECLARE_METATYPE(StringStringMap)
 
 //TODO check if d->isBlocking is honored everywhere it makes sense
 
-//TODO fold KSSLSetting and KSSLCertificateHome into KSslSettings, and figure out how to manipulate it.
+//TODO fold KSSLSetting and KSSLCertificateHome into KSslSettings and use that everywhere.
 
 //TODO recognize partially encrypted websites as "somewhat safe"
 
-//### isAutoSsl() - the name is pretty stupid. isSslFromStart()? isSslAfterConnect()?
-
 /* List of dialogs/messageboxes we need to use (current code location in parentheses)
-   Can the design of the "dontAskAgainName" thing be improved?
-   Ugh, there *is* some code for application side messageboxes. Use it.
+ - Can the "dontAskAgainName" thing be improved?
 
  - "SSLCertDialog" [select client cert] (SlaveInterface)
  - Enter password for client certificate (inline)
@@ -99,27 +103,64 @@ Q_DECLARE_METATYPE(StringStringMap)
 class TCPSlaveBase::TcpSlaveBasePrivate
 {
 public:
+    TcpSlaveBasePrivate(TCPSlaveBase* qq) : q(qq) {}
 
-    TcpSlaveBasePrivate()
-     : autoSSL(false),
-       sslNoUi(false)
-    {}
-    ~TcpSlaveBasePrivate() {}
-
-    QList<KSslError> nonIgnorableErrors(const QList<KSslError> &e) const
+    void prepareSslRelatedMetaData()
     {
-        QList<KSslError> ret;
-        //TODO :)
-        return ret;
+        KSslCipher cipher = socket.sessionCipher();
+        q->setMetaData("ssl_protocol_version", socket.negotiatedSslVersionName());
+        QString sslCipher = cipher.encryptionMethod() + '\n';
+        sslCipher += cipher.authenticationMethod() + '\n';
+        sslCipher += cipher.keyExchangeMethod() + '\n';
+        sslCipher += cipher.digestMethod();
+        q->setMetaData("ssl_cipher", sslCipher);
+        q->setMetaData("ssl_cipher_used_bits", QString::number(cipher.usedBits()));
+        q->setMetaData("ssl_cipher_bits", QString::number(cipher.supportedBits()));
+        q->setMetaData("ssl_peer_ip", ip);
+
+        // try to fill in the blanks, i.e. missing certificates, and just assume that
+        // those belong to the peer (==website or similar) certificate.
+        for (int i = 0; i < sslErrors.count(); i++) {
+            if (sslErrors[i].certificate().isNull()) {
+                sslErrors[i] = KSslError(sslErrors[i].error(),
+                                        socket.peerCertificateChain()[0]);
+            }
+        }
+
+        QString errorStr;
+        // encode the two-dimensional numeric error list using '\n' and '\t' as outer and inner separators
+        foreach (const QSslCertificate &cert, socket.peerCertificateChain()) {
+            foreach (const KSslError &error, sslErrors) {
+                if (error.certificate() == cert) {
+                    errorStr += QString::number(static_cast<int>(error.error())) + '\t';
+                }
+            }
+            if (errorStr.endsWith('\t')) {
+                errorStr.chop(1);
+            }
+            errorStr += '\n';
+        }
+        errorStr.chop(1);
+        q->setMetaData("ssl_cert_errors", errorStr);
+
+        QString peerCertChain;
+        foreach (const QSslCertificate &cert, socket.peerCertificateChain()) {
+            peerCertChain.append(cert.toPem());
+            peerCertChain.append('\x01');
+        }
+        peerCertChain.chop(1);
+        q->setMetaData("ssl_peer_chain", peerCertChain);
     }
 
-    QString host;
-    QString ip;
-    KTcpSocket socket;
+    TCPSlaveBase* q;
 
     int timeout;
     bool isBlocking;
 
+    KTcpSocket socket;
+
+    QString host;
+    QString ip;
     quint16 port;
     QByteArray serviceName;
 
@@ -128,6 +169,7 @@ public:
     bool autoSSL;
     bool sslNoUi; // If true, we just drop the connection silently
                   // if SSL certificate check fails in some way.
+    QList<KSslError> sslErrors;
 };
 
 
@@ -143,13 +185,15 @@ TCPSlaveBase::TCPSlaveBase(const QByteArray &protocol,
                            const QByteArray &appSocket,
                            bool autoSSL)
  : SlaveBase(protocol, poolSocket, appSocket),
-   d(new TcpSlaveBasePrivate)
+   d(new TcpSlaveBasePrivate(this))
 {
+    d->timeout = KProtocolManager::connectTimeout();
+    d->isBlocking = true;
+    d->port = 0;
     d->serviceName = protocol;
     d->usingSSL = false;
-    d->timeout = KProtocolManager::connectTimeout();
-    d->isBlocking = false;
     d->autoSSL = autoSSL;
+    d->sslNoUi = false;
 }
 
 
@@ -163,7 +207,7 @@ ssize_t TCPSlaveBase::write(const char *data, ssize_t len)
 {
     ssize_t written = d->socket.write(data, len);
     if (written == -1) {
-        kDebug(7029) << "d->socket.write() returned -1! Socket error is"
+        kDebug(7027) << "d->socket.write() returned -1! Socket error is"
                      << d->socket.error() << ", Socket state is" << d->socket.state();
     }
 
@@ -178,10 +222,10 @@ ssize_t TCPSlaveBase::write(const char *data, ssize_t len)
         success = d->socket.waitForBytesWritten(0);
     }
 
-    d->socket.flush();  //this *ahem* might help to get the data on the wire faster
+    d->socket.flush();  //this is supposed to get the data on the wire faster
 
-    if (d->socket.state() != KTcpSocket::ConnectedState && !success) {
-        kDebug(7029) << "Write failed, will return -1! Socket error is"
+    if (d->socket.state() != KTcpSocket::ConnectedState || !success) {
+        kDebug(7027) << "Write failed, will return -1! Socket error is"
                      << d->socket.error() << ", Socket state is" << d->socket.state()
                      << "Return value of waitForBytesWritten() is" << success;
         return -1;
@@ -199,11 +243,16 @@ ssize_t TCPSlaveBase::read(char* data, ssize_t len)
         return -1;
     }
 
-    if (d->isBlocking) {
-        if (!d->socket.bytesAvailable()) {
+    if (!d->socket.bytesAvailable()) {
+        if (d->isBlocking) {
             d->socket.waitForReadyRead(-1);
+        } else {
+            d->socket.waitForReadyRead(0);
         }
-    } else {
+    } else if (d->socket.encryptionMode() != KTcpSocket::SslClientMode ||
+               QNetworkProxy::applicationProxy().type() == QNetworkProxy::NoProxy) {
+        // we only do this when it doesn't trigger Qt socket bugs. When it doesn't break anything
+        // it seems to help performance.
         d->socket.waitForReadyRead(0);
     }
 
@@ -233,7 +282,7 @@ ssize_t TCPSlaveBase::readLine(char *data, ssize_t len)
         if (!d->socket.bytesAvailable())
             d->socket.waitForReadyRead(-1);
         ssize_t readStep = d->socket.readLine(&data[readTotal], len-readTotal);
-        if (readStep == -1) {
+        if (readStep == -1 || (readStep == 0 && d->socket.state() != KTcpSocket::ConnectedState)) {
             return -1;
         }
         readTotal += readStep;
@@ -243,12 +292,10 @@ ssize_t TCPSlaveBase::readLine(char *data, ssize_t len)
 }
 
 
-bool TCPSlaveBase::connectToHost(const QString &protocol,
+bool TCPSlaveBase::connectToHost(const QString &/*protocol*/,
                                  const QString &host,
                                  quint16 port)
 {
-    disconnectFromHost();  //Reset some state, even if we are already disconnected
-
     setMetaData("ssl_in_use", "FALSE"); //We have separate connection and SSL setup phases
 
     //  - leaving SSL - warn before we even connect
@@ -271,67 +318,93 @@ bool TCPSlaveBase::connectToHost(const QString &protocol,
                                     "WarnOnLeaveSSLMode");
 
             if (result == KMessageBox::Cancel) {
+                error(ERR_USER_CANCELED, host);
                 return false;
             }
         }
     }
 
-    d->host = host;
+    KTcpSocket::SslVersion trySslVersion = KTcpSocket::TlsV1;
+    while (true) {
+        disconnectFromHost();  //Reset some state, even if we are already disconnected
+        d->host = host;
 
-    //FIXME! KTcpSocket doesn't know or care about protocol ports! Fix it there, then use it here.
+        //FIXME! KTcpSocket doesn't know or care about protocol ports! Fix it there, then use it here.
 
-    kDebug(7029) << "before connectToHost: Socket error is "
-                 << d->socket.error() << ", Socket state is " << d->socket.state();
-    d->socket.connectToHost(host, port);
-    kDebug(7029) << "after connectToHost: Socket error is "
-                 << d->socket.error() << ", Socket state is " << d->socket.state();
+        QList<QHostAddress> addresses;
 
-    bool connectOk = d->socket.waitForConnected(d->timeout > -1 ? d->timeout * 1000 : -1);
-    kDebug(7029) << "after waitForConnected: Socket error is "
-                 << d->socket.error() << ", Socket state is " << d->socket.state()
-                 << ", waitForConnected returned " << connectOk;
-
-    if (d->socket.state() != KTcpSocket::ConnectedState) {
-        if (d->socket.error() == KTcpSocket::HostNotFoundError) {
-            error(ERR_UNKNOWN_HOST,
-                  host + QLatin1String(": ") + d->socket.errorString());
+        QHostAddress address;
+        if (address.setAddress(host)) {
+            addresses.append(address);
         } else {
-            error(ERR_COULD_NOT_CONNECT,
-                  host + QLatin1String(": ") + d->socket.errorString());
+            QHostInfo info;
+            lookupHost(host);
+            waitForHostInfo(info);
+            if (info.error() != QHostInfo::NoError) {
+                error(ERR_UNKNOWN_HOST, host);
+                return false;
+            }
+            addresses = info.addresses();
         }
-        return false;
-    }
 
-    //### check for proxyAuthenticationRequiredError
-
-    d->ip = d->socket.peerAddress().toString();
-    d->port = d->socket.peerPort();
-
-    if (d->autoSSL) {
-        SslResult res = startTLSInternal();
-        if (res == ResultFailed) {
-            //### more?
-            //TODO proper i18n, maybe a special error code.
-            error(ERR_COULD_NOT_CONNECT, 
-                  host + i18n(": SSL negotiation failed"));
-            return false;
+        QListIterator<QHostAddress> it(addresses);
+        int timeout = d->timeout * 1000;
+        QTime time;
+        forever {
+            time.start();
+            d->socket.connectToHost(it.next(), port);
+            if (d->socket.waitForConnected(timeout)) {
+                break;
+            }
+            timeout -= time.elapsed();
+            if (!it.hasNext() || (timeout < 0)) {
+                error(ERR_COULD_NOT_CONNECT,
+                      host + QLatin1String(": ") + d->socket.errorString());
+                return false;
+            }
         }
-    }
 
-    return true;
+        //### check for proxyAuthenticationRequiredError
+
+        d->ip = d->socket.peerAddress().toString();
+        d->port = d->socket.peerPort();
+
+        if (d->autoSSL) {
+            SslResult res = startTLSInternal(trySslVersion);
+            if ((res & ResultFailed) && (res & ResultFailedEarly)
+                && (trySslVersion == KTcpSocket::TlsV1)) {
+                trySslVersion = KTcpSocket::SslV3;
+                continue;
+                //### SSL 2.0 is (close to) dead and it's a good thing, too.
+            }
+            if (res & ResultFailed) {
+                error(ERR_COULD_NOT_CONNECT,
+                      i18nc("%1 is a host name", "%1: SSL negotiation failed", host));
+                return false;
+            }
+        }
+        return true;
+    }
+    Q_ASSERT(false);
 }
 
 void TCPSlaveBase::disconnectFromHost()
 {
-    kDebug(7029);
-    d->ip.clear();
+    kDebug(7027);
     d->host.clear();
+    d->ip.clear();
     d->usingSSL = false;
 
-    if (d->socket.state() == KTcpSocket::UnconnectedState)
+    if (d->socket.state() == KTcpSocket::UnconnectedState) {
+        // discard incoming data - the remote host might have disconnected us in the meantime
+        // but the visible effect of disconnectFromHost() should stay the same.
+        d->socket.close();
         return;
+    }
 
-    //stopTLS(); ### this is a reminder to save a session for reuse or do whatever on SSL shutdown
+    //### maybe save a session for reuse on SSL shutdown if and when QSslSocket
+    //    does that. QCA::TLS can do it apparently but that is not enough if
+    //    we want to present that as KDE API. Not a big loss in any case.
     d->socket.disconnectFromHost();
     if (d->socket.state() != KTcpSocket::UnconnectedState)
         d->socket.waitForDisconnected(-1); // wait for unsent data to be sent
@@ -362,12 +435,13 @@ bool TCPSlaveBase::startSsl()
 {
     if (d->usingSSL)
         return false;
-    return startTLSInternal() != ResultFailed;
+    return startTLSInternal(KTcpSocket::TlsV1) & ResultOk;
 }
 
 
-TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal()
+TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal(uint v_)
 {
+    KTcpSocket::SslVersion sslVersion = static_cast<KTcpSocket::SslVersion>(v_);
     selectClientCertificate();
 
     //setMetaData("ssl_session_id", d->kssl->session()->toString());
@@ -376,10 +450,12 @@ TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal()
     d->usingSSL = true;
     setMetaData("ssl_in_use", "TRUE");
 
+    d->socket.setAdvertisedSslVersion(sslVersion);
+
     /* Usually ignoreSslErrors() would be called in the slot invoked by the sslErrors()
        signal but that would mess up the flow of control. We will check for errors
        anyway to decide if we want to continue connecting. Otherwise ignoreSslErrors()
-       before connecting would be rather insecure. */
+       before connecting would be very insecure. */
     d->socket.ignoreSslErrors();
     d->socket.startClientEncryption();
     const bool encryptionStarted = d->socket.waitForEncrypted(-1);
@@ -388,74 +464,73 @@ TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal()
     KSslCipher cipher = d->socket.sessionCipher();
 
     if (!encryptionStarted || d->socket.encryptionMode() != KTcpSocket::SslClientMode
-        || cipher.isNull() || cipher.usedBits() == 0) {
-         //TODO error(foo, bar);
+        || cipher.isNull() || cipher.usedBits() == 0 || d->socket.peerCertificateChain().isEmpty()) {
         d->usingSSL = false;
         setMetaData("ssl_in_use", "FALSE");
         kDebug(7029) << "Initial SSL handshake failed. encryptionStarted is"
                      << encryptionStarted << ", cipher.isNull() is" << cipher.isNull()
                      << ", cipher.usedBits() is" << cipher.usedBits()
+                     << ", length of certificate chain is" << d->socket.peerCertificateChain().count()
                      << ", the socket says:" << d->socket.errorString()
                      << "and the list of SSL errors contains"
                      << d->socket.sslErrors().count() << "items.";
-        return ResultFailed;
+        return ResultFailed | ResultFailedEarly;
     }
 
     kDebug(7029) << "Cipher info - "
+                 << " advertised SSL protocol version" << d->socket.advertisedSslVersion()
+                 << " negotiated SSL protocol version" << d->socket.negotiatedSslVersion()
                  << " authenticationMethod:" << cipher.authenticationMethod()
                  << " encryptionMethod:" << cipher.encryptionMethod()
                  << " keyExchangeMethod:" << cipher.keyExchangeMethod()
                  << " name:" << cipher.name()
-                 << " protocol:" << cipher.protocol()
-                 << " protocolName:" << cipher.protocolName()
                  << " supportedBits:" << cipher.supportedBits()
                  << " usedBits:" << cipher.usedBits();
 
-    setMetaData("ssl_protocol_version", cipher.protocolName());
-    QString sslCipher = cipher.encryptionMethod() + '\n';
-    sslCipher += cipher.authenticationMethod() + '\n';
-    sslCipher += cipher.keyExchangeMethod() + '\n';
-    sslCipher += cipher.digestMethod();
-    setMetaData("ssl_cipher", sslCipher);
-    setMetaData("ssl_cipher_used_bits", QString::number(cipher.usedBits()));
-    setMetaData("ssl_cipher_bits", QString::number(cipher.supportedBits()));
-    setMetaData("ssl_peer_ip", d->ip);
-
-    //TODO Qt 4.4 actually has non-null certificates in QSslError::certificate() so
-    //     we should use that then
-    QString errorStr;
-    foreach(const KSslError &se, d->socket.sslErrors())
-        errorStr += QString::number(static_cast<int>(se.error())) + '\n';
-    setMetaData("ssl_cert_errors", errorStr);
-
-    QString peerCertChain;
-    foreach (const QSslCertificate &cert, d->socket.peerCertificateChain()) {
-        peerCertChain.append(cert.toPem());
-        peerCertChain.append('\x01');
+    // Since we connect by IP (cf. KIO::HostInfo) the SSL code will not recognize
+    // that the site certificate belongs to the domain. We therefore do the
+    // domain<->certificate matching here.
+    d->sslErrors = d->socket.sslErrors();
+    QSslCertificate peerCert = d->socket.peerCertificateChain().first();
+    QStringList domainPatterns(peerCert.subjectInfo(QSslCertificate::CommonName));
+    domainPatterns += peerCert.alternateSubjectNames().values(QSsl::DnsEntry);
+    QRegExp domainMatcher(QString(), Qt::CaseInsensitive, QRegExp::Wildcard);
+    QMutableListIterator<KSslError> it(d->sslErrors);
+    while (it.hasNext()) {
+        // As of 4.4.0 Qt does not assign a certificate to the QSslError it emits
+        // *in the case of HostNameMismatch*. A HostNameMismatch, however, will always
+        // be an error of the peer certificate so we just don't check the error's
+        // certificate().
+        if (it.next().error() != KSslError::HostNameMismatch) {
+            continue;
+        }
+        foreach (const QString &dp, domainPatterns) {
+            domainMatcher.setPattern(dp);
+            if (domainMatcher.exactMatch(d->host)) {
+                it.remove();
+            }
+        }
     }
-    peerCertChain.chop(1);
-    setMetaData("ssl_peer_chain", peerCertChain);
-    //The metadata is usually sent automatically before any payload data (including that of
-    //messageBox()) so *theroretically* we just assume that metadata will make it to the other
-    //side when needed.
-    //In practice, calling sendMetaData() here
-    //a) makes the SSL info dialog work
-    //b) breaks Konqueror which then never displays the
-    //   SSL shield icon.
-    //Until I have figured out how to unbreak metadata semantics let's have a working SSL info
-    //dialog and no shield icon in Konqueror. If anyone feels like implementing hacks to
-    //make it work right now - go ahead, have [air quotes here] fun.
-    //Gold star of KDE for fixing it right!
-    sendMetaData();
+
+    // The app side needs the metadata now for the SSL error dialog (if any) but
+    // the same metadata will be needed later, too. When "later" arrives the slave
+    // may actually be connected to a different application that doesn't know
+    // the metadata the slave sent to the previous application.
+    // The quite important SSL indicator icon in Konqi's URL bar relies on metadata
+    // from here, for example. And Konqi will be the second application to connect
+    // to the slave.
+    // Therefore we choose to have our metadata and send it, too :)
+    d->prepareSslRelatedMetaData();
+    sendAndKeepMetaData();
 
     SslResult rc = verifyServerCertificate();
-    if (rc == ResultFailed) {
+    if (rc & ResultFailed) {
         d->usingSSL = false;
         setMetaData("ssl_in_use", "FALSE");
         kDebug(7029) << "server certificate verification failed.";
         d->socket.disconnectFromHost();     //Make the connection fail (cf. ignoreSslErrors())
         return ResultFailed;
-    } else if (rc == ResultOverridden) {
+    } else if (rc & ResultOverridden) {
         kDebug(7029) << "server certificate verification failed but continuing at user's request.";
     }
 
@@ -482,7 +557,6 @@ TCPSlaveBase::SslResult TCPSlaveBase::startTLSInternal()
     return rc;
 }
 
-
 void TCPSlaveBase::selectClientCertificate()
 {
 #if 0 //hehe
@@ -562,7 +636,7 @@ void TCPSlaveBase::selectClientCertificate()
     if (prompt || forcePrompt) {
         QStringList certs = KSSLCertificateHome::getCertificateList();
 
-        QStringList::Iterator it = certs.begin();
+        QStringList::const_iterator it = certs.begin();
         while (it != certs.end()) {
             KSSLPKCS12 *pkcs = KSSLCertificateHome::getCertificateByName(*it);
             if (pkcs && (!pkcs->getCertificate() ||
@@ -654,63 +728,40 @@ void TCPSlaveBase::selectClientCertificate()
 #endif
 }
 
-
 TCPSlaveBase::SslResult TCPSlaveBase::verifyServerCertificate()
 {
     d->sslNoUi = hasMetaData("ssl_no_ui") && (metaData("ssl_no_ui") != "FALSE");
 
-    QList<KSslError> se = d->socket.sslErrors();
-    if (se.isEmpty())
+    if (d->sslErrors.isEmpty()) {
         return ResultOk;
-
-    if (d->sslNoUi)
+    } else if (d->sslNoUi) {
         return ResultFailed;
-
-    QString message = i18n("The server failed the authenticity check (%1)\n\n",
-                           d->host);
-
-    foreach (const KSslError &err, se) {
-        //### use our own wording that is "closer to the user"
-        message.append(err.errorString());
-        message.append('\n');
     }
 
-    //### Consider that hostname mismatch and e.g. an expired certificate are very different.
-    //    Maybe there should be no option to acceptForever a cert with bad hostname.
-
-    //### the truth of the following comment is disputed
-    //### The old code also checks if the CN (==domain name, here) matches the actual peer IP.
-    //This is not a feature of the SSL infrastructure itself so we need to do some work.
-
-    /* We need a list of ignorable errors. I don't think it makes sense to ignore
-       malformed certificates, for example, as other environments probably don't do
-       that either. It would be similar to a compiler trying to correct syntax errors.
-       There are also hard errors that are just impossible to override.
-      Ignorable so far, determined by watching real connection errors:
-        -UnableToGetLocalIssuerCertificate
-        -CertificateUntrusted
-        -UnableToVerifyFirstCertificate
-    */
-
-    KSslCertificateManager *const cm = KSslCertificateManager::self();
-    KSslCertificateRule rule = cm->rule(d->socket.peerCertificateChain().first(), d->host);
-
-    //TODO put nonIgnorableErrors into the cert manager
-    QList<KSslError> fatalErrors = d->nonIgnorableErrors(se);
+    QList<KSslError> fatalErrors = KSslCertificateManager::nonIgnorableErrors(d->sslErrors);
     if (!fatalErrors.isEmpty()) {
         //TODO message "sorry, fatal error, you can't override it"
         return ResultFailed;
     }
 
-    //throw out previously seen errors that are supposed to be ignored.
-    se = rule.filterErrors(se);
-    if (se.isEmpty()) {
+    KSslCertificateManager *const cm = KSslCertificateManager::self();
+    KSslCertificateRule rule = cm->rule(d->socket.peerCertificateChain().first(), d->host);
+
+    // remove previously seen and acknowledged errors
+    QList<KSslError> remainingErrors = rule.filterErrors(d->sslErrors);
+    if (remainingErrors.isEmpty()) {
         kDebug(7029) << "Error list empty after removing errors to be ignored. Continuing.";
-        return ResultOverridden;
+        return ResultOk | ResultOverridden;
     }
 
-    //### We don't present the option to permanently reject the certificate even though
-    //    it would be technically possible.
+    //### We don't ask to permanently reject the certificate
+
+    QString message = i18n("The server failed the authenticity check (%1).\n\n", d->host);
+    foreach (const KSslError &err, d->sslErrors) {
+        message.append(err.errorString());
+        message.append('\n');
+    }
+    message = message.trimmed();
 
     int msgResult;
     do {
@@ -718,8 +769,8 @@ TCPSlaveBase::SslResult TCPSlaveBase::verifyServerCertificate()
                                i18n("Server Authentication"),
                                i18n("&Details"), i18n("Co&ntinue"));
         if (msgResult == KMessageBox::Yes) {
-            //Details was chosen - we pop up the details dialog and present the choices again
-            messageBox(SSLMessageBox /*==the SSL info dialog*/, d->host);
+            //Details was chosen- show the certificate and error details
+            messageBox(SSLMessageBox /*the SSL info dialog*/, d->host);
         } else if (msgResult == KMessageBox::Cancel) {
             return ResultFailed;
         }
@@ -734,13 +785,13 @@ TCPSlaveBase::SslResult TCPSlaveBase::verifyServerCertificate()
                                  "being prompted?"),
                             i18n("Server Authentication"),
                             i18n("&Forever"),
-                            i18n("&Current Sessions Only"));
+                            i18n("&Current Session only"));
     QDateTime ruleExpiry = QDateTime::currentDateTime();
     if (msgResult == KMessageBox::Yes) {
-        //accept forever (we interpret this as "for a very long time")
+        //accept forever ("for a very long time")
         ruleExpiry = ruleExpiry.addYears(1000);
     } else {
-        //accept for a short time, say half an hour.
+        //accept "for a short time", half an hour.
         ruleExpiry = ruleExpiry.addSecs(30*60);
     }
 
@@ -748,10 +799,10 @@ TCPSlaveBase::SslResult TCPSlaveBase::verifyServerCertificate()
     //rule = KSslCertificateRule(d->socket.peerCertificateChain().first(), whatever);
 
     rule.setExpiryDateTime(ruleExpiry);
-    rule.setIgnoredErrors(se);
+    rule.setIgnoredErrors(d->sslErrors);
     cm->setRule(rule);
 
-    return ResultOverridden;
+    return ResultOk | ResultOverridden;
 #if 0 //### need to to do something like the old code about the main and subframe stuff
     kDebug(7029) << "SSL HTTP frame the parent? " << metaData("main_frame_request");
     if (!hasMetaData("main_frame_request") || metaData("main_frame_request") == "TRUE") {
@@ -855,7 +906,7 @@ TCPSlaveBase::SslResult TCPSlaveBase::verifyServerCertificate()
 
     return rc;
 #endif //#if 0
-    return ResultOk;
+    return ResultOk | ResultOverridden;
 }
 
 
@@ -876,6 +927,10 @@ bool TCPSlaveBase::waitForResponse(int t)
 
 void TCPSlaveBase::setBlocking(bool b)
 {
+    if (!b) {
+        kWarning(7029) << "Caller requested non-blocking mode, but that doesn't work";
+        return;
+    }
     d->isBlocking = b;
 }
 
@@ -885,4 +940,3 @@ void TCPSlaveBase::virtual_hook(int id, void* data)
 }
 
 
-#include "tcpslavebase.moc"
