@@ -23,8 +23,6 @@
    Boston, MA 02110-1301, USA.
 */
 
-#define _WIN32_WINNT 0x0500
-
 #include "file.h"
 
 #include <windows.h>
@@ -56,6 +54,54 @@ static DWORD CALLBACK CopyProgressRoutine(
     return PROGRESS_CONTINUE;
 }
 
+static UDSEntry createUDSEntryWin( const QFileInfo &fileInfo )
+{
+    UDSEntry entry;
+
+    entry.insert( KIO::UDSEntry::UDS_NAME, fileInfo.fileName() );
+    if( fileInfo.isSymLink() ) {
+        entry.insert( KIO::UDSEntry::UDS_TARGET_URL, fileInfo.symLinkTarget() );
+/* TODO - or not useful on windows?
+        if ( details > 1 ) {
+            // It is a link pointing to nowhere
+            type = S_IFMT - 1;
+            access = S_IRWXU | S_IRWXG | S_IRWXO;
+
+            entry.insert( KIO::UDSEntry::UDS_FILE_TYPE, type );
+            entry.insert( KIO::UDSEntry::UDS_ACCESS, access );
+            entry.insert( KIO::UDSEntry::UDS_SIZE, 0LL );
+            goto notype;
+
+        }
+*/
+    }
+    int type = S_IFREG;
+    int access = 0;
+    if( fileInfo.isDir() )
+        type = S_IFDIR;
+    else if( fileInfo.isSymLink() )
+        type = S_IFLNK;
+    if( fileInfo.isReadable() )
+        access |= S_IRUSR;
+    if( fileInfo.isWritable() )
+        access |= S_IWUSR;
+    if( fileInfo.isExecutable() )
+        access |= S_IXUSR;
+
+    entry.insert( KIO::UDSEntry::UDS_FILE_TYPE, type );
+    entry.insert( KIO::UDSEntry::UDS_ACCESS, access );
+    entry.insert( KIO::UDSEntry::UDS_SIZE, fileInfo.size() );
+    if( fileInfo.isHidden() )
+      entry.insert( KIO::UDSEntry::UDS_HIDDEN, true );
+
+    entry.insert( KIO::UDSEntry::UDS_MODIFICATION_TIME, fileInfo.lastModified().toTime_t() );
+    entry.insert( KIO::UDSEntry::UDS_USER, fileInfo.owner() );
+    entry.insert( KIO::UDSEntry::UDS_GROUP, fileInfo.group() );
+    entry.insert( KIO::UDSEntry::UDS_ACCESS_TIME, fileInfo.lastRead().toTime_t() );
+
+    return entry;
+}
+
 void FileProtocol::copy( const KUrl &src, const KUrl &dest,
                          int _mode, JobFlags _flags )
 {
@@ -134,6 +180,7 @@ void FileProtocol::listDir( const KUrl& url )
     }
 
     QDir dir( url.toLocalFile() );
+    dir.setFilter( QDir::AllEntries|QDir::Hidden );
 
     if ( !dir.exists() ) {
         kDebug(7101) << "========= ERR_DOES_NOT_EXIST  =========";
@@ -150,46 +197,7 @@ void FileProtocol::listDir( const KUrl& url )
     UDSEntry entry;
     while( it.hasNext() ) {
         it.next();
-        QFileInfo fileInfo = it.fileInfo();
-
-        entry.insert( KIO::UDSEntry::UDS_NAME, it.fileName() );
-        if( fileInfo.isSymLink() ) {
-            entry.insert( KIO::UDSEntry::UDS_LINK_DEST, fileInfo.symLinkTarget() );
-/* TODO - or not useful on windows?
-            if ( details > 1 ) {
-                // It is a link pointing to nowhere
-                type = S_IFMT - 1;
-                access = S_IRWXU | S_IRWXG | S_IRWXO;
-
-                entry.insert( KIO::UDSEntry::UDS_FILE_TYPE, type );
-                entry.insert( KIO::UDSEntry::UDS_ACCESS, access );
-                entry.insert( KIO::UDSEntry::UDS_SIZE, 0LL );
-                goto notype;
-
-            }
-*/
-        }
-        int type = S_IFREG;
-        int access = 0;
-        if( fileInfo.isDir() )
-            type = S_IFDIR;
-        else if( fileInfo.isSymLink() )
-            type = S_IFLNK;
-        if( fileInfo.isReadable() )
-            access |= S_IRUSR;
-        if( fileInfo.isWritable() )
-            access |= S_IWUSR;
-        if( fileInfo.isExecutable() )
-            access |= S_IXUSR;
-
-        entry.insert( KIO::UDSEntry::UDS_FILE_TYPE, type );
-        entry.insert( KIO::UDSEntry::UDS_ACCESS, access );
-        entry.insert( KIO::UDSEntry::UDS_SIZE, fileInfo.size() );
-
-        entry.insert( KIO::UDSEntry::UDS_MODIFICATION_TIME, fileInfo.lastModified().toTime_t() );
-        entry.insert( KIO::UDSEntry::UDS_USER, fileInfo.owner() );
-        entry.insert( KIO::UDSEntry::UDS_GROUP, fileInfo.group() );
-        entry.insert( KIO::UDSEntry::UDS_ACCESS_TIME, fileInfo.lastRead().toTime_t() );
+        UDSEntry entry = createUDSEntryWin( it.fileInfo() );
 
         listEntry( entry, false );
         entry.clear();
@@ -232,7 +240,7 @@ void FileProtocol::rename( const KUrl &src, const KUrl &dest,
            error( KIO::ERR_FILE_ALREADY_EXIST, _dest.filePath() );
            return;
         }
-        
+
         dwFlags = MOVEFILE_REPLACE_EXISTING;
     }
 
@@ -273,7 +281,7 @@ void FileProtocol::del( const KUrl& url, bool isfile )
 
     if (isfile) {
         kDebug( 7101 ) << "Deleting file " << _path;
-        
+
         if( DeleteFileW( ( LPCWSTR ) _path.utf16() ) == 0 ) {
             DWORD dwLastErr = GetLastError();
             if ( dwLastErr == ERROR_PATH_NOT_FOUND )
@@ -289,7 +297,9 @@ void FileProtocol::del( const KUrl& url, bool isfile )
             }
         }
     } else {
-        kDebug( 7101 ) << "Deleting directory " << url.url();
+        kDebug( 7101 ) << "Deleting directory " << _path;
+        if (!deleteRecursive(_path))
+            return;
         if( RemoveDirectoryW( ( LPCWSTR ) _path.utf16() ) == 0 ) {
             DWORD dwLastErr = GetLastError();
             if ( dwLastErr == ERROR_FILE_NOT_FOUND )
@@ -307,3 +317,30 @@ void FileProtocol::del( const KUrl& url, bool isfile )
     }
     finished();
 }
+
+void FileProtocol::chown( const KUrl& url, const QString&, const QString& )
+{
+    error( KIO::ERR_CANNOT_CHOWN, url.toLocalFile() );
+}
+
+void FileProtocol::stat( const KUrl & url )
+{
+    if (!url.isLocalFile()) {
+        KUrl redir(url);
+        redir.setProtocol(config()->readEntry("DefaultRemoteProtocol", "smb"));
+        redirection(redir);
+        kDebug(7101) << "redirecting to " << redir.url();
+        finished();
+        return;
+    }
+
+    const QString sDetails = metaData(QLatin1String("details"));
+    int details = sDetails.isEmpty() ? 2 : sDetails.toInt();
+    kDebug(7101) << "FileProtocol::stat details=" << details;
+
+    UDSEntry entry = createUDSEntryWin( QFileInfo(url.toLocalFile()) );
+
+    statEntry( entry );
+
+    finished();
+}
