@@ -4,6 +4,7 @@
  *  Copyright (C) 1999-2001 Harri Porten (porten@kde.org)
  *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
  *  Copyright (C) 2003 Apple Computer, Inc.
+ *  Copyright (C) 2008, 2009 Maksim Orlovich (maksim@kde.org)
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -29,7 +30,6 @@
 #include "array_object.h"
 #include "bool_object.h"
 #include "collector.h"
-#include "context.h"
 #include "date_object.h"
 #include "debugger.h"
 #include "error_object.h"
@@ -51,7 +51,7 @@
 #include "runtime.h"
 #endif
 
-#if defined _WIN32 || defined _WIN64 
+#if defined _WIN32 || defined _WIN64
 #undef HAVE_SYS_TIME_H // no setitimer in kdewin32
 #endif
 #if HAVE(SYS_TIME_H)
@@ -79,10 +79,10 @@ private:
 #if HAVE(SYS_TIME_H)
     static Interpreter* s_executingInterpreter;
     static void alarmHandler(int);
-    
+
     Interpreter* m_oldInterpreter;
     itimerval m_oldtv;
-    itimerval m_pausetv;    
+    itimerval m_pausetv;
     void (*m_oldAlarmHandler)(int);
 #endif
 };
@@ -92,28 +92,28 @@ Interpreter* TimeoutChecker::s_executingInterpreter = 0;
 #endif
 
 void TimeoutChecker::startTimeoutCheck(Interpreter *interpreter)
-{    
+{
     if (!interpreter->m_timeoutTime)
         return;
-    
+
     interpreter->m_startTimeoutCheckCount++;
-    
+
 #if HAVE(SYS_TIME_H)
     if (s_executingInterpreter == interpreter)
         return;
-    
+
     // Block signals
     m_oldAlarmHandler = signal(SIGALRM, SIG_IGN);
 
     m_oldInterpreter = s_executingInterpreter;
     s_executingInterpreter = interpreter;
-    
+
     itimerval tv = {
       { interpreter->m_timeoutTime / 1000, (interpreter->m_timeoutTime % 1000) * 1000 },
       { interpreter->m_timeoutTime / 1000, (interpreter->m_timeoutTime % 1000) * 1000 }
     };
     setitimer(ITIMER_REAL, &tv, &m_oldtv);
-    
+
     // Unblock signals
     signal(SIGALRM, alarmHandler);
 #endif
@@ -125,24 +125,24 @@ void TimeoutChecker::stopTimeoutCheck(Interpreter* interpreter)
         return;
 
     ASSERT(interpreter->m_startTimeoutCheckCount > 0);
-    
+
     interpreter->m_startTimeoutCheckCount--;
-    
+
     if (interpreter->m_startTimeoutCheckCount != 0)
         return;
-        
+
 #if HAVE(SYS_TIME_H)
     signal(SIGALRM, SIG_IGN);
 
     s_executingInterpreter = m_oldInterpreter;
-        
+
     setitimer(ITIMER_REAL, &m_oldtv, 0L);
     signal(SIGALRM, m_oldAlarmHandler);
-#endif    
+#endif
 }
 
 #if HAVE(SYS_TIME_H)
-void TimeoutChecker::alarmHandler(int) 
+void TimeoutChecker::alarmHandler(int)
 {
     s_executingInterpreter->m_timedOut = true;
 }
@@ -153,14 +153,14 @@ void TimeoutChecker::pauseTimeoutCheck(Interpreter* interpreter)
     if (interpreter->m_startTimeoutCheckCount == 0)
         return;
 
-#if HAVE(SYS_TIME_H)    
+#if HAVE(SYS_TIME_H)
     ASSERT(interpreter == s_executingInterpreter);
 
     void (*currentSignalHandler)(int);
-   
+
     // Block signal
     currentSignalHandler = signal(SIGALRM, SIG_IGN);
-    
+
     if (currentSignalHandler != alarmHandler) {
         signal(SIGALRM, currentSignalHandler);
         return;
@@ -188,10 +188,10 @@ void TimeoutChecker::resumeTimeoutCheck(Interpreter* interpreter)
 
 #if HAVE(SYS_TIME_H)
     void (*currentSignalHandler)(int);
-   
+
     // Check so we have the right handler
     currentSignalHandler = signal(SIGALRM, SIG_IGN);
-    
+
     if (currentSignalHandler != SIG_IGN) {
         signal(SIGALRM, currentSignalHandler);
         return;
@@ -200,30 +200,24 @@ void TimeoutChecker::resumeTimeoutCheck(Interpreter* interpreter)
     setitimer(ITIMER_REAL, &m_pausetv, 0L);
 
     // Unblock signal
-    currentSignalHandler = signal(SIGALRM, alarmHandler);    
+    currentSignalHandler = signal(SIGALRM, alarmHandler);
 #endif
 }
 
 Interpreter* Interpreter::s_hook = 0;
-    
-typedef HashMap<JSObject*, Interpreter*> InterpreterMap;
-static inline InterpreterMap &interpreterMap()
-{
-    static InterpreterMap* map = new InterpreterMap;
-    return* map;
-}
-    
-Interpreter::Interpreter(JSObject* globalObject)
-    : m_globalExec(this, 0),
-      m_globalObject(globalObject),
+
+
+Interpreter::Interpreter(JSGlobalObject* globalObject)
+    : m_globalObject(globalObject),
+      m_globalExec(this, globalObject),
       globPkg(0)
 {
     init();
 }
 
 Interpreter::Interpreter()
-    : m_globalExec(this, 0),
-      m_globalObject(new JSObject()),
+    : m_globalObject(new JSGlobalObject()),
+      m_globalExec(this, m_globalObject),
       globPkg(0)
 {
     init();
@@ -233,18 +227,27 @@ void Interpreter::init()
 {
     JSLock lock;
 
+    initInternedStringsTable();
+
     m_refCount = 0;
     m_timeoutTime = 0;
     m_recursion = 0;
     m_debugger= 0;
-    m_context = 0;
+    m_execState = 0;
     m_timedOut = false;
     m_timeoutChecker = 0;
     m_startTimeoutCheckCount = 0;
     m_pauseTimeoutCheckCount = 0;
     m_compatMode = NativeMode;
 
-    interpreterMap().set(m_globalObject, this);
+    const int initialStackSize = 8192;
+    stackBase = (unsigned char*)std::malloc(initialStackSize);
+    stackPtr  = stackBase;
+    stackEnd  = stackBase + initialStackSize;
+
+    m_numCachedActivations = 0;
+
+    m_globalObject->setInterpreter(this);
 
     if (s_hook) {
         prev = s_hook;
@@ -255,22 +258,24 @@ void Interpreter::init()
         // This is the first interpreter
         s_hook = next = prev = this;
     }
-    
+
     initGlobalObject();
 }
 
 Interpreter::~Interpreter()
 {
     JSLock lock;
-    
+
     ASSERT (m_startTimeoutCheckCount == 0);
     ASSERT (m_pauseTimeoutCheckCount == 0);
-    
+
     delete m_timeoutChecker;
-    
+
     if (m_debugger)
         m_debugger->detach(this);
 
+    std::free(stackBase);
+
     next->prev = prev;
     prev->next = next;
     s_hook = next;
@@ -278,11 +283,62 @@ Interpreter::~Interpreter()
         // This was the last interpreter
         s_hook = 0;
     }
-    
-    interpreterMap().remove(m_globalObject);
+
+    m_globalObject->setInterpreter(0);
 }
 
-JSObject* Interpreter::globalObject() const
+unsigned char* Interpreter::extendStack(size_t needed)
+{
+    unsigned char* oldBase = stackBase; // needed for fixing up localStores
+
+    size_t curSize = stackEnd - stackBase;
+    size_t avail = stackEnd - stackPtr;
+    size_t extra = needed - avail;
+
+    if (extra < 8192)
+        extra = 8192;
+    size_t newSize = curSize + extra;
+
+    //printf("Grow stack:%d -> %d\n", curSize, newSize);
+
+    stackBase = (unsigned char*)std::malloc(newSize); // Not realloc since we need the old stuff
+                                                      // ### seems optimizeable
+    std::memcpy(stackBase, oldBase, curSize);
+    stackPtr  = stackBase + (stackPtr - oldBase);
+    stackEnd  = stackBase + newSize;
+
+    // Now go through and fix up activations..
+    ExecState* e = m_execState;
+    while (e) {
+        if (e->codeType() == FunctionCode) {
+            ActivationImp* act = static_cast<ActivationImp*>(e->activationObject());
+            if (act->localStorage) {
+                act->localStorage = (LocalStorageEntry*)
+                                      (stackBase + ((unsigned char*)act->localStorage - oldBase));
+                e->updateLocalStorage(act->localStorage);
+            }
+        }
+
+	e = e->savedExecState();
+    }
+
+    std::free(oldBase);
+
+    return stackAlloc(needed);
+}
+
+void Interpreter::recycleActivation(ActivationImp* act)
+{
+    ASSERT(act->localStorage == 0); // Should not refer to anything by now
+    if (m_numCachedActivations >= MaxCachedActivations)
+        return;
+
+    act->clearProperties();
+    m_cachedActivations[m_numCachedActivations] = act;
+    ++m_numCachedActivations;
+}
+
+JSGlobalObject* Interpreter::globalObject() const
 {
   return m_globalObject;
 }
@@ -294,7 +350,7 @@ void Interpreter::initGlobalObject()
     ObjectPrototype *objProto = new ObjectPrototype(&m_globalExec, funcProto);
     m_ObjectPrototype = objProto;
     funcProto->setPrototype(m_ObjectPrototype);
-    
+
     ArrayPrototype *arrayProto = new ArrayPrototype(&m_globalExec, objProto);
     m_ArrayPrototype = arrayProto;
     StringPrototype *stringProto = new StringPrototype(&m_globalExec, objProto);
@@ -309,12 +365,12 @@ void Interpreter::initGlobalObject()
     m_RegExpPrototype = regexpProto;
     ErrorPrototype *errorProto = new ErrorPrototype(&m_globalExec, objProto, funcProto);
     m_ErrorPrototype = errorProto;
-    
+
     JSObject* o = m_globalObject;
     while (o->prototype()->isObject())
         o = static_cast<JSObject*>(o->prototype());
     o->setPrototype(m_ObjectPrototype);
-    
+
     // Constructors (Object, Array, etc.)
     m_Object = new ObjectObjectImp(&m_globalExec, objProto, funcProto);
     m_Function = new FunctionObjectImp(&m_globalExec, funcProto);
@@ -325,7 +381,7 @@ void Interpreter::initGlobalObject()
     m_Date = new DateObjectImp(&m_globalExec, funcProto, dateProto);
     m_RegExp = new RegExpObjectImp(&m_globalExec, funcProto, regexpProto);
     m_Error = new ErrorObjectImp(&m_globalExec, funcProto, errorProto);
-    
+
     // Error object prototypes
     m_EvalErrorPrototype = new NativeErrorPrototype(&m_globalExec, errorProto, EvalError, "EvalError", "EvalError");
     m_RangeErrorPrototype = new NativeErrorPrototype(&m_globalExec, errorProto, RangeError, "RangeError", "RangeError");
@@ -333,7 +389,7 @@ void Interpreter::initGlobalObject()
     m_SyntaxErrorPrototype = new NativeErrorPrototype(&m_globalExec, errorProto, SyntaxError, "SyntaxError", "SyntaxError");
     m_TypeErrorPrototype = new NativeErrorPrototype(&m_globalExec, errorProto, TypeError, "TypeError", "TypeError");
     m_UriErrorPrototype = new NativeErrorPrototype(&m_globalExec, errorProto, URIError, "URIError", "URIError");
-    
+
     // Error objects
     m_EvalError = new NativeErrorImp(&m_globalExec, funcProto, m_EvalErrorPrototype);
     m_RangeError = new NativeErrorImp(&m_globalExec, funcProto, m_RangeErrorPrototype);
@@ -341,10 +397,10 @@ void Interpreter::initGlobalObject()
     m_SyntaxError = new NativeErrorImp(&m_globalExec, funcProto, m_SyntaxErrorPrototype);
     m_TypeError = new NativeErrorImp(&m_globalExec, funcProto, m_TypeErrorPrototype);
     m_UriError = new NativeErrorImp(&m_globalExec, funcProto, m_UriErrorPrototype);
-    
+
     // ECMA 15.3.4.1
     funcProto->put(&m_globalExec, m_globalExec.propertyNames().constructor, m_Function, DontEnum);
-    
+
     m_globalObject->put(&m_globalExec, "Object", m_Object, DontEnum);
     m_globalObject->put(&m_globalExec, "Function", m_Function, DontEnum);
     m_globalObject->put(&m_globalExec, "Array", m_Array, DontEnum);
@@ -377,12 +433,12 @@ void Interpreter::initGlobalObject()
     m_SyntaxErrorPrototype->put(&m_globalExec, m_globalExec.propertyNames().constructor, m_SyntaxError, DontEnum | DontDelete | ReadOnly);
     m_TypeErrorPrototype->put(&m_globalExec, m_globalExec.propertyNames().constructor, m_TypeError, DontEnum | DontDelete | ReadOnly);
     m_UriErrorPrototype->put(&m_globalExec, m_globalExec.propertyNames().constructor, m_UriError, DontEnum | DontDelete | ReadOnly);
-    
+
     // built-in values
     m_globalObject->put(&m_globalExec, "NaN",        jsNaN(), DontEnum|DontDelete);
     m_globalObject->put(&m_globalExec, "Infinity",   jsNumber(Inf), DontEnum|DontDelete);
     m_globalObject->put(&m_globalExec, "undefined",  jsUndefined(), DontEnum|DontDelete);
-    
+
     // built-in functions
     m_globalObject->putDirectFunction(new GlobalFuncImp(&m_globalExec, funcProto, GlobalFuncImp::Eval, 1, "eval"), DontEnum);
     m_globalObject->putDirectFunction(new GlobalFuncImp(&m_globalExec, funcProto, GlobalFuncImp::ParseInt, 2, "parseInt"), DontEnum);
@@ -398,7 +454,7 @@ void Interpreter::initGlobalObject()
 #ifndef NDEBUG
     m_globalObject->putDirectFunction(new GlobalFuncImp(&m_globalExec, funcProto, GlobalFuncImp::KJSPrint, 1, "kjsprint"), DontEnum);
 #endif
-    
+
     // built-in objects
     m_globalObject->put(&m_globalExec, "Math", new MathObjectImp(&m_globalExec, objProto), DontEnum);
 }
@@ -443,51 +499,52 @@ Completion Interpreter::evaluate(const UString& sourceURL, int startingLineNumbe
 Completion Interpreter::evaluate(const UString& sourceURL, int startingLineNumber, const UChar* code, int codeLength, JSValue* thisV)
 {
     JSLock lock;
-    
+
     // prevent against infinite recursion
     if (m_recursion >= 20)
         return Completion(Throw, Error::create(&m_globalExec, GeneralError, "Recursion too deep"));
-    
+
     // parse the source code
     int sourceId;
     int errLine;
     UString errMsg;
     RefPtr<ProgramNode> progNode = parser().parseProgram(sourceURL, startingLineNumber, code, codeLength, &sourceId, &errLine, &errMsg);
-    
+
     // notify debugger that source has been parsed
     if (m_debugger) {
-        bool cont = m_debugger->sourceParsed(&m_globalExec, sourceId, sourceURL, UString(code, codeLength), startingLineNumber, errLine, errMsg);
-        if (!cont)
-            return Completion(Break);
+        m_debugger->reportSourceParsed(&m_globalExec, progNode.get(), sourceId, sourceURL, 
+                      UString(code, codeLength), startingLineNumber, errLine, errMsg);
     }
-    
+
     // no program node means a syntax error occurred
     if (!progNode) {
         Completion res(Throw, Error::create(&m_globalExec, SyntaxError, errMsg, errLine, sourceId, sourceURL));
-	if (shouldPrintExceptions())
-	  printException(res, sourceURL);
-	return res;
+        if (m_debugger)
+            m_debugger->reportException(&m_globalExec, res.value());
+
+        if (shouldPrintExceptions())
+            printException(res, sourceURL);
+        return res;
     }
-    
+
     m_globalExec.clearException();
-    
+
     m_recursion++;
-    
-    JSObject* globalObj = m_globalObject;
-    JSObject* thisObj = globalObj;
-    
+
+    JSGlobalObject* globalObj = m_globalObject;
+    JSObject*       thisObj   = globalObj;
+
     // "this" must be an object... use same rules as Function.prototype.apply()
     if (thisV && !thisV->isUndefinedOrNull())
         thisObj = thisV->toObject(&m_globalExec);
-    
+
     Completion res;
     if (m_globalExec.hadException())
         // the thisV->toObject() conversion above might have thrown an exception - if so, propagate it
         res = Completion(Throw, m_globalExec.exception());
     else {
         // execute the code
-        Context ctx(globalObj, this, thisObj, progNode.get());
-        ExecState newExec(this, &ctx);
+        InterpreterExecState newExec(this, globalObj, thisObj, progNode.get());
 
         if (m_debugger && !m_debugger->enterContext(&newExec, sourceId, startingLineNumber, 0, List::empty())) {
             // debugger requested we stop execution.
@@ -504,15 +561,33 @@ Completion Interpreter::evaluate(const UString& sourceURL, int startingLineNumbe
             return Completion(Break);
         }
     }
-    
+
     m_recursion--;
-    
+
     if (shouldPrintExceptions() && res.complType() == Throw)
         printException(res, sourceURL);
 
     return res;
 }
 
+bool Interpreter::normalizeCode(const UString& codeIn, UString* codeOut,
+                                int* errLine, UString* errMsg)
+{
+    assert(codeOut);
+    RefPtr<ProgramNode> progNode = parser().parseProgram("", // sourceURL
+                                                         0, // line
+                                                         codeIn.data(),
+                                                         codeIn.size(),
+                                                         0, // &sourceId
+                                                         errLine, errMsg);
+    if (progNode) {
+        *codeOut = progNode->toString();
+        return true;
+    } else {
+        return false;
+    }
+}
+
 JSObject *Interpreter::builtinObject() const
 {
   return m_Object;
@@ -670,17 +745,22 @@ bool Interpreter::collect()
 
 void Interpreter::mark(bool)
 {
-    if (m_context)
-        m_context->mark();
+    if (m_execState)
+        m_execState->mark();
     if (m_globalObject && !m_globalObject->marked())
         m_globalObject->mark();
     if (m_globalExec.exception() && !m_globalExec.exception()->marked())
         m_globalExec.exception()->mark();
+
+    // Do not let cached activations survive the GC; as they have an unfortunate
+    // tendenacy to pin blocks, increasing their number and hence spreading out
+    // the objects somewhat
+    m_numCachedActivations = 0;
 }
 
-Interpreter* Interpreter::interpreterWithGlobalObject(JSObject* globalObject)
+void Interpreter::markSourceCachedObjects()
 {
-    return interpreterMap().get(globalObject);
+    markInternedStringsTable();
 }
 
 #ifdef KJS_DEBUG_MEM
@@ -732,7 +812,7 @@ void Interpreter::saveBuiltins (SavedBuiltins& builtins) const
 {
     if (!builtins._internal)
         builtins._internal = new SavedBuiltinsInternal;
-    
+
     builtins._internal->m_Object = m_Object;
     builtins._internal->m_Function = m_Function;
     builtins._internal->m_Array = m_Array;
@@ -742,7 +822,7 @@ void Interpreter::saveBuiltins (SavedBuiltins& builtins) const
     builtins._internal->m_Date = m_Date;
     builtins._internal->m_RegExp = m_RegExp;
     builtins._internal->m_Error = m_Error;
-    
+
     builtins._internal->m_ObjectPrototype = m_ObjectPrototype;
     builtins._internal->m_FunctionPrototype = m_FunctionPrototype;
     builtins._internal->m_ArrayPrototype = m_ArrayPrototype;
@@ -752,14 +832,14 @@ void Interpreter::saveBuiltins (SavedBuiltins& builtins) const
     builtins._internal->m_DatePrototype = m_DatePrototype;
     builtins._internal->m_RegExpPrototype = m_RegExpPrototype;
     builtins._internal->m_ErrorPrototype = m_ErrorPrototype;
-    
+
     builtins._internal->m_EvalError = m_EvalError;
     builtins._internal->m_RangeError = m_RangeError;
     builtins._internal->m_ReferenceError = m_ReferenceError;
     builtins._internal->m_SyntaxError = m_SyntaxError;
     builtins._internal->m_TypeError = m_TypeError;
     builtins._internal->m_UriError = m_UriError;
-    
+
     builtins._internal->m_EvalErrorPrototype = m_EvalErrorPrototype;
     builtins._internal->m_RangeErrorPrototype = m_RangeErrorPrototype;
     builtins._internal->m_ReferenceErrorPrototype = m_ReferenceErrorPrototype;
@@ -782,7 +862,7 @@ void Interpreter::restoreBuiltins (const SavedBuiltins& builtins)
     m_Date = builtins._internal->m_Date;
     m_RegExp = builtins._internal->m_RegExp;
     m_Error = builtins._internal->m_Error;
-    
+
     m_ObjectPrototype = builtins._internal->m_ObjectPrototype;
     m_FunctionPrototype = builtins._internal->m_FunctionPrototype;
     m_ArrayPrototype = builtins._internal->m_ArrayPrototype;
@@ -792,14 +872,14 @@ void Interpreter::restoreBuiltins (const SavedBuiltins& builtins)
     m_DatePrototype = builtins._internal->m_DatePrototype;
     m_RegExpPrototype = builtins._internal->m_RegExpPrototype;
     m_ErrorPrototype = builtins._internal->m_ErrorPrototype;
-    
+
     m_EvalError = builtins._internal->m_EvalError;
     m_RangeError = builtins._internal->m_RangeError;
     m_ReferenceError = builtins._internal->m_ReferenceError;
     m_SyntaxError = builtins._internal->m_SyntaxError;
     m_TypeError = builtins._internal->m_TypeError;
     m_UriError = builtins._internal->m_UriError;
-    
+
     m_EvalErrorPrototype = builtins._internal->m_EvalErrorPrototype;
     m_RangeErrorPrototype = builtins._internal->m_RangeErrorPrototype;
     m_ReferenceErrorPrototype = builtins._internal->m_ReferenceErrorPrototype;
@@ -812,14 +892,14 @@ void Interpreter::startTimeoutCheck()
 {
     if (!m_timeoutChecker)
         m_timeoutChecker = new TimeoutChecker;
-    
+
     m_timeoutChecker->startTimeoutCheck(this);
 }
 
 void Interpreter::stopTimeoutCheck()
 {
     ASSERT(m_timeoutChecker);
-    
+
     m_timeoutChecker->stopTimeoutCheck(this);
 }
 
@@ -836,7 +916,7 @@ void Interpreter::restartTimeoutCheck()
 void Interpreter::pauseTimeoutCheck()
 {
     ASSERT(m_timeoutChecker);
-    
+
     m_timeoutChecker->pauseTimeoutCheck(this);
 }
 
@@ -854,12 +934,52 @@ bool Interpreter::handleTimeout()
     pauseTimeoutCheck();
     bool retval = shouldInterruptScript();
     resumeTimeoutCheck();
-    
+
     return retval;
 }
 
+Interpreter::InternedStringsTable* Interpreter::s_internedStrings;
 
-SavedBuiltins::SavedBuiltins() : 
+void Interpreter::initInternedStringsTable()
+{
+    if (!s_internedStrings)
+        s_internedStrings = new InternedStringsTable();
+}
+
+StringImp* Interpreter::internString(const UString& literal)
+{
+    std::pair<InternedStringsTable::iterator, bool> p = 
+        s_internedStrings->add(literal.rep(), std::make_pair((StringImp*)(0), 1));
+        
+    if (p.second) // actually added..
+        p.first.values()->first = static_cast<StringImp*>(jsOwnedString(literal));
+    else
+        ++p.first.values()->second; // just bump the ref count
+
+    return p.first.values()->first;
+}
+
+void Interpreter::releaseInternedString(const UString& literal)
+{
+    InternedStringsTable::iterator i = s_internedStrings->find(literal.rep());
+
+    --i.values()->second;
+    if (i.values()->second == 0)
+        s_internedStrings->remove(i);
+}
+    
+void Interpreter::markInternedStringsTable()
+{
+    for (InternedStringsTable::iterator i = s_internedStrings->begin();
+        i != s_internedStrings->end(); ++i) {
+        // Note: the StringImp* may be null here if we got called in the middle 
+        // of internString.
+        if (i.values()->first && !i.values()->first->marked())
+            i.values()->first->mark();
+    }
+}
+    
+SavedBuiltins::SavedBuiltins() :
   _internal(0)
 {
 }
@@ -894,3 +1014,5 @@ void UnicodeSupport::setToUpperFunction(StringConversionFunction f)
 }
 
 }
+
+// kate: indent-width 2; replace-tabs on; tab-width 4; space-indent on;
