@@ -29,13 +29,11 @@
 #include <QtGui/QLabel>
 #include <QtGui/QLayout>
 #include <QtCore/QPoint>
-#include <QtCore/QTextStream>
 
 #include <QtGui/QImage>
 
 #include <QtDBus/QtDBus>
 
-#include <kapplication.h>
 #include <kaboutdata.h>
 #include <kcmodule.h>
 #include <kcmoduleinfo.h>
@@ -92,7 +90,6 @@ void KCModuleProxyPrivate::loadModule()
 	{
 		topLayout = new QVBoxLayout( parent );
 		topLayout->setMargin( 0 );
-		topLayout->setSpacing( 0 );
 
 		QString name = modInfo.handle();
 		dbusPath = QLatin1String("/internal/KSettingsWidget/") + name;
@@ -216,7 +213,7 @@ void KCModuleProxy::deleteClient()
 	delete d->kcm;
 	d->kcm = 0;
 
-	kapp->syncX();
+	qApp->syncX();
 }
 
 void KCModuleProxyPrivate::_k_moduleChanged(bool c)
@@ -228,7 +225,7 @@ void KCModuleProxyPrivate::_k_moduleChanged(bool c)
     Q_Q(KCModuleProxy);
     changed = c;
     emit q->changed(c);
-    emit q->changed(this);
+    emit q->changed(q);
 }
 
 void KCModuleProxyPrivate::_k_moduleDestroyed()
@@ -343,7 +340,6 @@ QString KCModuleProxy::dbusPath() const
 
 QSize KCModuleProxy::minimumSizeHint() const
 {
-	realModule();
 	return QWidget::minimumSizeHint();
 }
 
